PROMPT Lanzar Paquete PAQUETE_CARGAR_DATOS

create or replace PACKAGE PAQUETE_CARGAR_DATOS AS 

-- Mtodo que recibir el XML de Solicitud y varios datos del Expediente (COD_PROCEDURE, FOLDER_ID y FECHA de ENTRADA), generar el Ttulo del Expediente a partir del Procedimiento que pertenece y
-- almacenar el Expediente en la base de datos (AB6102T00). El mtodo devolver el Identificador del Expediente para poder utilizarlo a la hora de almacenar el resto de la informacin relacionada con el expediente
-- y adems el ttulo del expediente.
FUNCTION ALMACENAR_TITULO_EXP (IN_xmltype_SOLICITUD IN XMLTYPE, IN_COD_PROCEDURE_ID IN AB6101T00.COD_PROCEDURE_ID%TYPE, 
                                                        IN_FOLDER_ID AB6118T00.FOLDER_ID%TYPE, IN_FECHA_ENTRADA AB6118T00.FECHA_ENTRADA%TYPE, 
                                                        OUT_TITULO IN OUT AB6102T00.TITULO%TYPE) RETURN AB6102T00.ID_EXPEDIENTE%TYPE;

-- Mtodo que recibir el XML de Solicitud y el Identificador del Expediente para obtener el/los tercero/s y la/las empresa/s solicitante/s y/o representante/s y almacenar la informacin en su tabla
-- correspondiente: AB6104T00 para los Terceros, AB6105T00 para las Empresas y AB6103T00 para guardar la relacin de estos con el Expediente. 
-- El mtodo devolver el Identificador del Tercero o Identificador de la Empresa del Solicitante y un VARCHAR2 con las TAREAS que ha realizado.
PROCEDURE ALMACENAR_TER_EMP_SOL_REP (IN_xmltype_SOLICITUD IN XMLTYPE, IN_ID_EXPEDIENTE IN AB6102T00.ID_EXPEDIENTE%TYPE, 
                                    OUT_ID_TERCERO IN OUT AB6104T00.ID_TERCERO%TYPE, OUT_ID_EMPRESA IN OUT AB6105T00.ID_EMPRESA%TYPE, OUT_TAREAS IN OUT AB6121T00.TAREAS%TYPE);

-- Mtodo que rebir el XML de Solicitud, el Identificador de Tercero y el Identificador de Empresa para almacenar el Domicilio del Titular. Obviamente si el titular es un tercero utilizar el identificador correspondiente
-- y almacenar la informacin en la tabla AB6104T00 y si el titular es una empresa en la tabla AB6105T00.
PROCEDURE ALMACENAR_DOMICILIO_TITULAR (IN_xmltype_SOLICITUD IN XMLTYPE, IN_ID_TERCERO IN OUT AB6104T00.ID_TERCERO%TYPE, IN_ID_EMPRESA IN OUT AB6105T00.ID_EMPRESA%TYPE, 
                                                         OUT_TAREAS IN OUT AB6121T00.TAREAS%TYPE);

-- Mtodo que recibir el XML de Solicitud y el Identificador del Expediente para obtener la/s Persona/s de Contacto (Terceros) y almacenar esta informacin en la tabla AB6104T00 y en la tabla AB6103T00 la relacin de estos con el Expediente.
-- El mtodo devolver un VARCHAR2 con las TAREAS que ha realizado.
PROCEDURE ALMACENAR_PERSONA_CONTACTO (IN_xmltype_SOLICITUD IN XMLTYPE, IN_ID_EXPEDIENTE IN AB6102T00.ID_EXPEDIENTE%TYPE, OUT_TAREAS IN OUT AB6121T00.TAREAS%TYPE);

-- Mtodo que recibir el XML de Solicitud y el Identificador del Expediente para obtener el/los Representante/s Legal/es (Terceros) y almacenar esta informacin en la tabla AB6104T00 y en la tabla AB6103T00 la relacin de estos con el Expediente.
-- El mtodo devolver un VARCHAR2 con las TAREAS que ha realizado.
PROCEDURE ALMACENAR_REPRESENTANTE_LEGAL(IN_xmltype_SOLICITUD IN XMLTYPE, IN_ID_EXPEDIENTE IN AB6102T00.ID_EXPEDIENTE%TYPE, OUT_TAREAS IN OUT AB6121T00.TAREAS%TYPE);

-- Mtodo que rebicir el XML de Solicitud y el Identificador de Empresa para obtener los datos de Plantilla (nmero de hombres y mujeres) y almacenar esta informacin en la tabla AB6105T00.
PROCEDURE ALMACENAR_PLANTILLA(IN_xmltype_SOLICITUD IN XMLTYPE, IN_ID_EMPRESA IN AB6105T00.ID_EMPRESA%TYPE, OUT_TAREAS IN OUT AB6121T00.TAREAS%TYPE);

-- Mtodo que recibir el Tipo de Documento (como viene en el XML de Solicitud) y devolver la Correlacin del Ctalogo de la Aplicacin (AB6107T00)
FUNCTION CORRELACION_TIPO_DOCUMENTO(IN_TIPO_DOCUMENTO IN VARCHAR2) RETURN AB6107T00.ID_TIPO_DOCUMENTO%TYPE;

-- Mtodo que rebicir la Fecha y el Idioma para obtener esta Fecha con el formato correcto
FUNCTION FORMATEAR_FECHA(IN_FECHA IN VARCHAR2, IN_IDIOMA IN VARCHAR2) RETURN DATE;

-- Mtodo que recibir el Sexo (como viene en el XML de Solicitud) y devolver la Correlacin del Ctalogo de la Aplicacin (AB6120T00)
FUNCTION CORRELACION_SEXO(IN_SEXO IN VARCHAR2) RETURN AB6120T00.ID_SEXO%TYPE;

-- Mtodo que recibir un Number y devolver este valor con el formato correcto
FUNCTION FORMATEAR_NUMBER_DECIMAL (IN_NUMBER IN VARCHAR2) RETURN NUMBER;

END PAQUETE_CARGAR_DATOS;

/