PROMPT Lanzar Procedimiento CARGAR_DATOS_BASICA

create or replace PROCEDURE CARGAR_DATOS_BASICA (IN_ID_SOLICITUD IN AB6118T00.ID_SOLICITUD%TYPE, OUT_TITULO IN OUT AB6102T00.TITULO%TYPE) AS

-- TABLAS:
-- AB6101T00: Tabla de Procedimientos
-- AB6102T00: Tabla de Expedientes
-- AB6103T00: Tabla de Relacin del Expediente con el Tercero / la Empresa
-- AB6104T00: Tabla de Terceros
-- AB6105T00: Tabla de Empresas
-- AB6106T00: Tabla de Tipos de Terceros
-- AB6107T00: Tabla de Tipos de Documentos
-- AB6115T00: Tabla de Tipos de Canal de Aviso
-- AB6116T00: Tabla de Canales de Aviso
-- AB6118T00: Tabla de Solicitudes
-- AB6120T00: Tabla de Tipos de Sexo
-- AB6121T00: Tabla de Trazas


n_COD_PROCEDURE_ID          AB6101T00.COD_PROCEDURE_ID%TYPE;                -- Cdigo de Procedimiento de PLATEA

n_ID_EXPEDIENTE             AB6102T00.ID_EXPEDIENTE%TYPE;                   -- Identificador del Expediente

n_ID_TERCERO                AB6104T00.ID_TERCERO%TYPE;                      -- Identificador del Tercero

n_ID_EMPRESA                AB6105T00.ID_EMPRESA%TYPE;                      -- Identificador de la Empresa


clob_SOLICITUD              AB6118T00.SOLICITUD%TYPE;                       -- Solicitud (XML) que llega para cargar su informacin en las tablas de la aplicacin
v_FOLDER_ID                 AB6118T00.FOLDER_ID%TYPE;                       -- Identificador del Expediente en PLATEA
f_FECHA_ENTRADA             AB6118T00.FECHA_ENTRADA%TYPE;                   -- Fecha en la que se ha presentado la Solicitud

n_ID_TRAZA                  AB6121T00.ID_TRAZA%TYPE;                        -- Identificador de la Traza
v_EXCEPCION                 AB6121T00.EXCEPCION%TYPE;                       -- Excepcin dentro de la Traza en caso de que tengamos un Error
v_TAREAS                    AB6121T00.TAREAS%TYPE;                          -- Tareas de la carga de la Solicitud que se han desarrollado
v_METODO                    Ab6121T00.METODO%TYPE;                          -- Mtodo (Funcin o Procedimiento) donde ha dado la Excepcin

xmltype_SOLICITUD XMLTYPE;          -- Variable XMLTYPE donde almacenaremos el XML de la Solicitud

BEGIN

v_METODO := 'CARGAR_DATOS_BASICA';

-- Obtenemos el XML (CLOB) de la tabla AB6118T00 a partir del Identificador del Solicitud
SELECT SOLICITUD, FOLDER_ID, FECHA_ENTRADA INTO clob_SOLICITUD, v_FOLDER_ID, f_FECHA_ENTRADA FROM AB6118T00 WHERE ID_SOLICITUD = IN_ID_SOLICITUD;
-- Convertimos el CLOB (que tiene el XML de la Solicitud) en el tipo XMLTYPE
xmltype_SOLICITUD := XMLTYPE.createXML(clob_SOLICITUD);

-- Obtenemos el Identificador del Procedimiento (en PLATEA) a partir de <body><basicInfo><procedureID>
n_COD_PROCEDURE_ID:= xmltype_SOLICITUD.EXTRACT('/ejgvDocument/body/basicInfo/procedureID/text()').getStringVal();

v_TAREAS := v_TAREAS || '-Cdigo Procedimiento (PLATEA): ' || n_COD_PROCEDURE_ID;

v_METODO := 'ALMACENAR_TITULO_EXP';
n_ID_EXPEDIENTE:= ALMACENAR_TITULO_EXP(xmltype_SOLICITUD, n_COD_PROCEDURE_ID, v_FOLDER_ID, f_FECHA_ENTRADA, OUT_TITULO);

v_TAREAS :=  v_TAREAS || chr(13) || '-Expediente Almacenado (Cdigo Expediente: ' || n_ID_EXPEDIENTE || ', Ttulo: ' || OUT_TITULO ||')';

/***** Almacenamos los Terceros y/o las Empresas SOLICITANTES y/o REPRESENTANTES *****/
v_METODO := 'ALMACENAR_TER_EMP_SOL_REP';
ALMACENAR_TER_EMP_SOL_REP (xmltype_SOLICITUD, n_ID_EXPEDIENTE, n_ID_TERCERO, n_ID_EMPRESA, v_TAREAS);



-- Una vez que hayamos almacenado toda la informacin, guardamos una Traza con las Tareas Realizadas en la tbala AB6121T00
SELECT S_AB6121T00.NEXTVAL INTO n_ID_TRAZA FROM DUAL;

-- En este caso el Estado ser igual a 1 ya que todo ha ido bien
v_METODO := 'CARGAR_DATOS_BASICA';
INSERT INTO AB61.AB6121T00 (ID_TRAZA, ID_SOLICITUD, ID_EXPEDIENTE, COD_PROCEDURE_ID, EXCEPCION, METODO, ESTADO_SOLICITUD, TAREAS, FECHA_REGISTRO) VALUES (n_ID_TRAZA, IN_ID_SOLICITUD, n_ID_EXPEDIENTE, n_COD_PROCEDURE_ID, '', v_METODO, 1, v_TAREAS, SYSDATE);

-- Haremos COMMIT cuando hayamos almacenado toda la informacin relacionada con la Solicitud
COMMIT;

EXCEPTION
WHEN NO_DATA_FOUND THEN
BEGIN
    ROLLBACK;

    SELECT S_AB6121T00.NEXTVAL INTO n_ID_TRAZA FROM DUAL;

    v_METODO := 'CARGAR_DATOS_BASICA';
    
    v_EXCEPCION:= SUBSTR(SQLERRM, 1, 2000);

    INSERT INTO AB61.AB6121T00 (ID_TRAZA, ID_SOLICITUD, ID_EXPEDIENTE, COD_PROCEDURE_ID, EXCEPCION, METODO, ESTADO_SOLICITUD, TAREAS, FECHA_REGISTRO)
    VALUES (n_ID_TRAZA, IN_ID_SOLICITUD, n_ID_EXPEDIENTE, n_COD_PROCEDURE_ID, v_EXCEPCION, v_METODO, 2, v_TAREAS, SYSDATE);

    COMMIT;

    -- Colocamos el parmetro de salida a NULL para "avisar" que ha habido un ERROR
    OUT_TITULO:= NULL;

    END;

WHEN OTHERS THEN
BEGIN
    ROLLBACK;

    SELECT S_AB6121T00.NEXTVAL INTO n_ID_TRAZA FROM DUAL;

      v_EXCEPCION:= SUBSTR(SQLERRM, 1, 2000);

    INSERT INTO AB61.AB6121T00 (ID_TRAZA, ID_SOLICITUD, ID_EXPEDIENTE, COD_PROCEDURE_ID, EXCEPCION, METODO, ESTADO_SOLICITUD, TAREAS, FECHA_REGISTRO)
    VALUES (n_ID_TRAZA, IN_ID_SOLICITUD, n_ID_EXPEDIENTE, n_COD_PROCEDURE_ID, v_EXCEPCION, v_METODO, 2, v_TAREAS, SYSDATE);

    COMMIT;

    -- Colocamos el parmetro de salida a NULL para "avisar" que ha habido un ERROR
    OUT_TITULO:= NULL;

    END;

END CARGAR_DATOS_BASICA;

/
