PROMPT Procedimiento CARGAR_DATOS_OFICIO_REGISTROS

create or replace PROCEDURE CARGAR_DATOS_OFICIO_REGISTROS (IN_ID_MENSAJE AB61.AB6117T00.ID_MENSAJE%TYPE) AS 

clob_MENSAJE                AB61.AB6117T00.MENSAJE%TYPE;                        -- Mensaje (XML)

n_ID_TRAZA                  AB61.AB6121T00.ID_TRAZA%TYPE;                       -- Identificador de la Traza
v_EXCEPCION                 AB61.AB6121T00.EXCEPCION%TYPE;                      -- Excepcin dentro de la Traza en caso de que tengamos un Error
v_METODO                    AB61.AB6121T00.METODO%TYPE;                         -- Mtodo (Funcin o Procedimiento) donde ha dado la Excepcin

n_COD_PROCEDURE_ID          AB61.AB6101T00.COD_PROCEDURE_ID%TYPE;               -- Cdigo de Procedimiento de PLATEA
n_ID_PROCEDIMIENTO          AB61.AB6101T00.ID_PROCEDIMIENTO%TYPE;

v_FOLDER_ID                 AB61.AB6102T00.FOLDER_ID%TYPE;                      -- Identificador del Expediente en PLATEA
v_TITULO                    AB61.AB6102T00.TITULO%TYPE;
n_ID_EXPEDIENTE             AB61.AB6102T00.ID_EXPEDIENTE%TYPE;                  -- Cdigo de Identificador del Expediente

n_ID_REL_EXP_TER            AB6134T00.ID_REL_EXP_TER%TYPE;
n_ID_TIPO_TERCERO           AB6134T00.ID_TIPO_TERCERO%TYPE;

n_ID_TERCERO                AB6135T00.ID_TERCERO%TYPE;                      -- Identificador del Tercero
v_DOCUMENTO                 AB6135T00.DOCUMENTO%TYPE;                       -- DNI, NIF o Pasaporte del tercero
v_NOMBRE                    AB6135T00.NOMBRE%TYPE;                          -- Nombre del Tercero
v_APELLIDO1                 AB6135T00.APELLIDO1%TYPE;                       -- Primer Apellido del Tercero
v_APELLIDO2                 AB6135T00.APELLIDO2%TYPE;                       -- Segundo Apellido del Tercero
n_TIPO_PARTICIPANTE         AB6135T00.TIPO_PARTICIPANTE%TYPE;
n_ID_TIPO_DOCUMENTO         AB6135T00.ID_TIPO_DOCUMENTO%TYPE;

xmltype_MENSAJE XMLTYPE;
xmltype_AUXILIAR XMLTYPE;

n_NumeroNodos NUMBER; 


BEGIN


-- Obtenemos el XML (CLOB) de la tabla AB6117T00 a partir del Identificador del Solicitud
SELECT MENSAJE INTO clob_MENSAJE FROM AB61.AB6117T00 WHERE ID_MENSAJE = IN_ID_MENSAJE;
-- Convertimos el CLOB (que tiene el XML de la Tarea) en el tipo XMLTYPE
xmltype_MENSAJE := XMLTYPE.createXML(clob_MENSAJE);


IF (xmltype_MENSAJE.EXISTSNODE('.//procedureID/text()') > 0) THEN
    n_COD_PROCEDURE_ID := xmltype_MENSAJE.EXTRACT('.//procedureID/text()').getStringVal();
END IF;

IF (xmltype_MENSAJE.EXISTSNODE('.//folderID/text()') > 0) THEN
    v_FOLDER_ID := xmltype_MENSAJE.EXTRACT('.//folderID/text()').getStringVal();
END IF;

IF (xmltype_MENSAJE.EXISTSNODE('.//folderDescription/text()') > 0) THEN
    v_TITULO := xmltype_MENSAJE.EXTRACT('.//folderDescription/text()').getStringVal();
END IF;


SELECT ID_PROCEDIMIENTO INTO n_ID_PROCEDIMIENTO FROM AB6101T00 WHERE COD_PROCEDURE_ID = n_COD_PROCEDURE_ID;

-- Obtenemos el Identificador del Expediente a partir de la secuencia S_AB6102T00
SELECT S_AB6102T00.NEXTVAL INTO n_ID_EXPEDIENTE FROM DUAL;

-- Almacenamos la informacin relacionada con el Expediente en la tabla AB6102T00. Aqu el FOLDER_NUMBER lo almacenamos como NULL porque no lo tenemos todava.
INSERT INTO AB6102T00 (ID_EXPEDIENTE, ID_PROCEDIMIENTO, ID_TERRITORIO_HISTORICO, ID_PROVINCIA_FISCAL, FOLDER_ID, TITULO, FECHA_REGISTRO, USUARIO_REGISTRO)
VALUES (n_ID_EXPEDIENTE,n_ID_PROCEDIMIENTO, NULL, NULL, v_FOLDER_ID, v_TITULO, SYSDATE, 'PL/SQL');


IF (xmltype_MENSAJE.EXISTSNODE('.//ApplicantsInfoEvent') >  0) THEN

    xmltype_AUXILIAR := xmltype_MENSAJE.EXTRACT('.//ApplicantsInfoEvent');

    n_NumeroNodos := 1;
    n_ID_TIPO_TERCERO := 3; -- Afectado / Kaltetua
    n_TIPO_PARTICIPANTE := 2; -- Otros interesados
    
    v_DOCUMENTO := NULL;
    v_NOMBRE := NULL;
    v_APELLIDO1 := NULL;
    v_APELLIDO2 := NULL;
    n_ID_TIPO_DOCUMENTO := NULL;
    
    WHILE xmltype_AUXILIAR.EXISTSNODE('ApplicantsInfoEvent/ApplicantInfoEvent[' || n_NumeroNodos || ']') = 1 LOOP

        v_DOCUMENTO := xmltype_AUXILIAR.EXTRACT('ApplicantsInfoEvent/ApplicantInfoEvent[' || n_NumeroNodos || ']/personsEvent/personEvent/id/text()').getStringVal();
        v_NOMBRE := xmltype_AUXILIAR.EXTRACT('ApplicantsInfoEvent/ApplicantInfoEvent[' || n_NumeroNodos || ']/personsEvent/personEvent/name/text()').getStringVal();
        -- Tenemos que mirar su existe el Primer Apellido ya que si el Tipo de Documento es igual a 5 (Otros) no obtenemos este campo
        IF (xmltype_AUXILIAR.EXISTSNODE('ApplicantsInfoEvent/ApplicantInfoEvent[' || n_NumeroNodos || ']/personsEvent/personEvent/lastname1/text()') > 0) THEN
            v_APELLIDO1 := xmltype_AUXILIAR.EXTRACT('ApplicantsInfoEvent/ApplicantInfoEvent[' || n_NumeroNodos || ']/personsEvent/personEvent/lastname1/text()').getStringVal();
        END IF;
        -- Tenemos que mirar su existe el Segundo Apellido ya que si el Tipo de Documento es igual a 5 (Otros) no obtenemos este campo
        IF (xmltype_AUXILIAR.EXISTSNODE('ApplicantsInfoEvent/ApplicantInfoEvent[' || n_NumeroNodos || ']/personsEvent/personEvent/lastname2/text()') > 0) THEN
            v_APELLIDO2 := xmltype_AUXILIAR.EXTRACT('ApplicantsInfoEvent/ApplicantInfoEvent[' || n_NumeroNodos || ']/personsEvent/personEvent/lastname2/text()').getStringVal();
        END IF;
        n_ID_TIPO_DOCUMENTO := xmltype_AUXILIAR.EXTRACT('ApplicantsInfoEvent/ApplicantInfoEvent[' || n_NumeroNodos || ']/personsEvent/personEvent/personType/text()').getStringVal();

        -- Obtenemos el Identificador del Tercero a partir de la secuencia S_AB6135T00
        SELECT S_AB6135T00.NEXTVAL INTO n_ID_TERCERO FROM DUAL;

        -- Almacenamos la informacin relacionada con el Tercero en la tabla AB6135T00
        INSERT INTO AB6135T00 (ID_TERCERO, DOCUMENTO, ID_TIPO_DOCUMENTO, NOMBRE, APELLIDO1, APELLIDO2, ID_SEXO, TIPO_PARTICIPANTE, FECHA_REGISTRO, USUARIO_REGISTRO)
        VALUES(n_ID_TERCERO, v_DOCUMENTO, n_ID_TIPO_DOCUMENTO, v_NOMBRE, v_APELLIDO1, v_APELLIDO2, NULL, n_TIPO_PARTICIPANTE, SYSDATE, 'PL/SQL');

        -- Obtenemos el Identificador de la Relacin del Expediente con el Tercero / la Empresa a partir de la secuencia S_AB6103T00
        SELECT S_AB6134T00.NEXTVAL INTO n_ID_REL_EXP_TER FROM DUAL;

        -- Almacenamos la informacin relacionada con la Relacin del Expediente con el Tercero / la Empresa en la tabla AB6103T00
        INSERT INTO AB6134T00 (ID_REL_EXP_TER, ID_TERCERO, ID_TIPO_TERCERO, ID_EXPEDIENTE, PARTICIPANT_ID, FECHA_REGISTRO, USUARIO_REGISTRO)
        VALUES (n_ID_REL_EXP_TER, n_ID_TERCERO, n_ID_TIPO_TERCERO, n_ID_EXPEDIENTE, 0, SYSDATE, 'PL/SQL');

        n_NumeroNodos :=  n_NumeroNodos + 1;

    END LOOP;

END IF;

SELECT AB61.S_AB6121T00.NEXTVAL INTO n_ID_TRAZA FROM DUAL;

-- En este caso el Estado ser igual a 1 ya que todo ha ido bien
v_METODO := 'CARGAR_DATOS_OFICIO_CESE';

INSERT INTO AB61.AB6121T00 (ID_TRAZA, ID_MENSAJE, ID_EXPEDIENTE, COD_PROCEDURE_ID, EXCEPCION, METODO, ESTADO_SOLICITUD, TAREAS, FECHA_REGISTRO)
VALUES (n_ID_TRAZA, IN_ID_MENSAJE, n_ID_EXPEDIENTE, n_COD_PROCEDURE_ID,v_EXCEPCION , v_METODO, 1, null, SYSDATE);

COMMIT;

EXCEPTION
WHEN NO_DATA_FOUND THEN
BEGIN
    ROLLBACK;

    SELECT AB61.S_AB6121T00.NEXTVAL INTO n_ID_TRAZA FROM DUAL;

     v_EXCEPCION:= SUBSTR(SQLERRM, 1, 2000) || ' (ID_MENSAJE: ' || IN_ID_MENSAJE || ')';

    v_METODO:= 'CARGAR_DATOS_OFICIO_REGISTROS';

    INSERT INTO AB61.AB6121T00 (ID_TRAZA, ID_MENSAJE, ID_EXPEDIENTE, COD_PROCEDURE_ID, EXCEPCION, METODO, ESTADO_SOLICITUD, TAREAS, FECHA_REGISTRO)
    VALUES (n_ID_TRAZA, IN_ID_MENSAJE, n_ID_EXPEDIENTE, n_COD_PROCEDURE_ID, v_EXCEPCION, v_METODO, 2, null, SYSDATE);

    COMMIT;

END;

WHEN OTHERS THEN
BEGIN
    ROLLBACK;

    SELECT AB61.S_AB6121T00.NEXTVAL INTO n_ID_TRAZA FROM DUAL;

    v_EXCEPCION:= SUBSTR(SQLERRM, 1, 2000);

    v_METODO:= 'CARGAR_DATOS_OFICIO_CESE';

    INSERT INTO AB61.AB6121T00 (ID_TRAZA, ID_MENSAJE, ID_EXPEDIENTE, COD_PROCEDURE_ID, EXCEPCION, METODO, ESTADO_SOLICITUD, TAREAS, FECHA_REGISTRO)
    VALUES (n_ID_TRAZA, IN_ID_MENSAJE, NULL, n_COD_PROCEDURE_ID, v_EXCEPCION, v_METODO, 2, null, SYSDATE);

    COMMIT;

END;

END CARGAR_DATOS_OFICIO_REGISTROS;

/