PROMPT Procedimiento (CARGAR_COMUNICACIONES_FECHAS)

create or replace PROCEDURE CARGAR_COMUNICACIONES_FECHAS (IN_ID_PROCEDIMIENTO IN AB6102T00.ID_PROCEDIMIENTO%TYPE) AS

-- IN_ID_PROCEDIMIENTO=15,16,17

n_ID_EXPEDIENTE             AB6102T00.ID_EXPEDIENTE%TYPE;                   -- Identificador del Expediente
n_ID_TERCERO                AB6135T00.ID_TERCERO%TYPE;                      -- Identificador del Tercero
clob_SOLICITUD              AB6118T00.SOLICITUD%TYPE;                       -- Solicitud (XML) que llega para cargar su información en las tablas de la aplicación
n_ID_SOLICITUD              AB6118T00.ID_SOLICITUD%TYPE;
n_COD_PROCEDURE_ID          AB6118T00.COD_PROCEDURE_ID%TYPE;

n_ID_TRAZA                  AB6121T00.ID_TRAZA%TYPE;                        -- Identificador de la Traza
v_EXCEPCION                 AB6121T00.EXCEPCION%TYPE;                       -- Excepción dentro de la Traza en caso de que tengamos un Error
v_TAREAS                    AB6121T00.TAREAS%TYPE;                          -- Tareas de la carga de la Solicitud que se han desarrollado
v_METODO                    Ab6121T00.METODO%TYPE;                          -- Método (Función o Procedimiento) donde ha dado la Excepción

xmltype_SOLICITUD XMLTYPE;          -- Variable XMLTYPE donde almacenaremos el XML de la Solicitud

-- Obtenemos el XML (CLOB) de la tabla AB6118T00 y el ID_TERCERO del titular de todos los expedientes de EMCO 2018
CURSOR cSOLICITUDES IS
SELECT TTERCERO.ID_EXPEDIENTE, T18.SOLICITUD, T18.ID_SOLICITUD , TTERCERO.ID_TERCERO, T18.COD_PROCEDURE_ID
FROM AB6118T00 T18, AB6145T00 T45,
(SELECT T35.ID_TERCERO, T02.ID_EXPEDIENTE, T02.FOLDER_ID FROM AB6135T00 T35, AB6134T00 T34, 
(SELECT ID_EXPEDIENTE, FOLDER_ID FROM AB6102T00 WHERE ID_PROCEDIMIENTO=IN_ID_PROCEDIMIENTO) T02 
WHERE T35.TIPO_PARTICIPANTE=0 AND T35.ID_TERCERO=T34.ID_TERCERO AND T34.ID_TIPO_TERCERO=1 AND T34.ID_EXPEDIENTE=T02.ID_EXPEDIENTE) TTERCERO
WHERE TTERCERO.FOLDER_ID= T18.FOLDER_ID (+) AND TTERCERO.ID_EXPEDIENTE=T45.ID_EXPEDIENTE (+) AND T45.FECHA_INICIO IS NULL
ORDER BY TTERCERO.ID_EXPEDIENTE;

BEGIN

  v_METODO := 'CARGAR_COMUNICACIONES_FECHAS';

  FOR sol IN cSOLICITUDES
  LOOP

    -- Convertimos el CLOB (que tiene el XML de la Solicitud) en el tipo XMLTYPE
    xmltype_SOLICITUD := XMLTYPE.createXML(sol.SOLICITUD);
    n_ID_TERCERO := sol.ID_TERCERO;
    n_ID_EXPEDIENTE := sol.ID_EXPEDIENTE;
    n_ID_SOLICITUD := sol.ID_SOLICITUD;
    n_COD_PROCEDURE_ID := sol.COD_PROCEDURE_ID;

    /***** Almacenamos la ubicacion del establecimiento *****/
    v_METODO := 'ALMACENAR_FECHAS_COMUNICACION';
    ALMACENAR_FECHAS_COMUNICACION (xmltype_SOLICITUD, n_ID_EXPEDIENTE, v_TAREAS);

    -- Una vez que hayamos almacenado toda la información, guardamos una Traza con las Tareas Realizadas en la tbala AB6121T00
    SELECT S_AB6121T00.NEXTVAL INTO n_ID_TRAZA FROM DUAL;

    -- En este caso el Estado será igual a 1 ya que todo ha ido bien
    v_METODO := 'CARGAR_COMUNICACIONES_FECHAS';
    INSERT INTO AB61.AB6121T00 (ID_TRAZA, ID_SOLICITUD, ID_EXPEDIENTE, COD_PROCEDURE_ID, EXCEPCION, METODO, ESTADO_SOLICITUD, TAREAS, FECHA_REGISTRO) VALUES
    (n_ID_TRAZA, n_ID_SOLICITUD, n_ID_EXPEDIENTE, n_COD_PROCEDURE_ID, '', v_METODO, 1, v_TAREAS, SYSDATE);

  END LOOP;

-- Haremos COMMIT cuando hayamos almacenado toda la información relacionada con la Solicitud
COMMIT;

EXCEPTION
WHEN NO_DATA_FOUND THEN
BEGIN
    ROLLBACK;

    SELECT S_AB6121T00.NEXTVAL INTO n_ID_TRAZA FROM DUAL;

    v_METODO := 'CARGAR_COMUNICACIONES_FECHAS';

    v_EXCEPCION:= SUBSTR(SQLERRM, 1, 2000);

    INSERT INTO AB61.AB6121T00 (ID_TRAZA, ID_SOLICITUD, ID_EXPEDIENTE, COD_PROCEDURE_ID, EXCEPCION, METODO, ESTADO_SOLICITUD, TAREAS, FECHA_REGISTRO)
    VALUES (n_ID_TRAZA, n_ID_SOLICITUD, n_ID_EXPEDIENTE, n_COD_PROCEDURE_ID, v_EXCEPCION, v_METODO, 2, v_TAREAS, SYSDATE);

    COMMIT;

    END;

WHEN OTHERS THEN
BEGIN
    ROLLBACK;

    SELECT S_AB6121T00.NEXTVAL INTO n_ID_TRAZA FROM DUAL;

      v_EXCEPCION:= SUBSTR(SQLERRM, 1, 2000);
      
    INSERT INTO AB61.AB6121T00 (ID_TRAZA, ID_SOLICITUD, ID_EXPEDIENTE, COD_PROCEDURE_ID, EXCEPCION, METODO, ESTADO_SOLICITUD, TAREAS, FECHA_REGISTRO)
    VALUES (n_ID_TRAZA, n_ID_SOLICITUD, n_ID_EXPEDIENTE, n_COD_PROCEDURE_ID, v_EXCEPCION, v_METODO, 2, v_TAREAS, SYSDATE);

    COMMIT;

    END;

END CARGAR_COMUNICACIONES_FECHAS;

/
