/**
 * 
 */
package com.ejie.ab61.adaptadores.service;

import java.net.URL;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.handler.HandlerResolver;
import javax.xml.ws.handler.PortInfo;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import com.ejie.ab61.clientesWS.X43Procedimientos.AExtensionPoint;
import com.ejie.ab61.clientesWS.X43Procedimientos.TConsultationContext;
import com.ejie.ab61.clientesWS.X43Procedimientos.X43FCP;
import com.ejie.ab61.clientesWS.X43Procedimientos.X43FCPPortType;
import com.ejie.ab61.clientesWS.X43Toolkit.Exception_Exception;
import com.ejie.ab61.comun.Constantes;
import com.ejie.ab61.comun.utilidades.config.Config;

/**
 *  
 * 
 */
@Service(value = "X43ProcedimientosServiceImpl")
public class X43ProcedimientosServiceImpl implements X43ProcedimientosService {

	private static final Logger logger = LoggerFactory.getLogger(X43ProcedimientosServiceImpl.class);
	private X43FCP service;

	private static final QName qname = new QName("com/ejie/x43s/v2/fcp/ws", "X43FCP");

	/**
	 * 
	 * @return X43FCPPortType
	 */
	private X43FCPPortType getX43ProcedimientosServicePort() {
		return this.getX43ProcedimientosServicePort(true);
	}

	/**
	 * 
	 * @param getPort
	 *            boolean
	 * 
	 * @return X43FCPPortType
	 */
	private X43FCPPortType getX43ProcedimientosServicePort(boolean getPort) {
		try {
			Properties prop = Config.loadProperties(Constantes.FICHERO_CONFIGURACION);

			if (this.service == null) {

				X43ProcedimientosServiceImpl.logger
						.info("URL del servicio X43Procedimientos = " + prop.getProperty("X43Procedimientos_URL"));

				// Creamos el servicio
				this.service = new X43FCP(new URL(prop.getProperty("X43Procedimientos_URL")),
						X43ProcedimientosServiceImpl.qname);

				// definimos un handler cliente
				this.service.setHandlerResolver(new HandlerResolver() {
					@SuppressWarnings(value = "unchecked")
					public List getHandlerChain(PortInfo portInfo) {
						return Collections.singletonList(new X43ProcedimientosServiceHandler());
					}
				});

				X43ProcedimientosServiceImpl.logger
						.info("getX43ProcedimientosServicePort. Dentro de creaci�n del servicio - Definido el handler");
			}
			if (getPort) {

				X43ProcedimientosServiceImpl.logger.info("getX43ProcedimientosServicePort. Antes de obtener el proxy");

				X43FCPPortType proxy = this.service.getX43FCPPort();

				X43ProcedimientosServiceImpl.logger
						.info("getX43ProcedimientosServicePort. Despues de obtener el proxy");

				Map<String, Object> ctxt = ((BindingProvider) proxy).getRequestContext();

				X43ProcedimientosServiceImpl.logger
						.info("getX43ProcedimientosServicePort. Despues de obtener el contexto");

				ctxt.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
						// prop.getProperty("URL_WS_X43SolicitudesYAportaciones"));
						prop.getProperty("X43Procedimientos_URL"));

				X43ProcedimientosServiceImpl.logger.info("getX43ProcedimientosServicePort. Cargado el contexto");

				return proxy;
			}

		} catch (Exception e) {

			X43ProcedimientosServiceImpl.logger.info("getX43ProcedimientosServicePort. Excepcionnnn:" + e.getMessage());

			this.service = null;
		}
		return null;
	}

	/**
	 * Metodo que devuelve información sobre un procedimiento.
	 * 
	 * @param procedureID:
	 *            String
	 * 
	 * @param ep
	 *            AExtensionPoint
	 * 
	 * @return TConsultationContext
	 * 
	 */
	public TConsultationContext getConsultationContext(String procedureID, AExtensionPoint ep)
			throws Exception_Exception {

		return this.getX43ProcedimientosServicePort().getConsultationContext(procedureID, ep);

	}

}
