package com.ejie.ab61.dao.configuracionEventos;

import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.sql.DataSource;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.model.configuracionEventos.ConfigEventos;

/**
 * ConfigEventosDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class ConfigEventosDaoImpl implements ConfigEventosDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDCONFIGEVENTOS", "TIPOEVENTO", "PLATEATYPE",
			"PLATEASUBTYPE", "CONFIGURACION", "FECHAREGISTRO", "USUARIOREGISTRO", "DESCRIPCIONCASTELLANO",
			"DESCRIPCIONEUSKERA" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<ConfigEventos> rwMap = new RowMapper<ConfigEventos>() {
		public ConfigEventos mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new ConfigEventos(resultSet.getInt("IDCONFIGEVENTOS"), resultSet.getString("TIPOEVENTO"),
					resultSet.getString("PLATEATYPE"), resultSet.getString("PLATEASUBTYPE"),
					resultSet.getInt("CONFIGURACION"), resultSet.getDate("FECHAREGISTRO"),
					resultSet.getString("USUARIOREGISTRO"), resultSet.getString("DESCRIPCIONCASTELLANO"),
					resultSet.getString("DESCRIPCIONEUSKERA"));
		}
	};

	private RowMapper<ConfigEventos> rwMapFindConfiguracion = new RowMapper<ConfigEventos>() {
		public ConfigEventos mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new ConfigEventos(null, null, null, null, resultSet.getInt("CONFIGURACION"), null, null, null, null);
		}
	};

	private RowMapper<ConfigEventos> rwMapPK = new RowMapper<ConfigEventos>() {
		public ConfigEventos mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new ConfigEventos(resultSet.getInt("IDCONFIGEVENTOS"));
		}
	};

	/**
	 * Rowmapper para Jerarquía
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	private RowMapper<JerarquiaDto<ConfigEventos>> rwMapJerarquia = new RowMapper<JerarquiaDto<ConfigEventos>>() {
		public JerarquiaDto<ConfigEventos> mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			ConfigEventos configeventos = new ConfigEventos(resultSet.getInt("IDCONFIGEVENTOS"),
					resultSet.getString("TIPOEVENTO"), resultSet.getString("PLATEATYPE"),
					resultSet.getString("PLATEASUBTYPE"), resultSet.getInt("CONFIGURACION"),
					resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO"),
					resultSet.getString("DESCRIPCIONCASTELLANO"), resultSet.getString("DESCRIPCIONEUSKERA"));

			JerarquiaDto<ConfigEventos> jerarquia = new JerarquiaDto<ConfigEventos>();
			jerarquia.setModel(configeventos);
			jerarquia.setLevel(resultSet.getBigDecimal("LEVEL").intValue());
			jerarquia.setParentNodes(resultSet.getString("PARENTNODES"));
			jerarquia.setIsLeaf(Boolean.parseBoolean(resultSet.getString("ISLEAF")));
			jerarquia.setFilter(Boolean.parseBoolean(resultSet.getString("FILTER")));
			return jerarquia;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the ConfigEventos table.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	public ConfigEventos add(ConfigEventos configeventos) {
		String query = "INSERT INTO CONFIG_EVENTOS (ID_CONFIG_EVENTOS, TIPO_EVENTO, PLATEA_TYPE, PLATEA_SUBTYPE, CONFIGURACION, DESCRIPCION_CASTELLANO, DESCRIPCION_EUSKERA, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (S_AB6141T00.NEXTVAL,?,?,?,?,?,?,?,?)";
		this.jdbcTemplate.update(query, configeventos.getTipoEvento(), configeventos.getPlateaType(),
				configeventos.getPlateaSubtype(), configeventos.getConfiguracion(),
				configeventos.getDescripcionCastellano(), configeventos.getDescripcionEuskera(),
				configeventos.getFechaRegistro(), configeventos.getUsuarioRegistro());
		return configeventos;
	}

	/**
	 * Updates a single row in the ConfigEventos table.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	public ConfigEventos update(ConfigEventos configeventos) {
		String query = "UPDATE CONFIG_EVENTOS SET TIPO_EVENTO=?, PLATEA_TYPE=?, PLATEA_SUBTYPE=?, CONFIGURACION=?, DESCRIPCION_CASTELLANO=?, DESCRIPCION_EUSKERA=?, FECHA_ULTMODIFICACION=?, USUARIO_ULTMODIFICACION=? WHERE ID_CONFIG_EVENTOS=?";
		this.jdbcTemplate.update(query, configeventos.getTipoEvento(), configeventos.getPlateaType(),
				configeventos.getPlateaSubtype(), configeventos.getConfiguracion(),
				configeventos.getDescripcionCastellano(), configeventos.getDescripcionEuskera(),
				configeventos.getFechaUltmodificacion(), configeventos.getUsuarioUltmodificacion(),
				configeventos.getIdConfigEventos());
		return configeventos;
	}

	/**
	 * Finds a single row in the ConfigEventos table.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	@Transactional(readOnly = true)
	public ConfigEventos find(ConfigEventos configeventos) {
		String query = "SELECT t1.ID_CONFIG_EVENTOS IDCONFIGEVENTOS, t1.TIPO_EVENTO TIPOEVENTO, t1.PLATEA_TYPE PLATEATYPE, t1.PLATEA_SUBTYPE PLATEASUBTYPE, t1.CONFIGURACION CONFIGURACION, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO, t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO, t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA FROM CONFIG_EVENTOS t1  WHERE t1.ID_CONFIG_EVENTOS = ?  ";

		List<ConfigEventos> configeventosList = this.jdbcTemplate.query(query, this.rwMap,
				configeventos.getIdConfigEventos());
		return (ConfigEventos) DataAccessUtils.uniqueResult(configeventosList);
	}

	/**
	 * Encuentra la configuracion de un evento determinado
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	@Transactional(readOnly = true)
	public ConfigEventos findConfiguracion(ConfigEventos configeventos) {
		String query = "SELECT t1.CONFIGURACION CONFIGURACION FROM CONFIG_EVENTOS t1  WHERE t1.TIPO_EVENTO=? AND t1.PLATEA_TYPE=? AND T1.PLATEA_SUBTYPE = ? ";

		List<ConfigEventos> configeventosList = this.jdbcTemplate.query(query, this.rwMapFindConfiguracion,
				configeventos.getTipoEvento(), configeventos.getPlateaType(), configeventos.getPlateaSubtype());
		return (ConfigEventos) DataAccessUtils.uniqueResult(configeventosList);
	}

	/**
	 * Encuentra la configuracion de un evento determinado
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	@Transactional(readOnly = true)
	public ConfigEventos findConfiguracionTareas(ConfigEventos configeventos) {
		String query = "SELECT t1.CONFIGURACION CONFIGURACION FROM CONFIG_EVENTOS t1  WHERE t1.TIPO_EVENTO=? ";

		List<ConfigEventos> configeventosList = this.jdbcTemplate.query(query, this.rwMapFindConfiguracion,
				configeventos.getTipoEvento());
		return (ConfigEventos) DataAccessUtils.uniqueResult(configeventosList);
	}

	/**
	 * Removes a single row in the ConfigEventos table.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return
	 */
	public void remove(ConfigEventos configeventos) {
		String query = "DELETE FROM CONFIG_EVENTOS WHERE ID_CONFIG_EVENTOS=?";
		this.jdbcTemplate.update(query, configeventos.getIdConfigEventos());
	}

	@Override
	public Integer getNumeroConfiguraciones(ConfigEventos configEventos) {
		String query = "SELECT COUNT(1) NUMCONFIGURACIONES FROM CONFIGURACION t1 WHERE t1.ID_CONFIG_EVENTOS = ?";
		return this.jdbcTemplate.queryForInt(query, configEventos.getIdConfigEventos());
	}

	/**
	 * Finds a list of rows in the ConfigEventos table.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<ConfigEventos>
	 */
	@Transactional(readOnly = true)
	public List<ConfigEventos> findAll(ConfigEventos configeventos, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_CONFIG_EVENTOS IDCONFIGEVENTOS,t1.TIPO_EVENTO TIPOEVENTO,t1.PLATEA_TYPE PLATEATYPE,t1.PLATEA_SUBTYPE PLATEASUBTYPE,t1.CONFIGURACION CONFIGURACION,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO, t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO, t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA ");
		query.append("FROM CONFIG_EVENTOS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(configeventos);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, ConfigEventosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<ConfigEventos>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the ConfigEventos table using like.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<ConfigEventos>
	 */
	@Transactional(readOnly = true)
	public List<ConfigEventos> findAllLike(ConfigEventos configeventos, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_CONFIG_EVENTOS IDCONFIGEVENTOS,t1.TIPO_EVENTO TIPOEVENTO,t1.PLATEA_TYPE PLATEATYPE,t1.PLATEA_SUBTYPE PLATEASUBTYPE,t1.CONFIGURACION CONFIGURACION,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO, t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO, t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA ");
		query.append("FROM CONFIG_EVENTOS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(configeventos, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, ConfigEventosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<ConfigEventos>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the ConfigEventos table.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(ConfigEventos configeventos) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM CONFIG_EVENTOS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(configeventos);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the ConfigEventos table using like.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(ConfigEventos configeventos, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM CONFIG_EVENTOS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(configeventos, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of ConfigEventos selected for rup_table
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<ConfigEventos>>
	 */
	@Override
	public List<TableRowDto<ConfigEventos>> reorderSelection(ConfigEventos configeventos,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_CONFIG_EVENTOS IDCONFIGEVENTOS,t1.TIPO_EVENTO TIPOEVENTO,t1.PLATEA_TYPE PLATEATYPE,t1.PLATEA_SUBTYPE PLATEASUBTYPE,t1.CONFIGURACION CONFIGURACION,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		query.append("FROM CONFIG_EVENTOS t1 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(configeventos, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto,
				ConfigEventos.class, filterParamList, "ID_CONFIG_EVENTOS");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<ConfigEventos>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
	 *
	 * @param filterParams
	 *            ConfigEventos
	 * @param searchParams
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<ConfigEventos>>
	 */
	@Override
	public List<TableRowDto<ConfigEventos>> search(ConfigEventos filterParams, ConfigEventos searchParams,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_CONFIG_EVENTOS IDCONFIGEVENTOS,t1.TIPO_EVENTO TIPOEVENTO,t1.PLATEA_TYPE PLATEATYPE,t1.PLATEA_SUBTYPE PLATEASUBTYPE,t1.CONFIGURACION CONFIGURACION,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		query.append("FROM CONFIG_EVENTOS t1 ");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto, ConfigEventos.class,
				filterParamList, searchSQL, searchParamList, from_alias, "ID_CONFIG_EVENTOS");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<ConfigEventos>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
	 *
	 * @param filterconfigeventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	@Override
	public void removeMultiple(ConfigEventos filterconfigeventos, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_CONFIG_EVENTOS IDCONFIGEVENTOS,t1.TIPO_EVENTO TIPOEVENTO,t1.PLATEA_TYPE PLATEATYPE,t1.PLATEA_SUBTYPE PLATEASUBTYPE,t1.CONFIGURACION CONFIGURACION,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		query.append("FROM CONFIG_EVENTOS t1 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filterconfigeventos, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto, ConfigEventos.class,
				query, params, "ID_CONFIG_EVENTOS");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Filter in the hierarchical ConfigEventos table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<ConfigEventos>>
	 */
	@Override
	public List<JerarquiaDto<ConfigEventos>> findAllLikeJerarquia(ConfigEventos filterConfigEventos,
			JQGridRequestDto jqGridRequestDto) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_CONFIG_EVENTOS IDCONFIGEVENTOS,t1.TIPO_EVENTO TIPOEVENTO,t1.PLATEA_TYPE PLATEATYPE,t1.PLATEA_SUBTYPE PLATEASUBTYPE,t1.CONFIGURACION CONFIGURACION,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO, t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO, t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA ");
		// FROM
		// StringBuilder from = new StringBuilder("FROM CONFIG_EVENTOS t1 ");
		List<String> from = new ArrayList<String>();
		from.add("CONFIG_EVENTOS");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterConfigEventos, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID_CONFIG_EVENTOS", "ID_PADRE",
				"NOMBRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query,
		// mapaWhere, "ID", "ID_PADRE", "NOMBRE", from, from_alias, joins,
		// businessFilters, businessParams);

		// PAGINACION
		if (jqGridRequestDto != null) {
			query = JQGridManagerJerarquia.getPaginationQuery(jqGridRequestDto, query,
					ConfigEventosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.query(query.toString(), this.rwMapJerarquia, params.toArray());
	}

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	@Override
	public Long findAllLikeCountJerarquia(ConfigEventos filterConfigEventos, JQGridRequestDto jqGridRequestDto) {
		// FROM
		// StringBuilder from = new StringBuilder("FROM CONFIG_EVENTOS t1 ");
		List<String> from = new ArrayList<String>();
		from.add("CONFIG_EVENTOS");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterConfigEventos, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID_CONFIG_EVENTOS",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<ConfigEventos>>
	 */
	@Override
	public List<TableRowDto<ConfigEventos>> findAllChild(ConfigEventos filterConfigEventos,
			JQGridRequestDto jqGridRequestDto) {

		// FROM
		// StringBuilder from = new StringBuilder("FROM CONFIG_EVENTOS t1 ");
		List<String> from = new ArrayList<String>();
		from.add("CONFIG_EVENTOS");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterConfigEventos, false);

		// MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID_CONFIG_EVENTOS",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(),
				new RowNumResultSetExtractor<ConfigEventos>(this.rwMapPK, "ID_CONFIG_EVENTOS"), params.toArray());
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the ConfigEventos entity
	 *
	 * @param configeventos
	 *            ConfigEventos Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(ConfigEventos configeventos) {

		StringBuilder where = new StringBuilder(ConfigEventosDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (configeventos != null && configeventos.getIdConfigEventos() != null) {
			where.append(" AND t1.ID_CONFIG_EVENTOS = ?");
			params.add(configeventos.getIdConfigEventos());
		}
		if (configeventos != null && configeventos.getTipoEvento() != null) {
			where.append(" AND t1.TIPO_EVENTO = ?");
			params.add(configeventos.getTipoEvento());
		}
		if (configeventos != null && configeventos.getPlateaType() != null) {
			where.append(" AND t1.PLATEA_TYPE = ?");
			params.add(configeventos.getPlateaType());
		}
		if (configeventos != null && configeventos.getPlateaSubtype() != null) {
			where.append(" AND t1.PLATEA_SUBTYPE = ?");
			params.add(configeventos.getPlateaSubtype());
		}
		if (configeventos != null && configeventos.getConfiguracion() != null) {
			where.append(" AND t1.CONFIGURACION = ?");
			params.add(configeventos.getConfiguracion());
		}
		if (configeventos != null && configeventos.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(configeventos.getFechaRegistro());
		}
		if (configeventos != null && configeventos.getUsuarioRegistro() != null) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(configeventos.getUsuarioRegistro());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the ConfigEventos entity
	 *
	 * @param configeventos
	 *            ConfigEventos Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(ConfigEventos configeventos, Boolean startsWith) {

		StringBuilder where = new StringBuilder(ConfigEventosDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (configeventos != null && configeventos.getIdConfigEventos() != null) {
			where.append(" AND t1.ID_CONFIG_EVENTOS = ?");
			params.add(configeventos.getIdConfigEventos());
		}
		if (configeventos != null && configeventos.getTipoEvento() != null) {
			where.append(" AND UPPER(t1.TIPO_EVENTO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(configeventos.getTipoEvento().toUpperCase() + "%");
			} else {
				params.add("%" + configeventos.getTipoEvento().toUpperCase() + "%");
			}
			where.append(" AND t1.TIPO_EVENTO IS NOT NULL");
		}
		if (configeventos != null && configeventos.getPlateaType() != null) {
			where.append(" AND UPPER(t1.PLATEA_TYPE) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(configeventos.getPlateaType().toUpperCase() + "%");
			} else {
				params.add("%" + configeventos.getPlateaType().toUpperCase() + "%");
			}
			where.append(" AND t1.PLATEA_TYPE IS NOT NULL");
		}
		if (configeventos != null && configeventos.getPlateaSubtype() != null) {
			where.append(" AND UPPER(t1.PLATEA_SUBTYPE) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(configeventos.getPlateaSubtype().toUpperCase() + "%");
			} else {
				params.add("%" + configeventos.getPlateaSubtype().toUpperCase() + "%");
			}
			where.append(" AND t1.PLATEA_SUBTYPE IS NOT NULL");
		}
		if (configeventos != null && configeventos.getConfiguracion() != null) {
			where.append(" AND t1.CONFIGURACION = ?");
			params.add(configeventos.getConfiguracion());
		}
		if (configeventos != null && configeventos.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(configeventos.getFechaRegistro());
		}
		if (configeventos != null && configeventos.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(configeventos.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + configeventos.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
