package com.ejie.ab61.dao.expedientes;

import com.ejie.ab61.model.expedientes.Expedientes;
import com.ejie.ab61.model.procedimientos.Procedimientos;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.sql.DataSource;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/**
 * ExpedientesDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class ExpedientesDaoImpl implements ExpedientesDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDEXPEDIENTE", "IDPROCEDIMIENTO",
			"IDTERRITORIOHISTORICO", "FOLDERID", "FOLDERNUMBER", "TITULO", "FECHAENTRADA", "FECHAREGISTRO",
			"USUARIOREGISTRO" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<Expedientes> rwMap = new RowMapper<Expedientes>() {
		public Expedientes mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Expedientes(resultSet.getInt("IDEXPEDIENTE"), resultSet.getInt("IDPROCEDIMIENTO"),
					resultSet.getLong("IDTERRITORIOHISTORICO"), resultSet.getString("FOLDERID"),
					resultSet.getString("FOLDERNUMBER"), resultSet.getString("TITULO"),
					resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO"));
		}
	};

	private RowMapper<Expedientes> rwMapFindByFolderId = new RowMapper<Expedientes>() {
		public Expedientes mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Expedientes(resultSet.getInt("IDEXPEDIENTE"), resultSet.getInt("IDPROCEDIMIENTO"),
					resultSet.getLong("IDTERRITORIOHISTORICO"), resultSet.getString("FOLDERID"),
					resultSet.getString("FOLDERNUMBER"), resultSet.getString("TITULO"),
					resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO"),
					new Procedimientos(null, null, null, null, null, null, null, null, null,
							resultSet.getString("DESCRIPCIONCASTELLANO"), resultSet.getString("DESCRIPCIONEUSKERA")));
		}
	};

	// private RowMapper<Expedientes> rwMapFindByFolderId = new
	// RowMapper<Expedientes>() {
	// public Expedientes mapRow(ResultSet resultSet, int rowNum) throws
	// SQLException {
	// return new Expedientes(resultSet.getInt("IDEXPEDIENTE"),
	// resultSet.getInt("IDPROCEDIMIENTO"),
	// resultSet.getLong("IDTERRITORIOHISTORICO"),
	// resultSet.getString("FOLDERID"),
	// resultSet.getString("FOLDERNUMBER"), resultSet.getString("TITULO"),
	// resultSet.getDate("FECHAENTRADA"), resultSet.getDate("FECHAREGISTRO"),
	// resultSet.getString("USUARIOREGISTRO"),
	// resultSet.getString("DESCRIPCIONCASTELLANO"),
	// resultSet.getString("DESCRIPCIONEUSKERA"));
	// }
	// };

	private RowMapper<Expedientes> rwMapPK = new RowMapper<Expedientes>() {
		public Expedientes mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Expedientes(resultSet.getInt("IDEXPEDIENTE"));
		}
	};

	private RowMapper<Expedientes> rwMapTitulo = new RowMapper<Expedientes>() {
		public Expedientes mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Expedientes(resultSet.getString("TITULO"));
		}
	};
	/**
	 * Rowmapper para Jerarquía
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	private RowMapper<JerarquiaDto<Expedientes>> rwMapJerarquia = new RowMapper<JerarquiaDto<Expedientes>>() {
		public JerarquiaDto<Expedientes> mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			Expedientes expedientes = new Expedientes(resultSet.getInt("IDEXPEDIENTE"),
					resultSet.getInt("IDPROCEDIMIENTO"), resultSet.getLong("IDTERRITORIOHISTORICO"),
					resultSet.getString("FOLDERID"), resultSet.getString("FOLDERNUMBER"), resultSet.getString("TITULO"),
					resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO"));

			JerarquiaDto<Expedientes> jerarquia = new JerarquiaDto<Expedientes>();
			jerarquia.setModel(expedientes);
			jerarquia.setLevel(resultSet.getBigDecimal("LEVEL").intValue());
			jerarquia.setParentNodes(resultSet.getString("PARENTNODES"));
			jerarquia.setIsLeaf(Boolean.parseBoolean(resultSet.getString("ISLEAF")));
			jerarquia.setFilter(Boolean.parseBoolean(resultSet.getString("FILTER")));
			return jerarquia;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @return Expedientes
	 */
	public Expedientes add(Expedientes expedientes) {
		String query = "INSERT INTO EXPEDIENTES (ID_EXPEDIENTE, ID_PROCEDIMIENTO, ID_TERRITORIO_HISTORICO, FOLDER_ID, FOLDER_NUMBER, TITULO, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (?,?,?,?,?,?,?,?)";
		this.jdbcTemplate.update(query, expedientes.getIdExpediente(), expedientes.getIdProcedimiento(),
				expedientes.getIdTerritorioHistorico(), expedientes.getFolderId(), expedientes.getFolderNumber(),
				expedientes.getTitulo(), expedientes.getFechaRegistro(), expedientes.getUsuarioRegistro());
		return expedientes;
	}

	/**
	 * Updates a single row in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @return Expedientes
	 */
	public Expedientes update(Expedientes expedientes) {
		String query = "UPDATE EXPEDIENTES SET ID_PROCEDIMIENTO=?, ID_TERRITORIO_HISTORICO=?, FOLDER_ID=?, FOLDER_NUMBER=?, TITULO=?, FECHA_REGISTRO=?, USUARIO_REGISTRO=? WHERE ID_EXPEDIENTE=?";
		this.jdbcTemplate.update(query, expedientes.getIdProcedimiento(), expedientes.getIdTerritorioHistorico(),
				expedientes.getFolderId(), expedientes.getFolderNumber(), expedientes.getTitulo(),
				expedientes.getFechaRegistro(), expedientes.getUsuarioRegistro(), expedientes.getIdExpediente());
		return expedientes;
	}

	/**
	 * Updates a single row in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @return Expedientes
	 */
	public Expedientes updateFolderNumber(Expedientes expedientes) {
		String query = "UPDATE EXPEDIENTES SET FOLDER_NUMBER=? WHERE FOLDER_ID=?";
		this.jdbcTemplate.update(query, expedientes.getFolderNumber(), expedientes.getFolderId());
		return expedientes;
	}

	/**
	 * Finds a single row in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @return Expedientes
	 */
	@Transactional(readOnly = true)
	public Expedientes find(Expedientes expedientes) {
		String query = "SELECT t1.ID_EXPEDIENTE IDEXPEDIENTE, t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO, t1.ID_TERRITORIO_HISTORICO IDTERRITORIOHISTORICO, t1.FOLDER_ID FOLDERID, t1.FOLDER_NUMBER FOLDERNUMBER, t1.TITULO TITULO, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO FROM EXPEDIENTES t1  WHERE t1.ID_EXPEDIENTE = ?  ";

		List<Expedientes> expedientesList = this.jdbcTemplate.query(query, this.rwMap, expedientes.getIdExpediente());
		return (Expedientes) DataAccessUtils.uniqueResult(expedientesList);
	}

	/**
	 * Updates a single row in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @return Expedientes
	 */
	public Expedientes updateFolder(Expedientes expedientes) {
		String query = "UPDATE EXPEDIENTES SET FOLDER_NUMBER=?, FOLDER_ID=? WHERE ID_EXPEDIENTE=?";

		this.jdbcTemplate.update(query, expedientes.getFolderNumber(), expedientes.getFolderId(),
				expedientes.getIdExpediente());
		return expedientes;
	}

	/**
	 * Finds a single row in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @return Expedientes
	 */
	@Transactional(readOnly = true)
	public Expedientes findByFolderId(String folderId) {
		String query = "SELECT t1.ID_EXPEDIENTE IDEXPEDIENTE, t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO, t1.ID_TERRITORIO_HISTORICO IDTERRITORIOHISTORICO, t1.FOLDER_ID FOLDERID, t1.FOLDER_NUMBER FOLDERNUMBER, t1.TITULO TITULO, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO, t2.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO, t2.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA FROM EXPEDIENTES t1,PROCEDIMIENTOS t2 WHERE t1.ID_PROCEDIMIENTO = t2.ID_PROCEDIMIENTO AND t1.FOLDER_ID = ?";

		List<Expedientes> expedientesList = this.jdbcTemplate.query(query, this.rwMapFindByFolderId, folderId);
		return (Expedientes) DataAccessUtils.uniqueResult(expedientesList);
	}

	/**
	 * Removes a single row in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @return
	 */
	public void remove(Expedientes expedientes) {
		String query = "DELETE FROM EXPEDIENTES WHERE ID_EXPEDIENTE=?";
		this.jdbcTemplate.update(query, expedientes.getIdExpediente());
	}

	/**
	 * Finds a list of rows in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Expedientes>
	 */
	@Transactional(readOnly = true)
	public List<Expedientes> findAll(Expedientes expedientes, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_EXPEDIENTE IDEXPEDIENTE,t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO,t1.ID_TERRITORIO_HISTORICO IDTERRITORIOHISTORICO,t1.FOLDER_ID FOLDERID,t1.FOLDER_NUMBER FOLDERNUMBER,t1.TITULO TITULO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		query.append("FROM EXPEDIENTES t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(expedientes);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, ExpedientesDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Expedientes>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the Expedientes table using like.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Expedientes>
	 */
	@Transactional(readOnly = true)
	public List<Expedientes> findAllLike(Expedientes expedientes, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_EXPEDIENTE IDEXPEDIENTE,t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO,t1.ID_TERRITORIO_HISTORICO IDTERRITORIOHISTORICO,t1.FOLDER_ID FOLDERID,t1.FOLDER_NUMBER FOLDERNUMBER,t1.TITULO TITULO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		query.append("FROM EXPEDIENTES t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(expedientes, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, ExpedientesDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Expedientes>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(Expedientes expedientes) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM EXPEDIENTES t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(expedientes);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the Expedientes table using like.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(Expedientes expedientes, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM EXPEDIENTES t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(expedientes, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of Expedientes selected for rup_table
	 *
	 * @param expedientes
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Expedientes>>
	 */
	@Override
	public List<TableRowDto<Expedientes>> reorderSelection(Expedientes expedientes, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_EXPEDIENTE IDEXPEDIENTE,t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO,t1.ID_TERRITORIO_HISTORICO IDTERRITORIOHISTORICO,t1.FOLDER_ID FOLDERID,t1.FOLDER_NUMBER FOLDERNUMBER,t1.TITULO TITULO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		query.append("FROM EXPEDIENTES t1 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(expedientes, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto, Expedientes.class,
				filterParamList, "ID_EXPEDIENTE");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Expedientes>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
	 *
	 * @param filterParams
	 *            Expedientes
	 * @param searchParams
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Expedientes>>
	 */
	@Override
	public List<TableRowDto<Expedientes>> search(Expedientes filterParams, Expedientes searchParams,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_EXPEDIENTE IDEXPEDIENTE,t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO,t1.ID_TERRITORIO_HISTORICO IDTERRITORIOHISTORICO,t1.FOLDER_ID FOLDERID,t1.FOLDER_NUMBER FOLDERNUMBER,t1.TITULO TITULO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		query.append("FROM EXPEDIENTES t1 ");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto, Expedientes.class,
				filterParamList, searchSQL, searchParamList, from_alias, "ID_EXPEDIENTE");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Expedientes>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
	 *
	 * @param filterexpedientes
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	@Override
	public void removeMultiple(Expedientes filterexpedientes, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_EXPEDIENTE IDEXPEDIENTE,t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO,t1.ID_TERRITORIO_HISTORICO IDTERRITORIOHISTORICO,t1.FOLDER_ID FOLDERID,t1.FOLDER_NUMBER FOLDERNUMBER,t1.TITULO TITULO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		query.append("FROM EXPEDIENTES t1 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filterexpedientes, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto, Expedientes.class,
				query, params, "ID_EXPEDIENTE");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Filter in the hierarchical Expedientes table.
	 *
	 * @param filterExpedientes
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<Expedientes>>
	 */
	@Override
	public List<JerarquiaDto<Expedientes>> findAllLikeJerarquia(Expedientes filterExpedientes,
			JQGridRequestDto jqGridRequestDto) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_EXPEDIENTE IDEXPEDIENTE,t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO,t1.ID_TERRITORIO_HISTORICO IDTERRITORIOHISTORICO,t1.FOLDER_ID FOLDERID,t1.FOLDER_NUMBER FOLDERNUMBER,t1.TITULO TITULO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		// StringBuilder from = new StringBuilder("FROM EXPEDIENTES t1 ");
		List<String> from = new ArrayList<String>();
		from.add("EXPEDIENTES");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterExpedientes, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID_EXPEDIENTE", "ID_PADRE",
				"NOMBRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query,
		// mapaWhere, "ID", "ID_PADRE", "NOMBRE", from, from_alias, joins,
		// businessFilters, businessParams);

		// PAGINACION
		if (jqGridRequestDto != null) {
			query = JQGridManagerJerarquia.getPaginationQuery(jqGridRequestDto, query,
					ExpedientesDaoImpl.ORDER_BY_WHITE_LIST);
		}

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.query(query.toString(), this.rwMapJerarquia, params.toArray());
	}

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterExpedientes
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	@Override
	public Long findAllLikeCountJerarquia(Expedientes filterExpedientes, JQGridRequestDto jqGridRequestDto) {
		// FROM
		// StringBuilder from = new StringBuilder("FROM EXPEDIENTES t1 ");
		List<String> from = new ArrayList<String>();
		from.add("EXPEDIENTES");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterExpedientes, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID_EXPEDIENTE",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterExpedientes
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<Expedientes>>
	 */
	@Override
	public List<TableRowDto<Expedientes>> findAllChild(Expedientes filterExpedientes,
			JQGridRequestDto jqGridRequestDto) {

		// FROM
		// StringBuilder from = new StringBuilder("FROM EXPEDIENTES t1 ");
		List<String> from = new ArrayList<String>();
		from.add("EXPEDIENTES");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterExpedientes, false);

		// MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID_EXPEDIENTE",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(),
				new RowNumResultSetExtractor<Expedientes>(this.rwMapPK, "ID_EXPEDIENTE"), params.toArray());
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Expedientes entity
	 *
	 * @param expedientes
	 *            Expedientes Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(Expedientes expedientes) {

		StringBuilder where = new StringBuilder(ExpedientesDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (expedientes != null && expedientes.getIdExpediente() != null) {
			where.append(" AND t1.ID_EXPEDIENTE = ?");
			params.add(expedientes.getIdExpediente());
		}
		if (expedientes != null && expedientes.getIdProcedimiento() != null) {
			where.append(" AND t1.ID_PROCEDIMIENTO = ?");
			params.add(expedientes.getIdProcedimiento());
		}
		if (expedientes != null && expedientes.getIdTerritorioHistorico() != null) {
			where.append(" AND t1.ID_TERRITORIO_HISTORICO = ?");
			params.add(expedientes.getIdTerritorioHistorico());
		}
		if (expedientes != null && expedientes.getFolderId() != null) {
			where.append(" AND t1.FOLDER_ID = ?");
			params.add(expedientes.getFolderId());
		}
		if (expedientes != null && expedientes.getFolderNumber() != null) {
			where.append(" AND t1.FOLDER_NUMBER = ?");
			params.add(expedientes.getFolderNumber());
		}
		if (expedientes != null && expedientes.getTitulo() != null) {
			where.append(" AND t1.TITULO = ?");
			params.add(expedientes.getTitulo());
		}
		if (expedientes != null && expedientes.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(expedientes.getFechaRegistro());
		}
		if (expedientes != null && expedientes.getUsuarioRegistro() != null) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(expedientes.getUsuarioRegistro());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Expedientes entity
	 *
	 * @param expedientes
	 *            Expedientes Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(Expedientes expedientes, Boolean startsWith) {

		StringBuilder where = new StringBuilder(ExpedientesDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (expedientes != null && expedientes.getIdExpediente() != null) {
			where.append(" AND t1.ID_EXPEDIENTE = ?");
			params.add(expedientes.getIdExpediente());
		}
		if (expedientes != null && expedientes.getIdProcedimiento() != null) {
			where.append(" AND t1.ID_PROCEDIMIENTO = ?");
			params.add(expedientes.getIdProcedimiento());
		}
		if (expedientes != null && expedientes.getIdTerritorioHistorico() != null) {
			where.append(" AND t1.ID_TERRITORIO_HISTORICO = ?");
			params.add(expedientes.getIdTerritorioHistorico());
		}
		if (expedientes != null && expedientes.getFolderId() != null) {
			where.append(" AND UPPER(t1.FOLDER_ID) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(expedientes.getFolderId().toUpperCase() + "%");
			} else {
				params.add("%" + expedientes.getFolderId().toUpperCase() + "%");
			}
			where.append(" AND t1.FOLDER_ID IS NOT NULL");
		}
		if (expedientes != null && expedientes.getFolderNumber() != null) {
			where.append(" AND UPPER(t1.FOLDER_NUMBER) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(expedientes.getFolderNumber().toUpperCase() + "%");
			} else {
				params.add("%" + expedientes.getFolderNumber().toUpperCase() + "%");
			}
			where.append(" AND t1.FOLDER_NUMBER IS NOT NULL");
		}
		if (expedientes != null && expedientes.getTitulo() != null) {
			where.append(" AND UPPER(t1.TITULO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(expedientes.getTitulo().toUpperCase() + "%");
			} else {
				params.add("%" + expedientes.getTitulo().toUpperCase() + "%");
			}
			where.append(" AND t1.TITULO IS NOT NULL");
		}

		if (expedientes != null && expedientes.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(expedientes.getFechaRegistro());
		}
		if (expedientes != null && expedientes.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(expedientes.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + expedientes.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;

	/**
	 * obtenerTituloExpediente
	 * 
	 * @param folderID
	 * @return Expedientes
	 */
	@Override
	public List<Expedientes> obtenerTituloExpediente(String folderID) {
		String query = "SELECT TITULO FROM EXPEDIENTES WHERE FOLDER_ID=?";
		return this.jdbcTemplate.query(query, this.rwMapTitulo, folderID);
	}

	/**
	 * Método que actualizar el tiutlo del expediente con el titulo que viene de
	 * Platea (Tramitagune)
	 * 
	 * @param resultado
	 * @param usuario
	 */
	public void updateFolderTitle(Expedientes expediente) {
		String query = "UPDATE EXPEDIENTES SET TITULO=?, FECHA_REGISTRO=sysdate, USUARIO_REGISTRO=? WHERE FOLDER_ID=?";
		this.jdbcTemplate.update(query, expediente.getTitulo(), expediente.getUsuarioRegistro(),
				expediente.getFolderId());
	}

}
