package com.ejie.ab61.dao.expedientes;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.model.expedientes.FechasExpediente;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * FechasExpedienteDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class FechasExpedienteDaoImpl implements FechasExpedienteDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDFECHAEXPEDIENTE", "TIPOEVENTO", "PLATEATYPE",
			"PLATEASUBTYPE", "TIPODOKUSI", "SUBTIPODOKUSI", "ASUNTODOCUMENTAL", "TASKID", "SITUACIONID", "STATUS",
			"ETIQUETAFECHA", "DESCRIPCIONCASTELLANO", "DESCRIPCIONEUSKERA", "FECHAREGISTRO", "USUARIOREGISTRO",
			"INSTANCEID", "FECHAULTMODIFICACION", "USUARIOULTMODIFICACION" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<FechasExpediente> rwMap = new RowMapper<FechasExpediente>() {
		public FechasExpediente mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new FechasExpediente(resultSet.getInt("IDFECHAEXPEDIENTE"), resultSet.getString("TIPOEVENTO"),
					resultSet.getString("PLATEATYPE"), resultSet.getString("PLATEASUBTYPE"),
					resultSet.getString("TIPODOKUSI"), resultSet.getString("SUBTIPODOKUSI"),
					resultSet.getString("ASUNTODOCUMENTAL"), resultSet.getString("TASKID"),
					resultSet.getString("SITUACIONID"), resultSet.getBigDecimal("STATUS"),
					resultSet.getString("ETIQUETAFECHA"), resultSet.getString("DESCRIPCIONCASTELLANO"),
					resultSet.getString("DESCRIPCIONEUSKERA"), resultSet.getDate("FECHAREGISTRO"),
					resultSet.getString("USUARIOREGISTRO"), resultSet.getLong("INSTANCEID"),
					resultSet.getDate("FECHAULTMODIFICACION"), resultSet.getString("USUARIOULTMODIFICACION"));
		}
	};

	private RowMapper<FechasExpediente> rwMapPK = new RowMapper<FechasExpediente>() {
		public FechasExpediente mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new FechasExpediente(resultSet.getInt("IDFECHAEXPEDIENTE"));
		}
	};

	/**
	 * Rowmapper para Jerarquía
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	private RowMapper<JerarquiaDto<FechasExpediente>> rwMapJerarquia = new RowMapper<JerarquiaDto<FechasExpediente>>() {
		public JerarquiaDto<FechasExpediente> mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			FechasExpediente fechasexpediente = new FechasExpediente(resultSet.getInt("IDFECHAEXPEDIENTE"),
					resultSet.getString("TIPOEVENTO"), resultSet.getString("PLATEATYPE"),
					resultSet.getString("PLATEASUBTYPE"), resultSet.getString("TIPODOKUSI"),
					resultSet.getString("SUBTIPODOKUSI"), resultSet.getString("ASUNTODOCUMENTAL"),
					resultSet.getString("TASKID"), resultSet.getString("SITUACIONID"),
					resultSet.getBigDecimal("STATUS"), resultSet.getString("ETIQUETAFECHA"),
					resultSet.getString("DESCRIPCIONCASTELLANO"), resultSet.getString("DESCRIPCIONEUSKERA"),
					resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO"),
					resultSet.getLong("INSTANCEID"), resultSet.getDate("FECHAULTMODIFICACION"),
					resultSet.getString("USUARIOULTMODIFICACION"));

			JerarquiaDto<FechasExpediente> jerarquia = new JerarquiaDto<FechasExpediente>();
			jerarquia.setModel(fechasexpediente);
			jerarquia.setLevel(resultSet.getBigDecimal("LEVEL").intValue());
			jerarquia.setParentNodes(resultSet.getString("PARENTNODES"));
			jerarquia.setIsLeaf(Boolean.parseBoolean(resultSet.getString("ISLEAF")));
			jerarquia.setFilter(Boolean.parseBoolean(resultSet.getString("FILTER")));
			return jerarquia;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the FechasExpediente table.
	 *
	 * @param fechasexpediente
	 *            FechasExpediente
	 * @return FechasExpediente
	 */
	public FechasExpediente add(FechasExpediente fechasexpediente) {
		String query = "INSERT INTO FECHAS_EXPEDIENTE (ID_FECHA_EXPEDIENTE, TIPO_EVENTO, PLATEA_TYPE, PLATEA_SUBTYPE, TIPO_DOKUSI, SUBTIPO_DOKUSI, ASUNTO_DOCUMENTAL, TASK_ID, SITUACION_ID, STATUS, ETIQUETA_FECHA, DESCRIPCION_CASTELLANO, DESCRIPCION_EUSKERA, FECHA_REGISTRO, USUARIO_REGISTRO, INSTANCE_ID) VALUES (S_AB6144T00.NEXTVAL,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
		this.jdbcTemplate.update(query, fechasexpediente.getTipoEvento(), fechasexpediente.getPlateaType(),
				fechasexpediente.getPlateaSubtype(), fechasexpediente.getTipoDokusi(),
				fechasexpediente.getSubtipoDokusi(), fechasexpediente.getAsuntoDocumental(),
				fechasexpediente.getTaskId(), fechasexpediente.getSituacionId(), fechasexpediente.getStatus(),
				fechasexpediente.getEtiquetaFecha(), fechasexpediente.getDescripcionCastellano(),
				fechasexpediente.getDescripcionEuskera(), fechasexpediente.getFechaRegistro(),
				fechasexpediente.getUsuarioRegistro(), fechasexpediente.getInstanceId());
		return fechasexpediente;
	}

	/**
	 * Updates a single row in the FechasExpediente table.
	 *
	 * @param fechasexpediente
	 *            FechasExpediente
	 * @return FechasExpediente
	 */
	public FechasExpediente update(FechasExpediente fechasexpediente) {
		String query = "UPDATE FECHAS_EXPEDIENTE SET TIPO_EVENTO=?, PLATEA_TYPE=?, PLATEA_SUBTYPE=?, TIPO_DOKUSI=?, SUBTIPO_DOKUSI=?, ASUNTO_DOCUMENTAL=?, TASK_ID=?, SITUACION_ID=?, STATUS=?, ETIQUETA_FECHA=?, DESCRIPCION_CASTELLANO=?, DESCRIPCION_EUSKERA=?, INSTANCE_ID=?, FECHA_ULTMODIFICACION=?, USUARIO_ULTMODIFICACION=? WHERE ID_FECHA_EXPEDIENTE=?";
		this.jdbcTemplate.update(query, fechasexpediente.getTipoEvento(), fechasexpediente.getPlateaType(),
				fechasexpediente.getPlateaSubtype(), fechasexpediente.getTipoDokusi(),
				fechasexpediente.getSubtipoDokusi(), fechasexpediente.getAsuntoDocumental(),
				fechasexpediente.getTaskId(), fechasexpediente.getSituacionId(), fechasexpediente.getStatus(),
				fechasexpediente.getEtiquetaFecha(), fechasexpediente.getDescripcionCastellano(),
				fechasexpediente.getDescripcionEuskera(), fechasexpediente.getInstanceId(),
				fechasexpediente.getFechaUltmodificacion(), fechasexpediente.getUsuarioUltmodificacion(),
				fechasexpediente.getIdFechaExpediente());
		return fechasexpediente;
	}

	/**
	 * Finds a single row in the FechasExpediente table.
	 *
	 * @param fechasexpediente
	 *            FechasExpediente
	 * @return FechasExpediente
	 */
	@Transactional(readOnly = true)
	public FechasExpediente find(FechasExpediente fechasexpediente) {
		String query = "SELECT t1.ID_FECHA_EXPEDIENTE IDFECHAEXPEDIENTE, t1.TIPO_EVENTO TIPOEVENTO, t1.PLATEA_TYPE PLATEATYPE, t1.PLATEA_SUBTYPE PLATEASUBTYPE, t1.TIPO_DOKUSI TIPODOKUSI, t1.SUBTIPO_DOKUSI SUBTIPODOKUSI, t1.ASUNTO_DOCUMENTAL ASUNTODOCUMENTAL, t1.TASK_ID TASKID, t1.SITUACION_ID SITUACIONID, t1.STATUS STATUS, t1.ETIQUETA_FECHA ETIQUETAFECHA, t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO, t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO, t1.INSTANCE_ID INSTANCEID, t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION, t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION FROM FECHAS_EXPEDIENTE t1  WHERE t1.ID_FECHA_EXPEDIENTE = ?  ";

		List<FechasExpediente> fechasexpedienteList = this.jdbcTemplate.query(query, this.rwMap,
				fechasexpediente.getIdFechaExpediente());
		return (FechasExpediente) DataAccessUtils.uniqueResult(fechasexpedienteList);
	}

	/**
	 * Removes a single row in the FechasExpediente table.
	 *
	 * @param fechasexpediente
	 *            FechasExpediente
	 * @return
	 */
	public void remove(FechasExpediente fechasexpediente) {
		String query = "DELETE FROM FECHAS_EXPEDIENTE WHERE ID_FECHA_EXPEDIENTE=?";
		this.jdbcTemplate.update(query, fechasexpediente.getIdFechaExpediente());
	}

	@Override
	public Integer getNumeroRelExpFech(FechasExpediente fechasExpediente) {
		String query = "SELECT COUNT(1) NUMRELEXPFECH FROM REL_EXP_FECH WHERE ID_FECHA_EXPEDIENTE = ?";
		return this.jdbcTemplate.queryForInt(query, fechasExpediente.getIdFechaExpediente());
	}

	/**
	 * Finds a list of rows in the FechasExpediente table.
	 *
	 * @param fechasexpediente
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<FechasExpediente>
	 */
	@Transactional(readOnly = true)
	public List<FechasExpediente> findAll(FechasExpediente fechasexpediente, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_FECHA_EXPEDIENTE IDFECHAEXPEDIENTE,t1.TIPO_EVENTO TIPOEVENTO,t1.PLATEA_TYPE PLATEATYPE,t1.PLATEA_SUBTYPE PLATEASUBTYPE,t1.TIPO_DOKUSI TIPODOKUSI,t1.SUBTIPO_DOKUSI SUBTIPODOKUSI,t1.ASUNTO_DOCUMENTAL ASUNTODOCUMENTAL,t1.TASK_ID TASKID,t1.SITUACION_ID SITUACIONID,t1.STATUS STATUS,t1.ETIQUETA_FECHA ETIQUETAFECHA,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.INSTANCE_ID INSTANCEID,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		query.append("FROM FECHAS_EXPEDIENTE t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(fechasexpediente);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query,
					FechasExpedienteDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<FechasExpediente>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the FechasExpediente table using like.
	 *
	 * @param fechasexpediente
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<FechasExpediente>
	 */
	@Transactional(readOnly = true)
	public List<FechasExpediente> findAllLike(FechasExpediente fechasexpediente, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_FECHA_EXPEDIENTE IDFECHAEXPEDIENTE,t1.TIPO_EVENTO TIPOEVENTO,t1.PLATEA_TYPE PLATEATYPE,t1.PLATEA_SUBTYPE PLATEASUBTYPE,t1.TIPO_DOKUSI TIPODOKUSI,t1.SUBTIPO_DOKUSI SUBTIPODOKUSI,t1.ASUNTO_DOCUMENTAL ASUNTODOCUMENTAL,t1.TASK_ID TASKID,t1.SITUACION_ID SITUACIONID,t1.STATUS STATUS,t1.ETIQUETA_FECHA ETIQUETAFECHA,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.INSTANCE_ID INSTANCEID,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		query.append("FROM FECHAS_EXPEDIENTE t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(fechasexpediente, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query,
					FechasExpedienteDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<FechasExpediente>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the FechasExpediente table.
	 *
	 * @param fechasexpediente
	 *            FechasExpediente
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(FechasExpediente fechasexpediente) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM FECHAS_EXPEDIENTE t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(fechasexpediente);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the FechasExpediente table using like.
	 *
	 * @param fechasexpediente
	 *            FechasExpediente
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(FechasExpediente fechasexpediente, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM FECHAS_EXPEDIENTE t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(fechasexpediente, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of FechasExpediente selected for rup_table
	 *
	 * @param fechasexpediente
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<FechasExpediente>>
	 */
	@Override
	public List<TableRowDto<FechasExpediente>> reorderSelection(FechasExpediente fechasexpediente,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_FECHA_EXPEDIENTE IDFECHAEXPEDIENTE,t1.TIPO_EVENTO TIPOEVENTO,t1.PLATEA_TYPE PLATEATYPE,t1.PLATEA_SUBTYPE PLATEASUBTYPE,t1.TIPO_DOKUSI TIPODOKUSI,t1.SUBTIPO_DOKUSI SUBTIPODOKUSI,t1.ASUNTO_DOCUMENTAL ASUNTODOCUMENTAL,t1.TASK_ID TASKID,t1.SITUACION_ID SITUACIONID,t1.STATUS STATUS,t1.ETIQUETA_FECHA ETIQUETAFECHA,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.INSTANCE_ID INSTANCEID,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		query.append("FROM FECHAS_EXPEDIENTE t1 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(fechasexpediente, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto,
				FechasExpediente.class, filterParamList, "ID_FECHA_EXPEDIENTE");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<FechasExpediente>(this.rwMapPK, jqGridRequestDto),
				filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
	 *
	 * @param filterParams
	 *            FechasExpediente
	 * @param searchParams
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<FechasExpediente>>
	 */
	@Override
	public List<TableRowDto<FechasExpediente>> search(FechasExpediente filterParams, FechasExpediente searchParams,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_FECHA_EXPEDIENTE IDFECHAEXPEDIENTE,t1.TIPO_EVENTO TIPOEVENTO,t1.PLATEA_TYPE PLATEATYPE,t1.PLATEA_SUBTYPE PLATEASUBTYPE,t1.TIPO_DOKUSI TIPODOKUSI,t1.SUBTIPO_DOKUSI SUBTIPODOKUSI,t1.ASUNTO_DOCUMENTAL ASUNTODOCUMENTAL,t1.TASK_ID TASKID,t1.SITUACION_ID SITUACIONID,t1.STATUS STATUS,t1.ETIQUETA_FECHA ETIQUETAFECHA,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.INSTANCE_ID INSTANCEID,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		query.append("FROM FECHAS_EXPEDIENTE t1 ");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto,
				FechasExpediente.class, filterParamList, searchSQL, searchParamList, from_alias, "ID_FECHA_EXPEDIENTE");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<FechasExpediente>(this.rwMapPK, jqGridRequestDto),
				filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
	 *
	 * @param filterfechasexpediente
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	@Override
	public void removeMultiple(FechasExpediente filterfechasexpediente, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_FECHA_EXPEDIENTE IDFECHAEXPEDIENTE,t1.TIPO_EVENTO TIPOEVENTO,t1.PLATEA_TYPE PLATEATYPE,t1.PLATEA_SUBTYPE PLATEASUBTYPE,t1.TIPO_DOKUSI TIPODOKUSI,t1.SUBTIPO_DOKUSI SUBTIPODOKUSI,t1.ASUNTO_DOCUMENTAL ASUNTODOCUMENTAL,t1.TASK_ID TASKID,t1.SITUACION_ID SITUACIONID,t1.STATUS STATUS,t1.ETIQUETA_FECHA ETIQUETAFECHA,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.INSTANCE_ID INSTANCEID,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		query.append("FROM FECHAS_EXPEDIENTE t1 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filterfechasexpediente, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto,
				FechasExpediente.class, query, params, "ID_FECHA_EXPEDIENTE");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Filter in the hierarchical FechasExpediente table.
	 *
	 * @param filterFechasExpediente
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<FechasExpediente>>
	 */
	@Override
	public List<JerarquiaDto<FechasExpediente>> findAllLikeJerarquia(FechasExpediente filterFechasExpediente,
			JQGridRequestDto jqGridRequestDto) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_FECHA_EXPEDIENTE IDFECHAEXPEDIENTE,t1.TIPO_EVENTO TIPOEVENTO,t1.PLATEA_TYPE PLATEATYPE,t1.PLATEA_SUBTYPE PLATEASUBTYPE,t1.TIPO_DOKUSI TIPODOKUSI,t1.SUBTIPO_DOKUSI SUBTIPODOKUSI,t1.ASUNTO_DOCUMENTAL ASUNTODOCUMENTAL,t1.TASK_ID TASKID,t1.SITUACION_ID SITUACIONID,t1.STATUS STATUS,t1.ETIQUETA_FECHA ETIQUETAFECHA,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.INSTANCE_ID INSTANCEID,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		// StringBuilder from = new StringBuilder("FROM FECHAS_EXPEDIENTE t1 ");
		List<String> from = new ArrayList<String>();
		from.add("FECHAS_EXPEDIENTE");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterFechasExpediente, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID_FECHA_EXPEDIENTE", "ID_PADRE",
				"NOMBRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query,
		// mapaWhere, "ID", "ID_PADRE", "NOMBRE", from, from_alias, joins,
		// businessFilters, businessParams);

		// PAGINACION
		if (jqGridRequestDto != null) {
			query = JQGridManagerJerarquia.getPaginationQuery(jqGridRequestDto, query,
					FechasExpedienteDaoImpl.ORDER_BY_WHITE_LIST);
		}

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.query(query.toString(), this.rwMapJerarquia, params.toArray());
	}

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterFechasExpediente
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	@Override
	public Long findAllLikeCountJerarquia(FechasExpediente filterFechasExpediente, JQGridRequestDto jqGridRequestDto) {
		// FROM
		// StringBuilder from = new StringBuilder("FROM FECHAS_EXPEDIENTE t1 ");
		List<String> from = new ArrayList<String>();
		from.add("FECHAS_EXPEDIENTE");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterFechasExpediente, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID_FECHA_EXPEDIENTE",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterFechasExpediente
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<FechasExpediente>>
	 */
	@Override
	public List<TableRowDto<FechasExpediente>> findAllChild(FechasExpediente filterFechasExpediente,
			JQGridRequestDto jqGridRequestDto) {

		// FROM
		// StringBuilder from = new StringBuilder("FROM FECHAS_EXPEDIENTE t1 ");
		List<String> from = new ArrayList<String>();
		from.add("FECHAS_EXPEDIENTE");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterFechasExpediente, false);

		// MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere,
				"ID_FECHA_EXPEDIENTE", "ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(),
				new RowNumResultSetExtractor<FechasExpediente>(this.rwMapPK, "ID_FECHA_EXPEDIENTE"), params.toArray());
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the FechasExpediente entity
	 *
	 * @param fechasexpediente
	 *            FechasExpediente Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(FechasExpediente fechasexpediente) {

		StringBuilder where = new StringBuilder(FechasExpedienteDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (fechasexpediente != null && fechasexpediente.getIdFechaExpediente() != null) {
			where.append(" AND t1.ID_FECHA_EXPEDIENTE = ?");
			params.add(fechasexpediente.getIdFechaExpediente());
		}
		if (fechasexpediente != null && fechasexpediente.getTipoEvento() != null) {
			where.append(" AND t1.TIPO_EVENTO = ?");
			params.add(fechasexpediente.getTipoEvento());
		}
		if (fechasexpediente != null && fechasexpediente.getPlateaType() != null) {
			where.append(" AND t1.PLATEA_TYPE = ?");
			params.add(fechasexpediente.getPlateaType());
		}
		if (fechasexpediente != null && fechasexpediente.getPlateaSubtype() != null) {
			where.append(" AND t1.PLATEA_SUBTYPE = ?");
			params.add(fechasexpediente.getPlateaSubtype());
		}
		if (fechasexpediente != null && fechasexpediente.getTipoDokusi() != null) {
			where.append(" AND t1.TIPO_DOKUSI = ?");
			params.add(fechasexpediente.getTipoDokusi());
		}
		if (fechasexpediente != null && fechasexpediente.getSubtipoDokusi() != null) {
			where.append(" AND t1.SUBTIPO_DOKUSI = ?");
			params.add(fechasexpediente.getSubtipoDokusi());
		}
		if (fechasexpediente != null && fechasexpediente.getAsuntoDocumental() != null) {
			where.append(" AND t1.ASUNTO_DOCUMENTAL = ?");
			params.add(fechasexpediente.getAsuntoDocumental());
		}
		if (fechasexpediente != null && fechasexpediente.getTaskId() != null) {
			where.append(" AND t1.TASK_ID = ?");
			params.add(fechasexpediente.getTaskId());
		}
		if (fechasexpediente != null && fechasexpediente.getSituacionId() != null) {
			where.append(" AND t1.SITUACION_ID = ?");
			params.add(fechasexpediente.getSituacionId());
		}
		if (fechasexpediente != null && fechasexpediente.getStatus() != null) {
			where.append(" AND t1.STATUS = ?");
			params.add(fechasexpediente.getStatus());
		}
		if (fechasexpediente != null && fechasexpediente.getEtiquetaFecha() != null) {
			where.append(" AND t1.ETIQUETA_FECHA = ?");
			params.add(fechasexpediente.getEtiquetaFecha());
		}
		if (fechasexpediente != null && fechasexpediente.getDescripcionCastellano() != null) {
			where.append(" AND t1.DESCRIPCION_CASTELLANO = ?");
			params.add(fechasexpediente.getDescripcionCastellano());
		}
		if (fechasexpediente != null && fechasexpediente.getDescripcionEuskera() != null) {
			where.append(" AND t1.DESCRIPCION_EUSKERA = ?");
			params.add(fechasexpediente.getDescripcionEuskera());
		}
		if (fechasexpediente != null && fechasexpediente.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(fechasexpediente.getFechaRegistro());
		}
		if (fechasexpediente != null && fechasexpediente.getUsuarioRegistro() != null) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(fechasexpediente.getUsuarioRegistro());
		}
		if (fechasexpediente != null && fechasexpediente.getInstanceId() != null) {
			where.append(" AND t1.INSTANCE_ID = ?");
			params.add(fechasexpediente.getInstanceId());
		}
		if (fechasexpediente != null && fechasexpediente.getFechaUltmodificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(fechasexpediente.getFechaUltmodificacion());
		}
		if (fechasexpediente != null && fechasexpediente.getUsuarioUltmodificacion() != null) {
			where.append(" AND t1.USUARIO_ULTMODIFICACION = ?");
			params.add(fechasexpediente.getUsuarioUltmodificacion());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the FechasExpediente entity
	 *
	 * @param fechasexpediente
	 *            FechasExpediente Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(FechasExpediente fechasexpediente, Boolean startsWith) {

		StringBuilder where = new StringBuilder(FechasExpedienteDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (fechasexpediente != null && fechasexpediente.getIdFechaExpediente() != null) {
			where.append(" AND t1.ID_FECHA_EXPEDIENTE = ?");
			params.add(fechasexpediente.getIdFechaExpediente());
		}
		if (fechasexpediente != null && fechasexpediente.getTipoEvento() != null) {
			where.append(" AND UPPER(t1.TIPO_EVENTO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(fechasexpediente.getTipoEvento().toUpperCase() + "%");
			} else {
				params.add("%" + fechasexpediente.getTipoEvento().toUpperCase() + "%");
			}
			where.append(" AND t1.TIPO_EVENTO IS NOT NULL");
		}
		if (fechasexpediente != null && fechasexpediente.getPlateaType() != null) {
			where.append(" AND UPPER(t1.PLATEA_TYPE) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(fechasexpediente.getPlateaType().toUpperCase() + "%");
			} else {
				params.add("%" + fechasexpediente.getPlateaType().toUpperCase() + "%");
			}
			where.append(" AND t1.PLATEA_TYPE IS NOT NULL");
		}
		if (fechasexpediente != null && fechasexpediente.getPlateaSubtype() != null) {
			where.append(" AND UPPER(t1.PLATEA_SUBTYPE) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(fechasexpediente.getPlateaSubtype().toUpperCase() + "%");
			} else {
				params.add("%" + fechasexpediente.getPlateaSubtype().toUpperCase() + "%");
			}
			where.append(" AND t1.PLATEA_SUBTYPE IS NOT NULL");
		}
		if (fechasexpediente != null && fechasexpediente.getTipoDokusi() != null) {
			where.append(" AND UPPER(t1.TIPO_DOKUSI) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(fechasexpediente.getTipoDokusi().toUpperCase() + "%");
			} else {
				params.add("%" + fechasexpediente.getTipoDokusi().toUpperCase() + "%");
			}
			where.append(" AND t1.TIPO_DOKUSI IS NOT NULL");
		}
		if (fechasexpediente != null && fechasexpediente.getSubtipoDokusi() != null) {
			where.append(" AND UPPER(t1.SUBTIPO_DOKUSI) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(fechasexpediente.getSubtipoDokusi().toUpperCase() + "%");
			} else {
				params.add("%" + fechasexpediente.getSubtipoDokusi().toUpperCase() + "%");
			}
			where.append(" AND t1.SUBTIPO_DOKUSI IS NOT NULL");
		}
		if (fechasexpediente != null && fechasexpediente.getAsuntoDocumental() != null) {
			where.append(" AND UPPER(t1.ASUNTO_DOCUMENTAL) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(fechasexpediente.getAsuntoDocumental().toUpperCase() + "%");
			} else {
				params.add("%" + fechasexpediente.getAsuntoDocumental().toUpperCase() + "%");
			}
			where.append(" AND t1.ASUNTO_DOCUMENTAL IS NOT NULL");
		}
		if (fechasexpediente != null && fechasexpediente.getTaskId() != null) {
			where.append(" AND UPPER(t1.TASK_ID) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(fechasexpediente.getTaskId().toUpperCase() + "%");
			} else {
				params.add("%" + fechasexpediente.getTaskId().toUpperCase() + "%");
			}
			where.append(" AND t1.TASK_ID IS NOT NULL");
		}
		if (fechasexpediente != null && fechasexpediente.getSituacionId() != null) {
			where.append(" AND UPPER(t1.SITUACION_ID) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(fechasexpediente.getSituacionId().toUpperCase() + "%");
			} else {
				params.add("%" + fechasexpediente.getSituacionId().toUpperCase() + "%");
			}
			where.append(" AND t1.SITUACION_ID IS NOT NULL");
		}
		if (fechasexpediente != null && fechasexpediente.getStatus() != null) {
			where.append(" AND t1.STATUS = ?");
			params.add(fechasexpediente.getStatus());
		}
		if (fechasexpediente != null && fechasexpediente.getEtiquetaFecha() != null) {
			where.append(" AND UPPER(t1.ETIQUETA_FECHA) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(fechasexpediente.getEtiquetaFecha().toUpperCase() + "%");
			} else {
				params.add("%" + fechasexpediente.getEtiquetaFecha().toUpperCase() + "%");
			}
			where.append(" AND t1.ETIQUETA_FECHA IS NOT NULL");
		}
		if (fechasexpediente != null && fechasexpediente.getDescripcionCastellano() != null) {
			where.append(" AND UPPER(t1.DESCRIPCION_CASTELLANO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(fechasexpediente.getDescripcionCastellano().toUpperCase() + "%");
			} else {
				params.add("%" + fechasexpediente.getDescripcionCastellano().toUpperCase() + "%");
			}
			where.append(" AND t1.DESCRIPCION_CASTELLANO IS NOT NULL");
		}
		if (fechasexpediente != null && fechasexpediente.getDescripcionEuskera() != null) {
			where.append(" AND UPPER(t1.DESCRIPCION_EUSKERA) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(fechasexpediente.getDescripcionEuskera().toUpperCase() + "%");
			} else {
				params.add("%" + fechasexpediente.getDescripcionEuskera().toUpperCase() + "%");
			}
			where.append(" AND t1.DESCRIPCION_EUSKERA IS NOT NULL");
		}
		if (fechasexpediente != null && fechasexpediente.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(fechasexpediente.getFechaRegistro());
		}
		if (fechasexpediente != null && fechasexpediente.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(fechasexpediente.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + fechasexpediente.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}
		if (fechasexpediente != null && fechasexpediente.getInstanceId() != null) {
			where.append(" AND t1.INSTANCE_ID = ?");
			params.add(fechasexpediente.getInstanceId());
		}
		if (fechasexpediente != null && fechasexpediente.getFechaUltmodificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(fechasexpediente.getFechaUltmodificacion());
		}
		if (fechasexpediente != null && fechasexpediente.getUsuarioUltmodificacion() != null) {
			where.append(" AND UPPER(t1.USUARIO_ULTMODIFICACION) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(fechasexpediente.getUsuarioUltmodificacion().toUpperCase() + "%");
			} else {
				params.add("%" + fechasexpediente.getUsuarioUltmodificacion().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_ULTMODIFICACION IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
