package com.ejie.ab61.dao.mensajesError;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.model.mensajesError.MensajesError;
import com.ejie.ab61.model.solicitudes.MensajesSolicitud;

/**
 * MensajesErrorDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class MensajesErrorDaoImpl implements MensajesErrorDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDMENSAJEERROR", "IDMENSAJE", "FOLDERID",
			"FECHAREGISTRO" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<MensajesError> rwMap = new RowMapper<MensajesError>() {
		public MensajesError mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			MensajesError mensajesError = new MensajesError();
			MensajesSolicitud mensajeSolicitud = new MensajesSolicitud();
			mensajeSolicitud.setIdMensaje(resultSet.getInt("IDMENSAJE"));
			mensajesError.setIdMensajeError(resultSet.getInt("IDMENSAJEERROR"));
			mensajesError.setMensajesSolicitud(mensajeSolicitud);
			mensajesError.setFolderId(resultSet.getString("FOLDERID"));
			mensajesError.setFechaRegistro(resultSet.getDate("FECHAREGISTRO"));
			return mensajesError;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/**
	 * Inserts a single row in the MensajesError table.
	 *
	 * @param mensajesError
	 *            MensajesError
	 * @return MensajesError
	 */
	public MensajesError add(MensajesError mensajesError) {
		String query = "INSERT INTO MENSAJES_ERROR (ID_MENSAJE_ERROR,ID_MENSAJE,FOLDER_ID,FECHA_REGISTRO) VALUES (S_AB6132T00.NEXTVAL,?,?,?)";
		this.jdbcTemplate.update(query, mensajesError.getMensajesSolicitud().getIdMensaje(),
				mensajesError.getFolderId(), mensajesError.getFechaRegistro());
		return mensajesError;
	}

	/**
	 * Finds a single row in the MensajesError table.
	 *
	 * @param mensajesError
	 *            MensajesError
	 * @return MensajesError
	 */
	@Transactional(readOnly = true)
	public MensajesError find(MensajesError mensajesError) {
		String query = "SELECT t1.ID_MENSAJE_ERROR IDMENSAJEERROR, t1.ID_MENSAJE IDMENSAJE, t1.FOLDER_ID FOLDERID, t1.FECHA_REGISTRO FECHAREGISTRO  FROM MENSAJES_ERROR t1  WHERE t1.FOLDER_ID = ?  ";

		List<MensajesError> mensajesErrorList = this.jdbcTemplate.query(query, this.rwMap, mensajesError.getFolderId());
		return (MensajesError) DataAccessUtils.uniqueResult(mensajesErrorList);
	}

	/**
	 * Removes a single row in the MensajesError table.
	 *
	 * @param mensajesError
	 *            MensajesError
	 * @return
	 */
	public void remove(MensajesError mensajesError) {
		String query = "DELETE FROM MENSAJES_ERROR WHERE ID_MENSAJE_ERROR=?";
		this.jdbcTemplate.update(query, mensajesError.getIdMensajeError());
	}

	public static final int STRING_BUILDER_INIT = 4096;

}
