package com.ejie.ab61.dao.plantillas;

import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;

import com.ejie.ab61.model.plantillas.Documentos;
import com.ejie.ab61.model.usuario.Usuario;

/**
 * DocumentosDao  
 * 
 *  
 */

public interface DocumentosDao {

	/**
	 * Inserts a single row in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @return Documentos
	 */
	Documentos add(Documentos documentos);

	/**
	 * Updates a single row in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @return Documentos
	 */
	Documentos update(Documentos documentos);

	/**
	 * Finds a single row in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @return Documentos
	 */
	Documentos find(Documentos documentos);

	/**
	 * Removes a single row in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 */
	void remove(Documentos documentos);

	void copiarFicheros(Documentos documentos, Usuario usuario);

	/**
	 * Finds a List of rows in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List
	 */
	List<Documentos> findAll(Documentos documentos, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds rows in the Documentos table using like.
	 *
	 * @param documentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List
	 */
	List<Documentos> findAllLike(Documentos documentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Filter in the Documentos table.
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	void removeMultiple(Documentos filterDocumentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Counts rows in the Documentos table using like.
	 *
	 * @param documentos
	 *            Documentos
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	Long findAllLikeCount(Documentos documentos, Boolean startsWith);

	/**
	 * Counts rows in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @return Long
	 */
	Long findAllCount(Documentos documentos);

	/**
	 * Reorder selection.
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Documentos>>
	 */
	List<TableRowDto<Documentos>> reorderSelection(Documentos filterDocumentos, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith);

	/**
	 * Searches in the Usuario table.
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param searchDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Documentos>>
	 */
	List<TableRowDto<Documentos>> search(Documentos filterDocumentos, Documentos searchDocumentos,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Filter in the hierarchical Documentos table.
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<Documentos>>
	 */
	List<JerarquiaDto<Documentos>> findAllLikeJerarquia(Documentos filterDocumentos, JQGridRequestDto jqGridRequestDto);

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	Long findAllLikeCountJerarquia(Documentos filterDocumentos, JQGridRequestDto jqGridRequestDto);

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<Documentos>>
	 */
	List<TableRowDto<Documentos>> findAllChild(Documentos filterDocumentos, JQGridRequestDto jqGridRequestDto);

	Documentos borrarFichero(Documentos documento);

	Documentos updateSinFichero(Documentos documento);

	List<Documentos> findAllLikeDocumentosPorDireccion(Documentos documentos, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith, Integer direccion);

	/**
	 * Counts rows in the Documentos table using like.
	 *
	 * @param documentos
	 *            Documentos
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	Long findAllLikeCountDocumentosPorDireccion(Documentos documentos, Boolean startsWith, Integer direccion);

	List<Documentos> findAllPorDireccion(Documentos documentos, JQGridRequestDto jqGridRequestDto, Integer direccion);
}
