package com.ejie.ab61.dao.plantillas;

import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.sql.DataSource;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.comun.Constantes;
import com.ejie.ab61.model.plantillas.Documentos;
import com.ejie.ab61.model.usuario.Usuario;

/**
 * DocumentosDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class DocumentosDaoImpl implements DocumentosDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDDOCUMENTO", "NOMBREDOCUMENTO", "NOMBREARCHIVO",
			"ARCHIVO", "OBSERVACIONES", "FECHAREGISTRO", "USUARIOREGISTRO", "FECHAULTMODIFICACION",
			"USUARIOULTMODIFICACION", "RELPLACAM", "PLANTILLA" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<Documentos> rwMap = new RowMapper<Documentos>() {
		public Documentos mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Documentos(resultSet.getInt("IDDOCUMENTO"), resultSet.getString("NOMBREDOCUMENTO"),
					resultSet.getString("NOMBREARCHIVO"), resultSet.getBlob("ARCHIVO"),
					resultSet.getString("OBSERVACIONES"), resultSet.getDate("FECHAREGISTRO"),
					resultSet.getString("USUARIOREGISTRO"), resultSet.getDate("FECHAULTMODIFICACION"),
					resultSet.getString("USUARIOULTMODIFICACION"));
		}
	};

	private RowMapper<Documentos> rwMap2 = new RowMapper<Documentos>() {
		public Documentos mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Documentos(resultSet.getInt("IDDOCUMENTO"), resultSet.getString("NOMBREDOCUMENTO"),
					resultSet.getString("NOMBREARCHIVO"), resultSet.getBlob("ARCHIVO"),
					resultSet.getString("OBSERVACIONES"), resultSet.getDate("FECHAREGISTRO"),
					resultSet.getString("USUARIOREGISTRO"), resultSet.getDate("FECHAULTMODIFICACION"),
					resultSet.getString("USUARIOULTMODIFICACION"), resultSet.getInt("RELPLACAM"),
					resultSet.getInt("PLANTILLA"));
		}
	};

	private RowMapper<Documentos> rwMapPK = new RowMapper<Documentos>() {
		public Documentos mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Documentos(resultSet.getInt("IDDOCUMENTO"));
		}
	};

	/**
	 * Rowmapper para Jerarquía
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	private RowMapper<JerarquiaDto<Documentos>> rwMapJerarquia = new RowMapper<JerarquiaDto<Documentos>>() {
		public JerarquiaDto<Documentos> mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			Documentos documentos = new Documentos(resultSet.getInt("IDDOCUMENTO"),
					resultSet.getString("NOMBREDOCUMENTO"), resultSet.getString("NOMBREARCHIVO"),
					resultSet.getBlob("ARCHIVO"), resultSet.getString("OBSERVACIONES"),
					resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO"),
					resultSet.getDate("FECHAULTMODIFICACION"), resultSet.getString("USUARIOULTMODIFICACION"));

			JerarquiaDto<Documentos> jerarquia = new JerarquiaDto<Documentos>();
			jerarquia.setModel(documentos);
			jerarquia.setLevel(resultSet.getBigDecimal("LEVEL").intValue());
			jerarquia.setParentNodes(resultSet.getString("PARENTNODES"));
			jerarquia.setIsLeaf(Boolean.parseBoolean(resultSet.getString("ISLEAF")));
			jerarquia.setFilter(Boolean.parseBoolean(resultSet.getString("FILTER")));
			return jerarquia;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @return Documentos
	 */
	public Documentos add(Documentos documentos) {
		documentos.setIdDocumento(
				this.jdbcTemplate.queryForObject("SELECT S_AB6139T00.NEXTVAL FROM DUAL", Integer.class));

		String query = "INSERT INTO DOCUMENTOS (ID_DOCUMENTO, NOMBRE_DOCUMENTO, NOMBRE_ARCHIVO, ARCHIVO, OBSERVACIONES, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (?,?,?,?,?,?,?)";
		this.jdbcTemplate.update(query, documentos.getIdDocumento(), documentos.getNombreDocumento(),
				documentos.getNombreArchivo(), documentos.getFicheroArray(), documentos.getObservaciones(),
				documentos.getFechaRegistro(), documentos.getUsuarioRegistro());
		return documentos;
	}

	/**
	 * Updates a single row in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @return Documentos
	 */
	public Documentos update(Documentos documentos) {
		String query = "UPDATE DOCUMENTOS SET NOMBRE_DOCUMENTO=?, NOMBRE_ARCHIVO=?, ARCHIVO=?, OBSERVACIONES=?, FECHA_ULTMODIFICACION=?, USUARIO_ULTMODIFICACION=? WHERE ID_DOCUMENTO=?";
		this.jdbcTemplate.update(query, documentos.getNombreDocumento(), documentos.getNombreArchivo(),
				documentos.getFicheroArray(), documentos.getObservaciones(), documentos.getFechaUltmodificacion(),
				documentos.getUsuarioUltmodificacion(), documentos.getIdDocumento());
		return documentos;
	}

	@Override
	public Documentos updateSinFichero(Documentos documento) {
		String query = "UPDATE DOCUMENTOS SET NOMBRE_DOCUMENTO=?, NOMBRE_ARCHIVO=?, OBSERVACIONES=?, FECHA_ULTMODIFICACION=?, USUARIO_ULTMODIFICACION=? WHERE ID_DOCUMENTO=?";
		this.jdbcTemplate.update(query, documento.getNombreDocumento(), documento.getNombreArchivo(),
				documento.getObservaciones(), documento.getFechaUltmodificacion(),
				documento.getUsuarioUltmodificacion(), documento.getIdDocumento());
		return documento;
	}

	/**
	 * Finds a single row in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @return Documentos
	 */
	@Transactional(readOnly = true)
	public Documentos find(Documentos documentos) {
		String query = "SELECT t1.ID_DOCUMENTO IDDOCUMENTO, t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO, t1.NOMBRE_ARCHIVO NOMBREARCHIVO, t1.ARCHIVO ARCHIVO, t1.OBSERVACIONES OBSERVACIONES, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO, t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION, t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION FROM DOCUMENTOS t1 WHERE t1.ID_DOCUMENTO = ?";

		List<Documentos> documentosList = this.jdbcTemplate.query(query, this.rwMap, documentos.getIdDocumento());
		return (Documentos) DataAccessUtils.uniqueResult(documentosList);
	}

	/**
	 * Removes a single row in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @return
	 */
	public void remove(Documentos documentos) {
		String query = "UPDATE DOCUMENTOS SET FECHA_BAJA=SYSDATE WHERE ID_DOCUMENTO=?";
		this.jdbcTemplate.update(query, documentos.getIdDocumento());
	}

	public void copiarFicheros(Documentos documentos, Usuario usuario) {
		String querySelect = "SELECT t1.ID_DOCUMENTO IDDOCUMENTO, t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO, t1.NOMBRE_ARCHIVO NOMBREARCHIVO, t1.ARCHIVO ARCHIVO, t1.OBSERVACIONES OBSERVACIONES, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO, t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION, t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION FROM DOCUMENTOS t1 WHERE t1.ID_DOCUMENTO = ?";
		List<Documentos> documentosList = this.jdbcTemplate.query(querySelect, this.rwMap, documentos.getIdDocumento());
		Documentos documento = (Documentos) DataAccessUtils.uniqueResult(documentosList);

		String queryCount = "SELECT COUNT(*) FROM DOC_MODIFICADOS t1 WHERE t1.ID_DOCUMENTO = ?";
		Long numeroDeRegistros = this.jdbcTemplate.queryForLong(queryCount.toString(), documento.getIdDocumento());

		if (numeroDeRegistros == Constantes.MAXIMO_DOC_MODIFICADOS) {
			String queryDelete = "DELETE FROM DOC_MODIFICADOS WHERE ID_DOCUMENTO = ? AND FECHA_REGISTRO = (SELECT MIN(FECHA_REGISTRO) FROM DOC_MODIFICADOS WHERE ID_DOCUMENTO = ?)";
			this.jdbcTemplate.update(queryDelete, documento.getIdDocumento(), documento.getIdDocumento());
		}

		String queryCopy = "INSERT INTO DOC_MODIFICADOS (ID_DOC_MODIFICADO, ID_DOCUMENTO, NOMBRE_ARCHIVO, ARCHIVO, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (AB61.S_AB6104T00.NEXTVAL,?,?,?,sysdate,?)";
		this.jdbcTemplate.update(queryCopy, documento.getIdDocumento(), documento.getNombreArchivo(),
				documento.getArchivo(), usuario.getNombreUsuario());
	}

	/**
	 * Finds a list of rows in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Documentos>
	 */
	@Transactional(readOnly = true)
	public List<Documentos> findAll(Documentos documentos, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_DOCUMENTO IDDOCUMENTO,t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO,t1.NOMBRE_ARCHIVO NOMBREARCHIVO,t1.ARCHIVO ARCHIVO,t1.OBSERVACIONES OBSERVACIONES,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		query.append("FROM DOCUMENTOS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(documentos);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.FECHA_BAJA IS NULL");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, DocumentosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Documentos>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds a list of rows in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Documentos>
	 */
	@Transactional(readOnly = true)
	public List<Documentos> findAllPorDireccion(Documentos documentos, JQGridRequestDto jqGridRequestDto,
			Integer direccion) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_DOCUMENTO IDDOCUMENTO,t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO,t1.NOMBRE_ARCHIVO NOMBREARCHIVO,t1.ARCHIVO ARCHIVO,t1.OBSERVACIONES OBSERVACIONES,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		query.append("FROM DOCUMENTOS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(documentos);
		StringBuilder where = new StringBuilder(
				" WHERE 1=1 AND t1.FECHA_BAJA IS NULL AND t1.ID_DOCUMENTO IN (SELECT t1.ID_DOCUMENTO FROM DOCUMENTOS t1, PLANTILLAS t5, PROCEDIMIENTOS t6 WHERE 1=1 AND t1.ID_DOCUMENTO = t5.ID_DOCUMENTO AND t5.ID_PROCEDIMIENTO = t6.ID_PROCEDIMIENTO AND t6.ID_DIRECCION = "
						+ direccion
						+ " MINUS (SELECT t7.ID_DOCUMENTO FROM DOCUMENTOS t7, PLANTILLAS t10, PROCEDIMIENTOS t11 WHERE t7.ID_DOCUMENTO = t10.ID_DOCUMENTO AND t10.ID_PROCEDIMIENTO = t11.ID_PROCEDIMIENTO AND t11.ID_DIRECCION != "
						+ direccion + " ))");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, DocumentosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Documentos>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the Documentos table using like.
	 *
	 * @param documentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Documentos>
	 */
	@Transactional(readOnly = true)
	public List<Documentos> findAllLike(Documentos documentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_DOCUMENTO IDDOCUMENTO,t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO,t1.NOMBRE_ARCHIVO NOMBREARCHIVO,t1.ARCHIVO ARCHIVO,t1.OBSERVACIONES OBSERVACIONES,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION,(SELECT COUNT(t3.ID_CAMPO) FROM REL_PLA_CAM t3 WHERE t1.ID_DOCUMENTO = t3.ID_DOCUMENTO) RELPLACAM,(SELECT COUNT(t4.ID_PLANTILLA) FROM PLANTILLAS t4 WHERE t1.ID_DOCUMENTO = t4.ID_DOCUMENTO) PLANTILLA, t1.FECHA_BAJA ");
		query.append("FROM DOCUMENTOS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(documentos, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.FECHA_BAJA IS NULL ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, DocumentosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Documentos>) this.jdbcTemplate.query(query.toString(), this.rwMap2, params.toArray());
	}

	/**
	 * Finds rows in the Documentos table using like.
	 *
	 * @param documentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Documentos>
	 */
	@Transactional(readOnly = true)
	public List<Documentos> findAllLikeDocumentosPorDireccion(Documentos documentos, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith, Integer direccion) {
		StringBuilder query = new StringBuilder(
				"(SELECT t0.ID_DOCUMENTO IDDOCUMENTO, t0.NOMBRE_DOCUMENTO NOMBREDOCUMENTO, t0.NOMBRE_ARCHIVO NOMBREARCHIVO, t0.ARCHIVO ARCHIVO, t0.OBSERVACIONES OBSERVACIONES, t0.FECHA_REGISTRO FECHAREGISTRO, t0.USUARIO_REGISTRO USUARIOREGISTRO, t0.FECHA_ULTMODIFICACION FECHAULTMODIFICACION, t0.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION, (SELECT COUNT(t3.ID_CAMPO) FROM REL_PLA_CAM t3 WHERE t0.ID_DOCUMENTO = t3.ID_DOCUMENTO) RELPLACAM, (SELECT COUNT(t4.ID_PLANTILLA) FROM PLANTILLAS t4 WHERE t0.ID_DOCUMENTO = t4.ID_DOCUMENTO ) PLANTILLA ");
		query.append("FROM DOCUMENTOS t0");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMapPorDireccion(documentos, startsWith);
		StringBuilder where = new StringBuilder(
				" WHERE 1=1 AND t0.ID_DOCUMENTO in(SELECT t1.ID_DOCUMENTO FROM DOCUMENTOS t1, PLANTILLAS t5, PROCEDIMIENTOS t6 WHERE 1=1 AND t1.ID_DOCUMENTO = t5.ID_DOCUMENTO AND t5.ID_PROCEDIMIENTO = t6.ID_PROCEDIMIENTO AND t6.ID_DIRECCION = "
						+ direccion
						+ " MINUS (SELECT t7.ID_DOCUMENTO FROM DOCUMENTOS t7, PLANTILLAS t10, PROCEDIMIENTOS t11 WHERE t7.ID_DOCUMENTO = t10.ID_DOCUMENTO AND t10.ID_PROCEDIMIENTO = t11.ID_PROCEDIMIENTO AND t11.ID_DIRECCION != "
						+ direccion + "))");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, DocumentosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Documentos>) this.jdbcTemplate.query(query.toString(), this.rwMap2, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(Documentos documentos) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM DOCUMENTOS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(documentos);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the Documentos table using like.
	 *
	 * @param documentos
	 *            Documentos
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(Documentos documentos, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM DOCUMENTOS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(documentos, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.FECHA_BAJA IS NULL ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the Documentos table using like.
	 *
	 * @param documentos
	 *            Documentos
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCountDocumentosPorDireccion(Documentos documentos, Boolean startsWith, Integer direccion) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM DOCUMENTOS t0");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMapPorDireccionCount(documentos, startsWith);
		StringBuilder where = new StringBuilder(
				" WHERE 1=1 AND t0.ID_DOCUMENTO in(SELECT t1.ID_DOCUMENTO FROM DOCUMENTOS t1, PLANTILLAS t5, PROCEDIMIENTOS t6 WHERE 1=1 AND t1.ID_DOCUMENTO = t5.ID_DOCUMENTO AND t5.ID_PROCEDIMIENTO = t6.ID_PROCEDIMIENTO AND t6.ID_DIRECCION = "
						+ direccion
						+ " MINUS (SELECT t7.ID_DOCUMENTO FROM DOCUMENTOS t7, PLANTILLAS t10, PROCEDIMIENTOS t11 WHERE t7.ID_DOCUMENTO = t10.ID_DOCUMENTO AND t10.ID_PROCEDIMIENTO = t11.ID_PROCEDIMIENTO AND t11.ID_DIRECCION != "
						+ direccion + "))");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of Documentos selected for rup_table
	 *
	 * @param documentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Documentos>>
	 */
	@Override
	public List<TableRowDto<Documentos>> reorderSelection(Documentos documentos, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_DOCUMENTO IDDOCUMENTO,t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO,t1.NOMBRE_ARCHIVO NOMBREARCHIVO,t1.ARCHIVO ARCHIVO,t1.OBSERVACIONES OBSERVACIONES,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		query.append("FROM DOCUMENTOS t1 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(documentos, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto, Documentos.class,
				filterParamList, "ID_DOCUMENTO");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Documentos>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
	 *
	 * @param filterParams
	 *            Documentos
	 * @param searchParams
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Documentos>>
	 */
	@Override
	public List<TableRowDto<Documentos>> search(Documentos filterParams, Documentos searchParams,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_DOCUMENTO IDDOCUMENTO,t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO,t1.NOMBRE_ARCHIVO NOMBREARCHIVO,t1.ARCHIVO ARCHIVO,t1.OBSERVACIONES OBSERVACIONES,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		query.append("FROM DOCUMENTOS t1 ");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto, Documentos.class,
				filterParamList, searchSQL, searchParamList, from_alias, "ID_DOCUMENTO");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Documentos>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
	 *
	 * @param filterdocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	@Override
	public void removeMultiple(Documentos filterdocumentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_DOCUMENTO IDDOCUMENTO,t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO,t1.NOMBRE_ARCHIVO NOMBREARCHIVO,t1.ARCHIVO ARCHIVO,t1.OBSERVACIONES OBSERVACIONES,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		query.append("FROM DOCUMENTOS t1 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filterdocumentos, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto, Documentos.class,
				query, params, "ID_DOCUMENTO");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Filter in the hierarchical Documentos table.
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<Documentos>>
	 */
	@Override
	public List<JerarquiaDto<Documentos>> findAllLikeJerarquia(Documentos filterDocumentos,
			JQGridRequestDto jqGridRequestDto) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_DOCUMENTO IDDOCUMENTO,t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO,t1.NOMBRE_ARCHIVO NOMBREARCHIVO,t1.ARCHIVO ARCHIVO,t1.OBSERVACIONES OBSERVACIONES, t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		// StringBuilder from = new StringBuilder("FROM DOCUMENTOS t1 ");
		List<String> from = new ArrayList<String>();
		from.add("DOCUMENTOS");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterDocumentos, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID_DOCUMENTO", "ID_PADRE",
				"NOMBRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query,
		// mapaWhere, "ID", "ID_PADRE", "NOMBRE", from, from_alias, joins,
		// businessFilters, businessParams);

		// PAGINACION
		if (jqGridRequestDto != null) {
			query = JQGridManagerJerarquia.getPaginationQuery(jqGridRequestDto, query,
					DocumentosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.query(query.toString(), this.rwMapJerarquia, params.toArray());
	}

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	@Override
	public Long findAllLikeCountJerarquia(Documentos filterDocumentos, JQGridRequestDto jqGridRequestDto) {
		// FROM
		// StringBuilder from = new StringBuilder("FROM DOCUMENTOS t1 ");
		List<String> from = new ArrayList<String>();
		from.add("DOCUMENTOS");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterDocumentos, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID_DOCUMENTO",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<Documentos>>
	 */
	@Override
	public List<TableRowDto<Documentos>> findAllChild(Documentos filterDocumentos, JQGridRequestDto jqGridRequestDto) {

		// FROM
		// StringBuilder from = new StringBuilder("FROM DOCUMENTOS t1 ");
		List<String> from = new ArrayList<String>();
		from.add("DOCUMENTOS");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterDocumentos, false);

		// MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID_DOCUMENTO",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(),
				new RowNumResultSetExtractor<Documentos>(this.rwMapPK, "ID_DOCUMENTO"), params.toArray());
	}

	public Documentos borrarFichero(Documentos documento) {
		String query = "UPDATE DOCUMENTOS SET ARCHIVO=?, NOMBRE_ARCHIVO=?, FECHA_ULTMODIFICACION=?, USUARIO_ULTMODIFICACION=? WHERE ID_DOCUMENTO=?";
		this.jdbcTemplate.update(query, documento.getFicheroArray(), "", new Date(),
				documento.getUsuarioUltmodificacion(), documento.getIdDocumento());
		return documento;
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Documentos entity
	 *
	 * @param documentos
	 *            Documentos Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(Documentos documentos) {

		StringBuilder where = new StringBuilder(DocumentosDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (documentos != null && documentos.getIdDocumento() != null) {
			where.append(" AND t1.ID_DOCUMENTO = ?");
			params.add(documentos.getIdDocumento());
		}
		if (documentos != null && documentos.getNombreDocumento() != null) {
			where.append(" AND t1.NOMBRE_DOCUMENTO = ?");
			params.add(documentos.getNombreDocumento());
		}
		if (documentos != null && documentos.getNombreArchivo() != null) {
			where.append(" AND t1.NOMBRE_ARCHIVO = ?");
			params.add(documentos.getNombreArchivo());
		}
		if (documentos != null && documentos.getObservaciones() != null) {
			where.append(" AND t1.OBSERVACIONES = ?");
			params.add(documentos.getObservaciones());
		}
		if (documentos != null && documentos.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(documentos.getFechaRegistro());
		}
		if (documentos != null && documentos.getUsuarioRegistro() != null) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(documentos.getUsuarioRegistro());
		}
		if (documentos != null && documentos.getFechaUltmodificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(documentos.getFechaUltmodificacion());
		}
		if (documentos != null && documentos.getUsuarioUltmodificacion() != null) {
			where.append(" AND t1.USUARIO_ULTMODIFICACION = ?");
			params.add(documentos.getUsuarioUltmodificacion());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Documentos entity
	 *
	 * @param documentos
	 *            Documentos Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(Documentos documentos, Boolean startsWith) {

		StringBuilder where = new StringBuilder(DocumentosDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (documentos != null && documentos.getIdDocumento() != null) {
			where.append(" AND t1.ID_DOCUMENTO = ?");
			params.add(documentos.getIdDocumento());
		}
		if (documentos != null && documentos.getNombreDocumento() != null) {
			where.append(" AND UPPER(t1.NOMBRE_DOCUMENTO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentos.getNombreDocumento().toUpperCase() + "%");
			} else {
				params.add("%" + documentos.getNombreDocumento().toUpperCase() + "%");
			}
			where.append(" AND t1.NOMBRE_DOCUMENTO IS NOT NULL");
		}
		if (documentos != null && documentos.getNombreArchivo() != null) {
			where.append(" AND UPPER(t1.NOMBRE_ARCHIVO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentos.getNombreArchivo().toUpperCase() + "%");
			} else {
				params.add("%" + documentos.getNombreArchivo().toUpperCase() + "%");
			}
			where.append(" AND t1.NOMBRE_ARCHIVO IS NOT NULL");
		}
		if (documentos != null && documentos.getObservaciones() != null) {
			where.append(" AND UPPER(t1.OBSERVACIONES) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentos.getObservaciones().toUpperCase() + "%");
			} else {
				params.add("%" + documentos.getObservaciones().toUpperCase() + "%");
			}
			where.append(" AND t1.OBSERVACIONES IS NOT NULL");
		}
		if (documentos != null && documentos.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(documentos.getFechaRegistro());
		}
		if (documentos != null && documentos.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentos.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + documentos.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}
		if (documentos != null && documentos.getFechaUltmodificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(documentos.getFechaUltmodificacion());
		}
		if (documentos != null && documentos.getUsuarioUltmodificacion() != null) {
			where.append(" AND UPPER(t1.USUARIO_ULTMODIFICACION) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentos.getUsuarioUltmodificacion().toUpperCase() + "%");
			} else {
				params.add("%" + documentos.getUsuarioUltmodificacion().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_ULTMODIFICACION IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMapPorDireccion(Documentos documentos, Boolean startsWith) {

		StringBuilder where = new StringBuilder(DocumentosDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (documentos != null && documentos.getIdDocumento() != null) {
			where.append(" AND t0.ID_DOCUMENTO = ?");
			params.add(documentos.getIdDocumento());
		}
		if (documentos != null && documentos.getNombreDocumento() != null) {
			where.append(" AND UPPER(t0.NOMBRE_DOCUMENTO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentos.getNombreDocumento().toUpperCase() + "%");
			} else {
				params.add("%" + documentos.getNombreDocumento().toUpperCase() + "%");
			}
			where.append(" AND t0.NOMBRE_DOCUMENTO IS NOT NULL");
		}
		if (documentos != null && documentos.getNombreArchivo() != null) {
			where.append(" AND UPPER(t0.NOMBRE_ARCHIVO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentos.getNombreArchivo().toUpperCase() + "%");
			} else {
				params.add("%" + documentos.getNombreArchivo().toUpperCase() + "%");
			}
			where.append(" AND t0.NOMBRE_ARCHIVO IS NOT NULL");
		}
		if (documentos != null && documentos.getObservaciones() != null) {
			where.append(" AND UPPER(t0.OBSERVACIONES) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentos.getObservaciones().toUpperCase() + "%");
			} else {
				params.add("%" + documentos.getObservaciones().toUpperCase() + "%");
			}
			where.append(" AND t0.OBSERVACIONES IS NOT NULL");
		}
		if (documentos != null && documentos.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(documentos.getFechaRegistro());
		}
		if (documentos != null && documentos.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentos.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + documentos.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}
		if (documentos != null && documentos.getFechaUltmodificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(documentos.getFechaUltmodificacion());
		}
		if (documentos != null && documentos.getUsuarioUltmodificacion() != null) {
			where.append(" AND UPPER(t1.USUARIO_ULTMODIFICACION) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentos.getUsuarioUltmodificacion().toUpperCase() + "%");
			} else {
				params.add("%" + documentos.getUsuarioUltmodificacion().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_ULTMODIFICACION IS NOT NULL");
		}

		where.append(")");

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	private Map<String, Object> getWhereLikeMapPorDireccionCount(Documentos documentos, Boolean startsWith) {

		StringBuilder where = new StringBuilder(DocumentosDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (documentos != null && documentos.getIdDocumento() != null) {
			where.append(" AND t0.ID_DOCUMENTO = ?");
			params.add(documentos.getIdDocumento());
		}
		if (documentos != null && documentos.getNombreDocumento() != null) {
			where.append(" AND UPPER(t0.NOMBRE_DOCUMENTO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentos.getNombreDocumento().toUpperCase() + "%");
			} else {
				params.add("%" + documentos.getNombreDocumento().toUpperCase() + "%");
			}
			where.append(" AND t0.NOMBRE_DOCUMENTO IS NOT NULL");
		}
		if (documentos != null && documentos.getNombreArchivo() != null) {
			where.append(" AND UPPER(t0.NOMBRE_ARCHIVO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentos.getNombreArchivo().toUpperCase() + "%");
			} else {
				params.add("%" + documentos.getNombreArchivo().toUpperCase() + "%");
			}
			where.append(" AND t0.NOMBRE_ARCHIVO IS NOT NULL");
		}
		if (documentos != null && documentos.getObservaciones() != null) {
			where.append(" AND UPPER(t0.OBSERVACIONES) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentos.getObservaciones().toUpperCase() + "%");
			} else {
				params.add("%" + documentos.getObservaciones().toUpperCase() + "%");
			}
			where.append(" AND t0.OBSERVACIONES IS NOT NULL");
		}
		if (documentos != null && documentos.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(documentos.getFechaRegistro());
		}
		if (documentos != null && documentos.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentos.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + documentos.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}
		if (documentos != null && documentos.getFechaUltmodificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(documentos.getFechaUltmodificacion());
		}
		if (documentos != null && documentos.getUsuarioUltmodificacion() != null) {
			where.append(" AND UPPER(t1.USUARIO_ULTMODIFICACION) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentos.getUsuarioUltmodificacion().toUpperCase() + "%");
			} else {
				params.add("%" + documentos.getUsuarioUltmodificacion().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_ULTMODIFICACION IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;

}
