package com.ejie.ab61.dao.plantillas;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.model.plantillas.ParametrosConfig;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * ParametrosConfigDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class ParametrosConfigDaoImpl implements ParametrosConfigDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDPARAMETROCONFIG", "DESCRIPCION", "VALOR",
			"FECHAREGISTRO", "USUARIOREGISTRO", "FECHAULTMODIFICACION", "USUARIOULTMODIFICACION" };

	private JdbcTemplate jdbcTemplate;

	private RowMapper<ParametrosConfig> rwMap = new RowMapper<ParametrosConfig>() {
		public ParametrosConfig mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new ParametrosConfig(resultSet.getInt("IDPARAMETROCONFIG"), resultSet.getString("DESCRIPCION"),
					resultSet.getString("VALOR"), resultSet.getString("OBSERVACIONES"),
					resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO"),
					resultSet.getDate("FECHAULTMODIFICACION"), resultSet.getString("USUARIOULTMODIFICACION"));
		}
	};

	private RowMapper<ParametrosConfig> rwMapPK = new RowMapper<ParametrosConfig>() {
		public ParametrosConfig mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new ParametrosConfig(resultSet.getInt("IDPARAMETROCONFIG"));
		}
	};

	private RowMapper<ParametrosConfig> rwMapByDescripcion = new RowMapper<ParametrosConfig>() {
		public ParametrosConfig mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			ParametrosConfig parametrosConfig = new ParametrosConfig();
			parametrosConfig.setValor(resultSet.getString("VALOR"));

			return parametrosConfig;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	public ParametrosConfig add(ParametrosConfig parametrosConfig) {
		String query = "INSERT INTO PARAMETROS_CONFIG (ID_PARAMETRO_CONFIG, DESCRIPCION, VALOR, OBSERVACIONES, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (S_AB6140T00.NEXTVAL,?,?,?,?,?)";
		this.jdbcTemplate.update(query, parametrosConfig.getDescripcion(), parametrosConfig.getValor(),
				parametrosConfig.getObservaciones(), parametrosConfig.getFechaRegistro(),
				parametrosConfig.getUsuarioRegistro());
		return parametrosConfig;
	}

	/**
	 * Updates a single row in the ParametrosConfig table.
	 * 
	 * @param parametrosConfig
	 *            ParametrosConfig
	 * @return ParametrosConfig
	 */
	public ParametrosConfig update(ParametrosConfig parametrosConfig) {
		String query = "UPDATE PARAMETROS_CONFIG SET DESCRIPCION=?, VALOR=?, OBSERVACIONES=?, FECHA_ULTMODIFICACION=?, USUARIO_ULTMODIFICACION=? WHERE ID_PARAMETRO_CONFIG=?";
		this.jdbcTemplate.update(query, parametrosConfig.getDescripcion(), parametrosConfig.getValor(),
				parametrosConfig.getObservaciones(), parametrosConfig.getFechaUltmodificacion(),
				parametrosConfig.getUsuarioUltmodificacion(), parametrosConfig.getIdParametroConfig());
		return parametrosConfig;
	}

	@Transactional(readOnly = true)
	public ParametrosConfig find(ParametrosConfig parametrosConfig) {
		String query = "SELECT t1.ID_PARAMETRO_CONFIG IDPARAMETROCONFIG, t1.DESCRIPCION DESCRIPCION, t1.VALOR VALOR,t1.OBSERVACIONES OBSERVACIONES, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO, t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION, t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION FROM PARAMETROS_CONFIG t1 WHERE t1.ID_PARAMETRO_CONFIG = ? ";

		List<ParametrosConfig> parametrosConfigList = this.jdbcTemplate.query(query, this.rwMap,
				parametrosConfig.getIdParametroConfig());
		return (ParametrosConfig) DataAccessUtils.uniqueResult(parametrosConfigList);
	}

	@Transactional(readOnly = true)
	public ParametrosConfig findByDescripcion(ParametrosConfig parametrosConfig) {
		String query = "SELECT VALOR AS VALOR FROM PARAMETROS_CONFIG WHERE  DESCRIPCION = ?";
		List<ParametrosConfig> parametrosConfigList = this.jdbcTemplate.query(query, this.rwMapByDescripcion,
				parametrosConfig.getDescripcion());

		return (ParametrosConfig) DataAccessUtils.uniqueResult(parametrosConfigList);

	}

	/**
	 * Removes a single row in the ParametrosConfig table.
	 * 
	 * @param parametrosConfig
	 *            ParametrosConfig
	 * @return
	 */
	public void remove(ParametrosConfig parametrosConfig) {
		String query = "DELETE FROM PARAMETROS_CONFIG WHERE ID_PARAMETRO_CONFIG=?";
		this.jdbcTemplate.update(query, parametrosConfig.getIdParametroConfig());
	}

	/**
	 * Finds rows in the ParametrosConfig table using like.
	 *
	 * @param parametrosConfig
	 *            ParametrosConfig
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<ParametrosConfig>
	 */
	@Transactional(readOnly = true)
	public List<ParametrosConfig> findAllLike(ParametrosConfig parametrosConfig, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_PARAMETRO_CONFIG IDPARAMETROCONFIG,t1.DESCRIPCION DESCRIPCION,t1.VALOR VALOR,t1.OBSERVACIONES OBSERVACIONES,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		query.append("FROM PARAMETROS_CONFIG t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(parametrosConfig, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query,
					ParametrosConfigDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<ParametrosConfig>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Counts rows in the ParametrosConfig table using like.
	 *
	 * @param parametrosConfig
	 *            ParametrosConfig
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(ParametrosConfig parametrosConfig, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM PARAMETROS_CONFIG t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(parametrosConfig, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of ParametrosConfig selected for rup_table
	 *
	 * @param parametrosConfig
	 *            ParametrosConfig
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<ParametrosConfig>>
	 */
	@Override
	public List<TableRowDto<ParametrosConfig>> reorderSelection(ParametrosConfig parametrosConfig,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_PARAMETRO_CONFIG IDPARAMETROCONFIG,t1.DESCRIPCION DESCRIPCION,t1.VALOR VALOR,t1.OBSERVACIONES OBSERVACIONES,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		query.append("FROM PARAMETROS_CONFIG t1 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(parametrosConfig, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto,
				ParametrosConfig.class, filterParamList, "ID_PARAMETROS_CONFIG");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<ParametrosConfig>(this.rwMapPK, jqGridRequestDto),
				filterParamList.toArray());
	}

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the ParametrosConfig entity
	 *
	 * @param parametrosConfig
	 *            ParametrosConfig Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(ParametrosConfig parametrosConfig, Boolean startsWith) {

		StringBuilder where = new StringBuilder(ParametrosConfigDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (parametrosConfig != null && parametrosConfig.getIdParametroConfig() != null) {
			where.append(" AND t1.ID_PARAMETRO_CONFIG = ?");
			params.add(parametrosConfig.getIdParametroConfig());
		}
		if (parametrosConfig != null && parametrosConfig.getDescripcion() != null) {
			where.append(" AND UPPER(t1.DESCRIPCION) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(parametrosConfig.getDescripcion().toUpperCase() + "%");
			} else {
				params.add("%" + parametrosConfig.getDescripcion().toUpperCase() + "%");
			}
			where.append(" AND t1.DESCRIPCION IS NOT NULL");
		}
		if (parametrosConfig != null && parametrosConfig.getValor() != null) {
			where.append(" AND t1.VALOR = ?");
			params.add(parametrosConfig.getValor());
		}
		if (parametrosConfig != null && parametrosConfig.getObservaciones() != null) {
			where.append(" AND UPPER(t1.OBSERVACIONES) like ? ESCAPE '\\'");
			if (startsWith) {
				params.add(parametrosConfig.getObservaciones().toUpperCase() + "%");
			} else {
				params.add("%" + parametrosConfig.getObservaciones().toUpperCase() + "%");
			}
			where.append(" AND OBSERVACIONES IS NOT NULL");
		}
		if (parametrosConfig != null && parametrosConfig.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(parametrosConfig.getFechaRegistro());
		}
		if (parametrosConfig != null && parametrosConfig.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(parametrosConfig.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + parametrosConfig.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}
		if (parametrosConfig != null && parametrosConfig.getFechaUltmodificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(parametrosConfig.getFechaUltmodificacion());
		}
		if (parametrosConfig != null && parametrosConfig.getUsuarioUltmodificacion() != null) {
			where.append(" AND UPPER(t1.USUARIO_ULTMODIFICACION) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(parametrosConfig.getUsuarioUltmodificacion().toUpperCase() + "%");
			} else {
				params.add("%" + parametrosConfig.getUsuarioUltmodificacion().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_ULTMODIFICACION IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;

}
