package com.ejie.ab61.dao.plantillas;

import java.util.List;

import com.ejie.ab61.model.plantillas.Plantillas;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * PlantillasDao  
 * 
 *  
 */

public interface PlantillasDao {

	/**
	 * Inserts a single row in the Plantillas table.
	 *
	 * @param plantillas
	 *            Plantillas
	 * @return Plantillas
	 */
	Plantillas add(Plantillas plantillas);

	/**
	 * Updates a single row in the Plantillas table.
	 *
	 * @param plantillas
	 *            Plantillas
	 * @return Plantillas
	 */
	Plantillas update(Plantillas plantillas);

	/**
	 * Finds a single row in the Plantillas table.
	 *
	 * @param plantillas
	 *            Plantillas
	 * @return Plantillas
	 */
	Plantillas find(Plantillas plantillas);

	/**
	 * Removes a single row in the Plantillas table.
	 *
	 * @param plantillas
	 *            Plantillas
	 */
	void remove(Plantillas plantillas);

	/**
	 * Finds a List of rows in the Plantillas table.
	 *
	 * @param plantillas
	 *            Plantillas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List
	 */
	List<Plantillas> findAll(Plantillas plantillas, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds rows in the Plantillas table using like.
	 *
	 * @param plantillas
	 *            Plantillas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List
	 */
	List<Plantillas> findAllLike(Plantillas plantillas, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Finds rows in the Plantillas para el mantenimeiento de plantillas.
	 *
	 * @param plantillas
	 *            Plantillas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List
	 */
	List<Plantillas> findMantenimientoPlantillas(Plantillas plantillas, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Filter in the Plantillas table.
	 *
	 * @param filterPlantillas
	 *            Plantillas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	void removeMultiple(Plantillas filterPlantillas, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Counts rows in the Plantillas table using like.
	 *
	 * @param plantillas
	 *            Plantillas
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	Long findAllLikeCount(Plantillas plantillas, Boolean startsWith);

	/**
	 * Counts rows in the Plantillas table using like.
	 *
	 * @param plantillas
	 *            Plantillas
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */

	Long findAllLikeCountManteniemiento(Plantillas plantillas, Boolean startsWith);

	/**
	 * Counts rows in the Plantillas table.
	 *
	 * @param plantillas
	 *            Plantillas
	 * @return Long
	 */
	Long findAllCount(Plantillas plantillas);

	/**
	 * Poner a null los id_documentos de plantiilas, segun el id_documento dado.
	 *
	 * @param idDocumento
	 *            Integer
	 */
	void updateIdDocumento(Integer idDocumento);

	/**
	 * metodo para saber cuantas plantillas tienen asociado el mismo id de
	 * documento
	 * 
	 * 
	 * @return count.
	 */
	Long obtenerNumDocumentosCampo(Integer idDocumento);

	/**
	 * Reorder selection.
	 *
	 * @param filterPlantillas
	 *            Plantillas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Plantillas>>
	 */
	List<TableRowDto<Plantillas>> reorderSelection(Plantillas filterPlantillas, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith);

	/**
	 * Searches in the Usuario table.
	 *
	 * @param filterPlantillas
	 *            Plantillas
	 * @param searchPlantillas
	 *            Plantillas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Plantillas>>
	 */
	List<TableRowDto<Plantillas>> search(Plantillas filterPlantillas, Plantillas searchPlantillas,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Filter in the hierarchical Plantillas table.
	 *
	 * @param filterPlantillas
	 *            Plantillas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<Plantillas>>
	 */
	List<JerarquiaDto<Plantillas>> findAllLikeJerarquia(Plantillas filterPlantillas, JQGridRequestDto jqGridRequestDto);

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterPlantillas
	 *            Plantillas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	Long findAllLikeCountJerarquia(Plantillas filterPlantillas, JQGridRequestDto jqGridRequestDto);

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterPlantillas
	 *            Plantillas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<Plantillas>>
	 */
	List<TableRowDto<Plantillas>> findAllChild(Plantillas filterPlantillas, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds rows in the Plantillas para el mantenimeiento de plantillas.
	 *
	 * @param plantillas
	 *            Plantillas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List
	 */
	List<Plantillas> findMantenimientoPlantillasPorDireccion(Plantillas plantillas, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith, Integer direccion);

	/**
	 * Counts rows in the Plantillas table using like.
	 *
	 * @param plantillas
	 *            Plantillas
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */

	Long findAllLikeCountManteniemientoPorDireccion(Plantillas plantillas, Boolean startsWith, Integer direccion);

}
