package com.ejie.ab61.dao.plantillas;

import com.ejie.ab61.model.plantillas.TipoCampo;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;

/**
 * TipoCampoDao  
 * 
 *  
 */

public interface TipoCampoDao {

	/**
	 * Inserts a single row in the TipoCampo table.
	 *
	 * @param tipocampo
	 *            TipoCampo
	 * @return TipoCampo
	 */
	TipoCampo add(TipoCampo tipocampo);

	/**
	 * Updates a single row in the TipoCampo table.
	 *
	 * @param tipocampo
	 *            TipoCampo
	 * @return TipoCampo
	 */
	TipoCampo update(TipoCampo tipocampo);

	/**
	 * Finds a single row in the TipoCampo table.
	 *
	 * @param tipocampo
	 *            TipoCampo
	 * @return TipoCampo
	 */
	TipoCampo find(TipoCampo tipocampo);

	/**
	 * Removes a single row in the TipoCampo table.
	 *
	 * @param tipocampo
	 *            TipoCampo
	 */
	void remove(TipoCampo tipocampo);

	/**
	 * Finds a List of rows in the TipoCampo table.
	 *
	 * @param tipocampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List
	 */
	List<TipoCampo> findAll(TipoCampo tipocampo, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds rows in the TipoCampo table using like.
	 *
	 * @param tipocampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List
	 */
	List<TipoCampo> findAllLike(TipoCampo tipocampo, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Filter in the TipoCampo table.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	void removeMultiple(TipoCampo filterTipoCampo, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Counts rows in the TipoCampo table using like.
	 *
	 * @param tipocampo
	 *            TipoCampo
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	Long findAllLikeCount(TipoCampo tipocampo, Boolean startsWith);

	/**
	 * Counts rows in the TipoCampo table.
	 *
	 * @param tipocampo
	 *            TipoCampo
	 * @return Long
	 */
	Long findAllCount(TipoCampo tipocampo);

	/**
	 * Reorder selection.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<TipoCampo>>
	 */
	List<TableRowDto<TipoCampo>> reorderSelection(TipoCampo filterTipoCampo, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith);

	/**
	 * Searches in the Usuario table.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param searchTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<TipoCampo>>
	 */
	List<TableRowDto<TipoCampo>> search(TipoCampo filterTipoCampo, TipoCampo searchTipoCampo,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Filter in the hierarchical TipoCampo table.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<TipoCampo>>
	 */
	List<JerarquiaDto<TipoCampo>> findAllLikeJerarquia(TipoCampo filterTipoCampo, JQGridRequestDto jqGridRequestDto);

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	Long findAllLikeCountJerarquia(TipoCampo filterTipoCampo, JQGridRequestDto jqGridRequestDto);

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<TipoCampo>>
	 */
	List<TableRowDto<TipoCampo>> findAllChild(TipoCampo filterTipoCampo, JQGridRequestDto jqGridRequestDto);

}
