package com.ejie.ab61.dao.procedimientos;

import com.ejie.ab61.model.procedimientos.Departamentos;
import com.ejie.ab61.model.procedimientos.Direcciones;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.sql.DataSource;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/**
 * DireccionesDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class DireccionesDaoImpl implements DireccionesDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDDIRECCION", "DESCRIPCIONCASTELLANO",
			"DESCRIPCIONEUSKERA", "FECHAREGISTRO", "USUARIOREGISTRO", "DEPIDDEPARTAMENTO", "DEPDESCRIPCIONCASTELLANO",
			"DEPDESCRIPCIONEUSKERA", "DEPFECHAREGISTRO", "DEPSUSUARIOREGISTRO" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<Direcciones> rwMap = new RowMapper<Direcciones>() {
		public Direcciones mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Direcciones(resultSet.getInt("IDDIRECCION"), resultSet.getString("DESCRIPCIONCASTELLANO"),
					resultSet.getString("DESCRIPCIONEUSKERA"), resultSet.getDate("FECHAREGISTRO"),
					resultSet.getString("USUARIOREGISTRO"),
					new Departamentos(resultSet.getInt("DEPIDDEPARTAMENTO"),
							resultSet.getString("DEPDESCRIPCIONCASTELLANO"),
							resultSet.getString("DEPDESCRIPCIONEUSKERA"), resultSet.getDate("DEPFECHAREGISTRO"),
							resultSet.getString("DEPSUSUARIOREGISTRO")));
		}
	};

	private RowMapper<Direcciones> rwMapPK = new RowMapper<Direcciones>() {
		public Direcciones mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Direcciones(resultSet.getInt("IDDIRECCION"));
		}
	};

	/**
	 * Rowmapper para Jerarquía
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	private RowMapper<JerarquiaDto<Direcciones>> rwMapJerarquia = new RowMapper<JerarquiaDto<Direcciones>>() {
		public JerarquiaDto<Direcciones> mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			Direcciones direcciones = new Direcciones(resultSet.getInt("IDDIRECCION"),
					resultSet.getString("DESCRIPCIONCASTELLANO"), resultSet.getString("DESCRIPCIONEUSKERA"),
					resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO"),
					new Departamentos(resultSet.getInt("DEPIDDEPARTAMENTO"),
							resultSet.getString("DEPDESCRIPCIONCASTELLANO"),
							resultSet.getString("DEPDESCRIPCIONEUSKERA"), resultSet.getDate("DEPFECHAREGISTRO"),
							resultSet.getString("DEPSUSUARIOREGISTRO")));

			JerarquiaDto<Direcciones> jerarquia = new JerarquiaDto<Direcciones>();
			jerarquia.setModel(direcciones);
			jerarquia.setLevel(resultSet.getBigDecimal("LEVEL").intValue());
			jerarquia.setParentNodes(resultSet.getString("PARENTNODES"));
			jerarquia.setIsLeaf(Boolean.parseBoolean(resultSet.getString("ISLEAF")));
			jerarquia.setFilter(Boolean.parseBoolean(resultSet.getString("FILTER")));
			return jerarquia;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the Direcciones table.
	 *
	 * @param direcciones
	 *            Direcciones
	 * @return Direcciones
	 */
	public Direcciones add(Direcciones direcciones) {
		String query = "INSERT INTO DIRECCIONES (ID_DIRECCION, ID_DEPARTAMENTO, DESCRIPCION_CASTELLANO, DESCRIPCION_EUSKERA, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (?,?,?,?,?,?)";
		Object getDepartamentosIdDepartamentoAux = null;
		if (direcciones.getDepartamentos() != null && direcciones.getDepartamentos().getIdDepartamento() != null) {
			getDepartamentosIdDepartamentoAux = direcciones.getDepartamentos().getIdDepartamento();
		}
		this.jdbcTemplate.update(query, direcciones.getIdDireccion(), getDepartamentosIdDepartamentoAux,
				direcciones.getDescripcionCastellano(), direcciones.getDescripcionEuskera(),
				direcciones.getFechaRegistro(), direcciones.getUsuarioRegistro());
		return direcciones;
	}

	/**
	 * Updates a single row in the Direcciones table.
	 *
	 * @param direcciones
	 *            Direcciones
	 * @return Direcciones
	 */
	public Direcciones update(Direcciones direcciones) {
		String query = "UPDATE DIRECCIONES SET ID_DEPARTAMENTO=?, DESCRIPCION_CASTELLANO=?, DESCRIPCION_EUSKERA=?, FECHA_REGISTRO=?, USUARIO_REGISTRO=? WHERE ID_DIRECCION=?";
		Object getDepartamentosIdDepartamentoAux = null;
		if (direcciones.getDepartamentos() != null && direcciones.getDepartamentos().getIdDepartamento() != null) {
			getDepartamentosIdDepartamentoAux = direcciones.getDepartamentos().getIdDepartamento();
		}
		this.jdbcTemplate.update(query, getDepartamentosIdDepartamentoAux, direcciones.getDescripcionCastellano(),
				direcciones.getDescripcionEuskera(), direcciones.getFechaRegistro(), direcciones.getUsuarioRegistro(),
				direcciones.getIdDireccion());
		return direcciones;
	}

	/**
	 * Finds a single row in the Direcciones table.
	 *
	 * @param direcciones
	 *            Direcciones
	 * @return Direcciones
	 */
	@Transactional(readOnly = true)
	public Direcciones find(Direcciones direcciones) {
		String query = "SELECT t1.ID_DIRECCION IDDIRECCION, t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO, t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO, t2.ID_DEPARTAMENTO DEPIDDEPARTAMENTO, t2.DESCRIPCION_CASTELLANO DEPDESCRIPCIONCASTELLANO, t2.DESCRIPCION_EUSKERA DEPDESCRIPCIONEUSKERA, t2.FECHA_REGISTRO DEPFECHAREGISTRO, t2.USUARIO_REGISTRO DEPSUSUARIOREGISTRO FROM DIRECCIONES t1 , DEPARTAMENTOS t2  WHERE t1.ID_DIRECCION = ?   AND t1.ID_DEPARTAMENTO= t2.ID_DEPARTAMENTO(+)";

		List<Direcciones> direccionesList = this.jdbcTemplate.query(query, this.rwMap, direcciones.getIdDireccion());
		return (Direcciones) DataAccessUtils.uniqueResult(direccionesList);
	}

	/**
	 * Removes a single row in the Direcciones table.
	 *
	 * @param direcciones
	 *            Direcciones
	 * @return
	 */
	public void remove(Direcciones direcciones) {
		String query = "DELETE FROM DIRECCIONES WHERE ID_DIRECCION=?";
		this.jdbcTemplate.update(query, direcciones.getIdDireccion());
	}

	/**
	 * Finds a list of rows in the Direcciones table.
	 *
	 * @param direcciones
	 *            Direcciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Direcciones>
	 */
	@Transactional(readOnly = true)
	public List<Direcciones> findAll(Direcciones direcciones, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_DIRECCION IDDIRECCION,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_DEPARTAMENTO DEPIDDEPARTAMENTO,t2.DESCRIPCION_CASTELLANO DEPDESCRIPCIONCASTELLANO,t2.DESCRIPCION_EUSKERA DEPDESCRIPCIONEUSKERA,t2.FECHA_REGISTRO DEPFECHAREGISTRO,t2.USUARIO_REGISTRO DEPSUSUARIOREGISTRO ");
		query.append("FROM DIRECCIONES t1 ,DEPARTAMENTOS t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(direcciones);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_DEPARTAMENTO= t2.ID_DEPARTAMENTO(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, DireccionesDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Direcciones>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the Direcciones table using like.
	 *
	 * @param direcciones
	 *            Direcciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Direcciones>
	 */
	@Transactional(readOnly = true)
	public List<Direcciones> findAllLike(Direcciones direcciones, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_DIRECCION IDDIRECCION,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_DEPARTAMENTO DEPIDDEPARTAMENTO,t2.DESCRIPCION_CASTELLANO DEPDESCRIPCIONCASTELLANO,t2.DESCRIPCION_EUSKERA DEPDESCRIPCIONEUSKERA,t2.FECHA_REGISTRO DEPFECHAREGISTRO,t2.USUARIO_REGISTRO DEPSUSUARIOREGISTRO ");
		query.append("FROM DIRECCIONES t1 ,DEPARTAMENTOS t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(direcciones, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_DEPARTAMENTO= t2.ID_DEPARTAMENTO(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, DireccionesDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Direcciones>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the Direcciones table.
	 *
	 * @param direcciones
	 *            Direcciones
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(Direcciones direcciones) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM DIRECCIONES t1 , DEPARTAMENTOS t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(direcciones);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_DEPARTAMENTO= t2.ID_DEPARTAMENTO(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the Direcciones table using like.
	 *
	 * @param direcciones
	 *            Direcciones
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(Direcciones direcciones, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM DIRECCIONES t1 ,DEPARTAMENTOS t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(direcciones, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_DEPARTAMENTO= t2.ID_DEPARTAMENTO(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of Direcciones selected for rup_table
	 *
	 * @param direcciones
	 *            Direcciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Direcciones>>
	 */
	@Override
	public List<TableRowDto<Direcciones>> reorderSelection(Direcciones direcciones, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_DIRECCION IDDIRECCION,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_DEPARTAMENTO DEPIDDEPARTAMENTO,t2.DESCRIPCION_CASTELLANO DEPDESCRIPCIONCASTELLANO,t2.DESCRIPCION_EUSKERA DEPDESCRIPCIONEUSKERA,t2.FECHA_REGISTRO DEPFECHAREGISTRO,t2.USUARIO_REGISTRO DEPSUSUARIOREGISTRO ");
		// FROM
		query.append("FROM DIRECCIONES t1 ,DEPARTAMENTOS t2 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(direcciones, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_DEPARTAMENTO= t2.ID_DEPARTAMENTO(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto, Direcciones.class,
				filterParamList, "ID_DIRECCION");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Direcciones>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
	 *
	 * @param filterParams
	 *            Direcciones
	 * @param searchParams
	 *            Direcciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Direcciones>>
	 */
	@Override
	public List<TableRowDto<Direcciones>> search(Direcciones filterParams, Direcciones searchParams,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_DIRECCION IDDIRECCION,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_DEPARTAMENTO DEPIDDEPARTAMENTO,t2.DESCRIPCION_CASTELLANO DEPDESCRIPCIONCASTELLANO,t2.DESCRIPCION_EUSKERA DEPDESCRIPCIONEUSKERA,t2.FECHA_REGISTRO DEPFECHAREGISTRO,t2.USUARIO_REGISTRO DEPSUSUARIOREGISTRO ");
		// FROM
		query.append("FROM DIRECCIONES t1 ,DEPARTAMENTOS t2 ");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_DEPARTAMENTO= t2.ID_DEPARTAMENTO(+) ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto, Direcciones.class,
				filterParamList, searchSQL, searchParamList, from_alias, "ID_DIRECCION");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Direcciones>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
	 *
	 * @param filterdirecciones
	 *            Direcciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	@Override
	public void removeMultiple(Direcciones filterdirecciones, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_DIRECCION IDDIRECCION,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_DEPARTAMENTO DEPIDDEPARTAMENTO,t2.DESCRIPCION_CASTELLANO DEPDESCRIPCIONCASTELLANO,t2.DESCRIPCION_EUSKERA DEPDESCRIPCIONEUSKERA,t2.FECHA_REGISTRO DEPFECHAREGISTRO,t2.USUARIO_REGISTRO DEPSUSUARIOREGISTRO ");
		// FROM
		query.append("FROM DIRECCIONES t1 ,DEPARTAMENTOS t2 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filterdirecciones, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto, Direcciones.class,
				query, params, "ID_DIRECCION");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Filter in the hierarchical Direcciones table.
	 *
	 * @param filterDirecciones
	 *            Direcciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<Direcciones>>
	 */
	@Override
	public List<JerarquiaDto<Direcciones>> findAllLikeJerarquia(Direcciones filterDirecciones,
			JQGridRequestDto jqGridRequestDto) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_DIRECCION IDDIRECCION,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_DEPARTAMENTO DEPIDDEPARTAMENTO,t2.DESCRIPCION_CASTELLANO DEPDESCRIPCIONCASTELLANO,t2.DESCRIPCION_EUSKERA DEPDESCRIPCIONEUSKERA,t2.FECHA_REGISTRO DEPFECHAREGISTRO,t2.USUARIO_REGISTRO DEPSUSUARIOREGISTRO ");
		// FROM
		// StringBuilder from = new StringBuilder("FROM DIRECCIONES t1
		// ,DEPARTAMENTOS t2 ");
		List<String> from = new ArrayList<String>();
		from.add("DIRECCIONES");
		from.add("DEPARTAMENTOS");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterDirecciones, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID_DIRECCION", "ID_PADRE",
				"NOMBRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query,
		// mapaWhere, "ID", "ID_PADRE", "NOMBRE", from, from_alias, joins,
		// businessFilters, businessParams);

		// PAGINACION
		if (jqGridRequestDto != null) {
			query = JQGridManagerJerarquia.getPaginationQuery(jqGridRequestDto, query,
					DireccionesDaoImpl.ORDER_BY_WHITE_LIST);
		}

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.query(query.toString(), this.rwMapJerarquia, params.toArray());
	}

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterDirecciones
	 *            Direcciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	@Override
	public Long findAllLikeCountJerarquia(Direcciones filterDirecciones, JQGridRequestDto jqGridRequestDto) {
		// FROM
		// StringBuilder from = new StringBuilder("FROM DIRECCIONES t1
		// ,DEPARTAMENTOS t2 ");
		List<String> from = new ArrayList<String>();
		from.add("DIRECCIONES");
		from.add("DEPARTAMENTOS");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterDirecciones, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID_DIRECCION",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterDirecciones
	 *            Direcciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<Direcciones>>
	 */
	@Override
	public List<TableRowDto<Direcciones>> findAllChild(Direcciones filterDirecciones,
			JQGridRequestDto jqGridRequestDto) {

		// FROM
		// StringBuilder from = new StringBuilder("FROM DIRECCIONES t1
		// ,DEPARTAMENTOS t2 ");
		List<String> from = new ArrayList<String>();
		from.add("DIRECCIONES");
		from.add("DEPARTAMENTOS");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterDirecciones, false);

		// MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID_DIRECCION",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(),
				new RowNumResultSetExtractor<Direcciones>(this.rwMapPK, "ID_DIRECCION"), params.toArray());
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Direcciones entity
	 *
	 * @param direcciones
	 *            Direcciones Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(Direcciones direcciones) {

		StringBuilder where = new StringBuilder(DireccionesDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (direcciones != null && direcciones.getIdDireccion() != null) {
			where.append(" AND t1.ID_DIRECCION = ?");
			params.add(direcciones.getIdDireccion());
		}
		if (direcciones != null && direcciones.getDepartamentos() != null
				&& direcciones.getDepartamentos().getIdDepartamento() != null) {
			where.append(" AND t2.ID_DEPARTAMENTO = ?");
			params.add(direcciones.getDepartamentos().getIdDepartamento());
		}
		if (direcciones != null && direcciones.getDepartamentos() != null
				&& direcciones.getDepartamentos().getDescripcionCastellano() != null) {
			where.append(" AND t2.DESCRIPCION_CASTELLANO = ?");
			params.add(direcciones.getDepartamentos().getDescripcionCastellano());
		}
		if (direcciones != null && direcciones.getDepartamentos() != null
				&& direcciones.getDepartamentos().getDescripcionEuskera() != null) {
			where.append(" AND t2.DESCRIPCION_EUSKERA = ?");
			params.add(direcciones.getDepartamentos().getDescripcionEuskera());
		}
		if (direcciones != null && direcciones.getDepartamentos() != null
				&& direcciones.getDepartamentos().getFechaRegistro() != null) {
			where.append(" AND t2.FECHA_REGISTRO = ?");
			params.add(direcciones.getDepartamentos().getFechaRegistro());
		}
		if (direcciones != null && direcciones.getDepartamentos() != null
				&& direcciones.getDepartamentos().getUsuarioRegistro() != null) {
			where.append(" AND t2.USUARIO_REGISTRO = ?");
			params.add(direcciones.getDepartamentos().getUsuarioRegistro());
		}
		if (direcciones != null && direcciones.getDescripcionCastellano() != null) {
			where.append(" AND t1.DESCRIPCION_CASTELLANO = ?");
			params.add(direcciones.getDescripcionCastellano());
		}
		if (direcciones != null && direcciones.getDescripcionEuskera() != null) {
			where.append(" AND t1.DESCRIPCION_EUSKERA = ?");
			params.add(direcciones.getDescripcionEuskera());
		}
		if (direcciones != null && direcciones.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(direcciones.getFechaRegistro());
		}
		if (direcciones != null && direcciones.getUsuarioRegistro() != null) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(direcciones.getUsuarioRegistro());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Direcciones entity
	 *
	 * @param direcciones
	 *            Direcciones Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(Direcciones direcciones, Boolean startsWith) {

		StringBuilder where = new StringBuilder(DireccionesDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (direcciones != null && direcciones.getIdDireccion() != null) {
			where.append(" AND t1.ID_DIRECCION = ?");
			params.add(direcciones.getIdDireccion());
		}
		if (direcciones != null && direcciones.getDepartamentos() != null
				&& direcciones.getDepartamentos().getIdDepartamento() != null) {
			where.append(" AND t2.ID_DEPARTAMENTO = ?");
			params.add(direcciones.getDepartamentos().getIdDepartamento());
		}
		if (direcciones != null && direcciones.getDepartamentos() != null
				&& direcciones.getDepartamentos().getDescripcionCastellano() != null) {
			where.append(" AND UPPER(t2.DESCRIPCION_CASTELLANO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(direcciones.getDepartamentos().getDescripcionCastellano().toUpperCase() + "%");
			} else {
				params.add("%" + direcciones.getDepartamentos().getDescripcionCastellano().toUpperCase() + "%");
			}
			where.append(" AND t2.DESCRIPCION_CASTELLANO IS NOT NULL");
		}
		if (direcciones != null && direcciones.getDepartamentos() != null
				&& direcciones.getDepartamentos().getDescripcionEuskera() != null) {
			where.append(" AND UPPER(t2.DESCRIPCION_EUSKERA) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(direcciones.getDepartamentos().getDescripcionEuskera().toUpperCase() + "%");
			} else {
				params.add("%" + direcciones.getDepartamentos().getDescripcionEuskera().toUpperCase() + "%");
			}
			where.append(" AND t2.DESCRIPCION_EUSKERA IS NOT NULL");
		}
		if (direcciones != null && direcciones.getDepartamentos() != null
				&& direcciones.getDepartamentos().getFechaRegistro() != null) {
			where.append(" AND t2.FECHA_REGISTRO = ?");
			params.add(direcciones.getDepartamentos().getFechaRegistro());
		}
		if (direcciones != null && direcciones.getDepartamentos() != null
				&& direcciones.getDepartamentos().getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t2.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(direcciones.getDepartamentos().getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + direcciones.getDepartamentos().getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t2.USUARIO_REGISTRO IS NOT NULL");
		}
		if (direcciones != null && direcciones.getDescripcionCastellano() != null) {
			where.append(" AND UPPER(t1.DESCRIPCION_CASTELLANO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(direcciones.getDescripcionCastellano().toUpperCase() + "%");
			} else {
				params.add("%" + direcciones.getDescripcionCastellano().toUpperCase() + "%");
			}
			where.append(" AND t1.DESCRIPCION_CASTELLANO IS NOT NULL");
		}
		if (direcciones != null && direcciones.getDescripcionEuskera() != null) {
			where.append(" AND UPPER(t1.DESCRIPCION_EUSKERA) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(direcciones.getDescripcionEuskera().toUpperCase() + "%");
			} else {
				params.add("%" + direcciones.getDescripcionEuskera().toUpperCase() + "%");
			}
			where.append(" AND t1.DESCRIPCION_EUSKERA IS NOT NULL");
		}
		if (direcciones != null && direcciones.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(direcciones.getFechaRegistro());
		}
		if (direcciones != null && direcciones.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(direcciones.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + direcciones.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
