package com.ejie.ab61.dao.solicitudes;

import com.ejie.ab61.model.procedimientos.Procedimientos;
import com.ejie.ab61.model.solicitudes.Configuracion;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;

/**
 * ConfiguracionDao  
 * 
 *  
 */

public interface ConfiguracionDao {

	/**
	 * Inserts a single row in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @return Configuracion
	 */
	Configuracion add(Configuracion configuracion);

	/**
	 * Updates a single row in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @return Configuracion
	 */
	Configuracion update(Configuracion configuracion);

	/**
	 * Finds a single row in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @return Configuracion
	 */
	Configuracion find(Configuracion configuracion);

	/**
	 * Removes a single row in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 */
	void remove(Configuracion configuracion);

	/**
	 * Finds a List of rows in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List
	 */
	List<Configuracion> findAll(Configuracion configuracion, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds rows in the Configuracion table using like.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List
	 */
	List<Configuracion> findAllLike(Configuracion configuracion, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Filter in the Configuracion table.
	 *
	 * @param filterConfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	void removeMultiple(Configuracion filterConfiguracion, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Counts rows in the Configuracion table using like.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	Long findAllLikeCount(Configuracion configuracion, Boolean startsWith);

	/**
	 * Counts rows in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @return Long
	 */
	Long findAllCount(Configuracion configuracion);

	/**
	 * Reorder selection.
	 *
	 * @param filterConfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Configuracion>>
	 */
	List<TableRowDto<Configuracion>> reorderSelection(Configuracion filterConfiguracion,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Searches in the Usuario table.
	 *
	 * @param filterConfiguracion
	 *            Configuracion
	 * @param searchConfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Configuracion>>
	 */
	List<TableRowDto<Configuracion>> search(Configuracion filterConfiguracion, Configuracion searchConfiguracion,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Filter in the hierarchical Configuracion table.
	 *
	 * @param filterConfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<Configuracion>>
	 */
	List<JerarquiaDto<Configuracion>> findAllLikeJerarquia(Configuracion filterConfiguracion,
			JQGridRequestDto jqGridRequestDto);

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterConfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	Long findAllLikeCountJerarquia(Configuracion filterConfiguracion, JQGridRequestDto jqGridRequestDto);

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterConfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<Configuracion>>
	 */
	List<TableRowDto<Configuracion>> findAllChild(Configuracion filterConfiguracion, JQGridRequestDto jqGridRequestDto);

	Integer comprobarConfigEventosRepetido(String codProcedureId, Integer idConfigEventos);

	void removeByCodProcedureId(String codProcedureId);

	List<Configuracion> findByCodProcedureId(String codProcedureId);

}
