package com.ejie.ab61.dao.solicitudes;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.model.configuracionEventos.ConfigEventos;
import com.ejie.ab61.model.procedimientos.Procedimientos;
import com.ejie.ab61.model.solicitudes.Configuracion;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * ConfiguracionDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class ConfiguracionDaoImpl implements ConfiguracionDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDCONFIGURACION", "CODPROCEDUREID", "TIPOEVENTO",
			"PLSQLCARGA", "FECHAREGISTRO", "USUARIOREGISTRO", "CONFIGEVENTOSDESCRIPCIONES",
			"CONFIGEVENTOSDESCRIPCIONEU" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<Configuracion> rwMap = new RowMapper<Configuracion>() {
		public Configuracion mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Configuracion(resultSet.getInt("IDCONFIGURACION"), resultSet.getString("CODPROCEDUREID"),
					resultSet.getString("TIPOEVENTO"), resultSet.getString("PLSQLCARGA"), null, null,
					new ConfigEventos(resultSet.getInt("IDCONFIGEVENTOS"), resultSet.getString("TIPOEVENTO"),
							resultSet.getString("PLATEATYPE"), resultSet.getString("PLATEASUBTYPE"),
							resultSet.getInt("CONFIGURACION"), resultSet.getDate("CONFIGEVENTOSFECHAREGISTRO"),
							resultSet.getString("CONFIGEVENTOSUSUARIOREGISTRO"),
							resultSet.getString("CONFIGEVENTOSDESCRIPCIONES"),
							resultSet.getString("CONFIGEVENTOSDESCRIPCIONEU")));
		}
	};

	private RowMapper<Configuracion> rwMapPK = new RowMapper<Configuracion>() {
		public Configuracion mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Configuracion(resultSet.getInt("IDCONFIGURACION"));
		}
	};

	/**
	 * Rowmapper para Jerarquía
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	private RowMapper<JerarquiaDto<Configuracion>> rwMapJerarquia = new RowMapper<JerarquiaDto<Configuracion>>() {
		public JerarquiaDto<Configuracion> mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			Configuracion configuracion = new Configuracion(resultSet.getInt("IDCONFIGURACION"),
					resultSet.getString("CODPROCEDUREID"), resultSet.getString("TIPOEVENTO"),
					resultSet.getString("PLSQLCARGA"), resultSet.getDate("FECHAREGISTRO"),
					resultSet.getString("USUARIOREGISTRO"),
					new ConfigEventos(resultSet.getInt("IDCONFIGEVENTOS"), resultSet.getString("TIPOEVENTO"),
							resultSet.getString("PLATEATYPE"), resultSet.getString("PLATEASUBTYPE"),
							resultSet.getInt("CONFIGURACION"), resultSet.getDate("CONFIGEVENTOSFECHAREGISTRO"),
							resultSet.getString("CONFIGEVENTOSUSUARIOREGISTRO"),
							resultSet.getString("CONFIGEVENTOSDESCRIPCIONES"),
							resultSet.getString("CONFIGEVENTOSDESCRIPCIONEU")));

			JerarquiaDto<Configuracion> jerarquia = new JerarquiaDto<Configuracion>();
			jerarquia.setModel(configuracion);
			jerarquia.setLevel(resultSet.getBigDecimal("LEVEL").intValue());
			jerarquia.setParentNodes(resultSet.getString("PARENTNODES"));
			jerarquia.setIsLeaf(Boolean.parseBoolean(resultSet.getString("ISLEAF")));
			jerarquia.setFilter(Boolean.parseBoolean(resultSet.getString("FILTER")));
			return jerarquia;
		}
	};

	private RowMapper<Configuracion> rwMapByCodProcedureId = new RowMapper<Configuracion>() {
		public Configuracion mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			Configuracion configuracion = new Configuracion();
			configuracion.setIdConfiguracion(resultSet.getInt("IDCONFIGURACION"));
			configuracion.setCodProcedureId(resultSet.getString("CODPROCEDUREID"));
			configuracion.setPlsqlCarga(resultSet.getString("PLSQLCARGA"));
			configuracion.setFechaRegistro(resultSet.getDate("FECHAREGISTRO"));
			configuracion.setUsuarioRegistro(resultSet.getString("USUARIOREGISTRO"));
			configuracion.setFechaUltmodificacion(resultSet.getDate("FECHAULTMODIFICACION"));
			configuracion.setUsuarioUltmodificacion(resultSet.getString("USUARIOULTMODIFICACION"));

			ConfigEventos configEventos = new ConfigEventos();
			configEventos.setIdConfigEventos(resultSet.getInt("IDCONFIGEVENTOS"));
			if (resultSet.wasNull()) {
				configEventos.setIdConfigEventos(null);
			}
			configuracion.setConfigEventos(configEventos);

			return configuracion;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @return Configuracion
	 */
	public Configuracion add(Configuracion configuracion) {
		String query = "INSERT INTO CONFIGURACION (ID_CONFIGURACION, COD_PROCEDURE_ID, PLSQL_CARGA, FECHA_REGISTRO, USUARIO_REGISTRO, ID_CONFIG_EVENTOS) VALUES (S_AB6119T00.NEXTVAL,?,?,?,?,?)";
		this.jdbcTemplate.update(query, configuracion.getCodProcedureId(), configuracion.getPlsqlCarga(),
				configuracion.getFechaRegistro(), configuracion.getUsuarioRegistro(),
				configuracion.getConfigEventos().getIdConfigEventos());
		return configuracion;
	}

	/**
	 * Updates a single row in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @return Configuracion
	 */
	public Configuracion update(Configuracion configuracion) {
		String query = "UPDATE CONFIGURACION SET COD_PROCEDURE_ID=?, PLSQL_CARGA=?, ID_CONFIG_EVENTOS=?, FECHA_ULTMODIFICACION=?, USUARIO_ULTMODIFICACION=? WHERE ID_CONFIGURACION=?";
		this.jdbcTemplate.update(query, configuracion.getCodProcedureId(), configuracion.getPlsqlCarga(),
				configuracion.getConfigEventos().getIdConfigEventos(), configuracion.getFechaUltmodificacion(),
				configuracion.getUsuarioUltmodificacion(), configuracion.getIdConfiguracion());
		return configuracion;
	}

	/**
	 * Finds a single row in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @return Configuracion
	 */
	@Transactional(readOnly = true)
	public Configuracion find(Configuracion configuracion) {
		String query = "SELECT t1.ID_CONFIGURACION IDCONFIGURACION, t1.COD_PROCEDURE_ID CODPROCEDUREID, t1.PLSQL_CARGA PLSQLCARGA, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO, t2. ID_CONFIG_EVENTOS IDCONFIGEVENTOS, t2.TIPO_EVENTO TIPOEVENTO, t2.PLATEA_TYPE PLATEATYPE, t2.PLATEA_SUBTYPE PLATEASUBTYPE, t2.CONFIGURACION CONFIGURACION, t2.DESCRIPCION_CASTELLANO CONFIGEVENTOSDESCRIPCIONES, t2.DESCRIPCION_EUSKERA CONFIGEVENTOSDESCRIPCIONEU, t2.FECHA_REGISTRO CONFIGEVENTOSFECHAREGISTRO, t2.USUARIO_REGISTRO CONFIGEVENTOSUSUARIOREGISTRO  FROM CONFIGURACION t1, CONFIG_EVENTOS t2  WHERE t1.ID_CONFIGURACION = ? AND t1.ID_CONFIG_EVENTOS = t2.ID_CONFIG_EVENTOS  ";

		List<Configuracion> configuracionList = this.jdbcTemplate.query(query, this.rwMap,
				configuracion.getIdConfiguracion());
		return (Configuracion) DataAccessUtils.uniqueResult(configuracionList);
	}

	@Override
	public List<Configuracion> findByCodProcedureId(String codProcedureId) {
		String query = "SELECT t1.ID_CONFIGURACION IDCONFIGURACION, t1.COD_PROCEDURE_ID CODPROCEDUREID, t1.PLSQL_CARGA PLSQLCARGA, t1.ID_CONFIG_EVENTOS IDCONFIGEVENTOS, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO, t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION, t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION FROM CONFIGURACION t1 WHERE t1.COD_PROCEDURE_ID = ?";
		return this.jdbcTemplate.query(query, this.rwMapByCodProcedureId, codProcedureId);
	}

	@Override
	public Integer comprobarConfigEventosRepetido(String codProcedureId, Integer idConfigEventos) {
		String query = "SELECT COUNT(1) FROM CONFIGURACION t1 WHERE t1.COD_PROCEDURE_ID = ? AND ID_CONFIG_EVENTOS = ?";

		return this.jdbcTemplate.queryForInt(query, codProcedureId, idConfigEventos);
	}

	/**
	 * Removes a single row in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @return
	 */
	public void remove(Configuracion configuracion) {
		String query = "DELETE FROM CONFIGURACION WHERE ID_CONFIGURACION=?";
		this.jdbcTemplate.update(query, configuracion.getIdConfiguracion());
	}

	@Override
	public void removeByCodProcedureId(String codProcedureId) {
		String query = "DELETE FROM CONFIGURACION WHERE COD_PROCEDURE_ID=?";
		this.jdbcTemplate.update(query, codProcedureId);
	}

	/**
	 * Finds a list of rows in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Configuracion>
	 */
	@Transactional(readOnly = true)
	public List<Configuracion> findAll(Configuracion configuracion, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_CONFIGURACION IDCONFIGURACION, t1.COD_PROCEDURE_ID CODPROCEDUREID, t1.PLSQL_CARGA PLSQLCARGA, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO, t2. ID_CONFIG_EVENTOS IDCONFIGEVENTOS, t2.TIPO_EVENTO TIPOEVENTO, t2.PLATEA_TYPE PLATEATYPE, t2.PLATEA_SUBTYPE PLATEASUBTYPE, t2.CONFIGURACION CONFIGURACION, t2.DESCRIPCION_CASTELLANO CONFIGEVENTOSDESCRIPCIONES, t2.DESCRIPCION_EUSKERA CONFIGEVENTOSDESCRIPCIONEU, t2.FECHA_REGISTRO CONFIGEVENTOSFECHAREGISTRO, t2.USUARIO_REGISTRO CONFIGEVENTOSUSUARIOREGISTRO  ");
		query.append("FROM CONFIGURACION t1, CONFIG_EVENTOS t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(configuracion);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);
		query.append(" AND T1.ID_CONFIG_EVENTOS = T2.ID_CONFIG_EVENTOS");

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, ConfiguracionDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Configuracion>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the Configuracion table using like.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Configuracion>
	 */
	@Transactional(readOnly = true)
	public List<Configuracion> findAllLike(Configuracion configuracion, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_CONFIGURACION IDCONFIGURACION,t1.COD_PROCEDURE_ID CODPROCEDUREID,t1.PLSQL_CARGA PLSQLCARGA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO, t2. ID_CONFIG_EVENTOS IDCONFIGEVENTOS, t2.TIPO_EVENTO TIPOEVENTO, t2.PLATEA_TYPE PLATEATYPE, t2.PLATEA_SUBTYPE PLATEASUBTYPE, t2.CONFIGURACION CONFIGURACION, t2.DESCRIPCION_CASTELLANO CONFIGEVENTOSDESCRIPCIONES, t2.DESCRIPCION_EUSKERA CONFIGEVENTOSDESCRIPCIONEU, t2.FECHA_REGISTRO CONFIGEVENTOSFECHAREGISTRO, t2.USUARIO_REGISTRO CONFIGEVENTOSUSUARIOREGISTRO ");
		query.append("FROM CONFIGURACION t1, CONFIG_EVENTOS t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(configuracion, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND T1.ID_CONFIG_EVENTOS = T2.ID_CONFIG_EVENTOS ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, ConfiguracionDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Configuracion>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(Configuracion configuracion) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM CONFIGURACION t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(configuracion);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the Configuracion table using like.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(Configuracion configuracion, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM CONFIGURACION t1, CONFIG_EVENTOS t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(configuracion, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND T1.ID_CONFIG_EVENTOS = T2.ID_CONFIG_EVENTOS ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of Configuracion selected for rup_table
	 *
	 * @param configuracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Configuracion>>
	 */
	@Override
	public List<TableRowDto<Configuracion>> reorderSelection(Configuracion configuracion,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_CONFIGURACION IDCONFIGURACION,t1.COD_PROCEDURE_ID CODPROCEDUREID,t1.PLSQL_CARGA PLSQLCARGA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO, t2. ID_CONFIG_EVENTOS IDCONFIGEVENTOS, t2.TIPO_EVENTO TIPOEVENTO, t2.PLATEA_TYPE PLATEATYPE, t2.PLATEA_SUBTYPE PLATEASUBTYPE, t2.CONFIGURACION CONFIGURACION, t2.DESCRIPCION_CASTELLANO CONFIGEVENTOSDESCRIPCIONES, t2.DESCRIPCION_EUSKERA CONFIGEVENTOSDESCRIPCIONEU, t2.FECHA_REGISTRO CONFIGEVENTOSFECHAREGISTRO, t2.USUARIO_REGISTRO CONFIGEVENTOSUSUARIOREGISTRO ");
		// FROM
		query.append("FROM CONFIGURACION t1, CONFIG_EVENTOS t2 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(configuracion, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND T1.ID_CONFIG_EVENTOS = T2.ID_CONFIG_EVENTOS ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto,
				Configuracion.class, filterParamList, "ID_CONFIGURACION");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Configuracion>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
	 *
	 * @param filterParams
	 *            Configuracion
	 * @param searchParams
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Configuracion>>
	 */
	@Override
	public List<TableRowDto<Configuracion>> search(Configuracion filterParams, Configuracion searchParams,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_CONFIGURACION IDCONFIGURACION,t1.COD_PROCEDURE_ID CODPROCEDUREID,t1.PLSQL_CARGA PLSQLCARGA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		query.append("FROM CONFIGURACION t1 ");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto, Configuracion.class,
				filterParamList, searchSQL, searchParamList, from_alias, "ID_CONFIGURACION");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Configuracion>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
	 *
	 * @param filterconfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	@Override
	public void removeMultiple(Configuracion filterconfiguracion, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_CONFIGURACION IDCONFIGURACION,t1.COD_PROCEDURE_ID CODPROCEDUREID,t1.PLSQL_CARGA PLSQLCARGA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		query.append("FROM CONFIGURACION t1 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filterconfiguracion, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto, Configuracion.class,
				query, params, "ID_CONFIGURACION");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Filter in the hierarchical Configuracion table.
	 *
	 * @param filterConfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<Configuracion>>
	 */
	@Override
	public List<JerarquiaDto<Configuracion>> findAllLikeJerarquia(Configuracion filterConfiguracion,
			JQGridRequestDto jqGridRequestDto) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_CONFIGURACION IDCONFIGURACION,t1.COD_PROCEDURE_ID CODPROCEDUREID,t1.PLSQL_CARGA PLSQLCARGA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO, t2. ID_CONFIG_EVENTOS IDCONFIGEVENTOS, t2.TIPO_EVENTO TIPOEVENTO, t2.PLATEA_TYPE PLATEATYPE, t2.PLATEA_SUBTYPE PLATEASUBTYPE, t2.CONFIGURACION CONFIGURACION, t2.DESCRIPCION_CASTELLANO CONFIGEVENTOSDESCRIPCIONES, t2.DESCRIPCION_EUSKERA CONFIGEVENTOSDESCRIPCIONEU, t2.FECHA_REGISTRO CONFIGEVENTOSFECHAREGISTRO, t2.USUARIO_REGISTRO CONFIGEVENTOSUSUARIOREGISTRO ");
		// FROM
		// StringBuilder from = new StringBuilder("FROM CONFIGURACION t1 ");
		List<String> from = new ArrayList<String>();
		from.add("CONFIGURACION");
		from.add("CONFIG_EVENTOS");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterConfiguracion, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID_CONFIGURACION", "ID_PADRE",
				"NOMBRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query,
		// mapaWhere, "ID", "ID_PADRE", "NOMBRE", from, from_alias, joins,
		// businessFilters, businessParams);

		// PAGINACION
		if (jqGridRequestDto != null) {
			query = JQGridManagerJerarquia.getPaginationQuery(jqGridRequestDto, query,
					ConfiguracionDaoImpl.ORDER_BY_WHITE_LIST);
		}

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.query(query.toString(), this.rwMapJerarquia, params.toArray());
	}

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterConfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	@Override
	public Long findAllLikeCountJerarquia(Configuracion filterConfiguracion, JQGridRequestDto jqGridRequestDto) {
		// FROM
		// StringBuilder from = new StringBuilder("FROM CONFIGURACION t1 ");
		List<String> from = new ArrayList<String>();
		from.add("CONFIGURACION");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterConfiguracion, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID_CONFIGURACION",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterConfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<Configuracion>>
	 */
	@Override
	public List<TableRowDto<Configuracion>> findAllChild(Configuracion filterConfiguracion,
			JQGridRequestDto jqGridRequestDto) {

		// FROM
		// StringBuilder from = new StringBuilder("FROM CONFIGURACION t1 ");
		List<String> from = new ArrayList<String>();
		from.add("CONFIGURACION");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterConfiguracion, false);

		// MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID_CONFIGURACION",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(),
				new RowNumResultSetExtractor<Configuracion>(this.rwMapPK, "ID_CONFIGURACION"), params.toArray());
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Configuracion entity
	 *
	 * @param configuracion
	 *            Configuracion Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(Configuracion configuracion) {

		StringBuilder where = new StringBuilder(ConfiguracionDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (configuracion != null && configuracion.getIdConfiguracion() != null) {
			where.append(" AND t1.ID_CONFIGURACION = ?");
			params.add(configuracion.getIdConfiguracion());
		}
		if (configuracion != null && configuracion.getCodProcedureId() != null) {
			where.append(" AND t1.COD_PROCEDURE_ID = ?");
			params.add(configuracion.getCodProcedureId());
		}
		if (configuracion != null && configuracion.getTipoEvento() != null) {
			where.append(" AND t2.TIPO_EVENTO = ?");
			params.add(configuracion.getTipoEvento());
		}
		if (configuracion != null && configuracion.getPlsqlCarga() != null) {
			where.append(" AND t1.PLSQL_CARGA = ?");
			params.add(configuracion.getPlsqlCarga());
		}
		if (configuracion != null && configuracion.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(configuracion.getFechaRegistro());
		}
		if (configuracion != null && configuracion.getUsuarioRegistro() != null) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(configuracion.getUsuarioRegistro());
		}
		if (configuracion != null && configuracion.getPlateaType() != null) {
			where.append(" AND t2.PLATEA_TYPE = ?");
			params.add(configuracion.getPlateaType());
		} else {
			where.append(" AND t2.PLATEA_TYPE IS NULL");
		}
		if (configuracion != null && configuracion.getPlateaSubType() != null) {
			where.append(" AND t2.PLATEA_SUBTYPE = ?");
			params.add(configuracion.getPlateaSubType());
		} else {
			where.append(" AND t2.PLATEA_SUBTYPE IS NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Configuracion entity
	 *
	 * @param configuracion
	 *            Configuracion Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(Configuracion configuracion, Boolean startsWith) {

		StringBuilder where = new StringBuilder(ConfiguracionDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (configuracion != null && configuracion.getIdConfiguracion() != null) {
			where.append(" AND t1.ID_CONFIGURACION = ?");
			params.add(configuracion.getIdConfiguracion());
		}
		if (configuracion != null && configuracion.getCodProcedureId() != null) {
			where.append(" AND UPPER(t1.COD_PROCEDURE_ID) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(configuracion.getCodProcedureId().toUpperCase() + "%");
			} else {
				params.add("%" + configuracion.getCodProcedureId().toUpperCase() + "%");
			}
			where.append(" AND t1.COD_PROCEDURE_ID IS NOT NULL");
		}
		if (configuracion != null && configuracion.getTipoEvento() != null) {
			where.append(" AND UPPER(t1.TIPO_EVENTO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(configuracion.getTipoEvento().toUpperCase() + "%");
			} else {
				params.add("%" + configuracion.getTipoEvento().toUpperCase() + "%");
			}
			where.append(" AND t1.TIPO_EVENTO IS NOT NULL");
		}
		if (configuracion != null && configuracion.getPlsqlCarga() != null) {
			where.append(" AND UPPER(t1.PLSQL_CARGA) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(configuracion.getPlsqlCarga().toUpperCase() + "%");
			} else {
				params.add("%" + configuracion.getPlsqlCarga().toUpperCase() + "%");
			}
			where.append(" AND t1.PLSQL_CARGA IS NOT NULL");
		}
		if (configuracion != null && configuracion.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(configuracion.getFechaRegistro());
		}
		if (configuracion != null && configuracion.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(configuracion.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + configuracion.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
