package com.ejie.ab61.service.configuracionEventos;

import com.ejie.ab61.dao.configuracionEventos.ConfigEventosDao;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.model.configuracionEventos.ConfigEventos;

/**
 * ConfigEventosServiceImpl  
 * 
 *  
 */

@Service(value = "configEventosService")
public class ConfigEventosServiceImpl implements ConfigEventosService {

	@Autowired
	private ConfigEventosDao configEventosDao;

	/**
	 * Inserts a single row in the ConfigEventos table.
	 *
	 * @param configEventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	@Transactional(rollbackFor = Throwable.class)
	public ConfigEventos add(ConfigEventos configEventos) {
		return this.configEventosDao.add(configEventos);
	}

	/**
	 * Updates a single row in the ConfigEventos table.
	 *
	 * @param configEventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	@Transactional(rollbackFor = Throwable.class)
	public ConfigEventos update(ConfigEventos configEventos) {
		return this.configEventosDao.update(configEventos);
	}

	/**
	 * Finds a single row in the ConfigEventos table.
	 *
	 * @param configEventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	public ConfigEventos find(ConfigEventos configEventos) {
		return (ConfigEventos) this.configEventosDao.find(configEventos);
	}

	/**
	 * Encuentra la configuracion de un evento determinado
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	@Transactional(readOnly = true)
	public ConfigEventos findConfiguracion(ConfigEventos configeventos) {
		return (ConfigEventos) this.configEventosDao.findConfiguracion(configeventos);
	}

	/**
	 * Deletes a single row in the ConfigEventos table.
	 *
	 * @param configEventos
	 *            ConfigEventos
	 */
	@Transactional(rollbackFor = Throwable.class)
	public void remove(ConfigEventos configEventos) {
		this.configEventosDao.remove(configEventos);
	}

	/**
	 * Finds a list of rows in the ConfigEventos table.
	 *
	 * @param configEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<ConfigEventos>
	 */
	public List<ConfigEventos> findAll(ConfigEventos configEventos, JQGridRequestDto jqGridRequestDto) {
		return (List<ConfigEventos>) this.configEventosDao.findAll(configEventos, jqGridRequestDto);
	}

	/**
	 * Finds rows in the ConfigEventos table using like.
	 *
	 * @param configEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<ConfigEventos>
	 */
	public List<ConfigEventos> findAllLike(ConfigEventos configEventos, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		return (List<ConfigEventos>) this.configEventosDao.findAllLike(configEventos, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Removes rows from the ConfigEventos table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	public void removeMultiple(ConfigEventos filterConfigEventos, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		this.configEventosDao.removeMultiple(filterConfigEventos, jqGridRequestDto, startsWith);
	}

	/**
	 * Filter method in the ConfigEventos table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<ConfigEventos>
	 */
	public JQGridResponseDto<ConfigEventos> filter(ConfigEventos filterConfigEventos, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		List<ConfigEventos> listaConfigEventos = this.configEventosDao.findAllLike(filterConfigEventos,
				jqGridRequestDto, false);

		for (ConfigEventos configEventos : listaConfigEventos) {
			configEventos.setNumeroConfiguraciones(this.configEventosDao.getNumeroConfiguraciones(configEventos));
		}

		Long recordNum = this.configEventosDao
				.findAllLikeCount(filterConfigEventos != null ? filterConfigEventos : new ConfigEventos(), false);
		if (jqGridRequestDto.getMultiselection().getSelectedIds() != null) {
			List<TableRowDto<ConfigEventos>> reorderSelection = this.configEventosDao
					.reorderSelection(filterConfigEventos, jqGridRequestDto, startsWith);
			return new JQGridResponseDto<ConfigEventos>(jqGridRequestDto, recordNum, listaConfigEventos,
					reorderSelection);
		}
		return new JQGridResponseDto<ConfigEventos>(jqGridRequestDto, recordNum, listaConfigEventos);
	}

	/**
	 * Searches rows in the ConfigEventos table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param searchConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<ConfigEventos>>
	 */
	public List<TableRowDto<ConfigEventos>> search(ConfigEventos filterConfigEventos, ConfigEventos searchConfigEventos,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		return this.configEventosDao.search(filterConfigEventos, searchConfigEventos, jqGridRequestDto, startsWith);
	}

	/**
	 * Reorder the selection made in ConfigEventos table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return Object
	 */
	public Object reorderSelection(ConfigEventos filterConfigEventos, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		return this.configEventosDao.reorderSelection(filterConfigEventos, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Finder method in the hierarchical the ConfigEventos table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<JerarquiaDto<ConfigEventos>>
	 */
	public JQGridResponseDto<JerarquiaDto<ConfigEventos>> jerarquia(ConfigEventos filterConfigEventos,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		List<JerarquiaDto<ConfigEventos>> listaConfigEventos = this.configEventosDao
				.findAllLikeJerarquia(filterConfigEventos, jqGridRequestDto);
		Long recordNum = this.configEventosDao.findAllLikeCountJerarquia(filterConfigEventos, jqGridRequestDto);
		return new JQGridResponseDto<JerarquiaDto<ConfigEventos>>(jqGridRequestDto, recordNum, listaConfigEventos);
	}

	/**
	 * Finder method for siblings rows in the hierarchical the ConfigEventos
	 * table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<ConfigEventos>>
	 */
	public JQGridResponseDto<JerarquiaDto<ConfigEventos>> jerarquiaChildren(ConfigEventos filterConfigEventos,
			JQGridRequestDto jqGridRequestDto) {
		JQGridResponseDto<JerarquiaDto<ConfigEventos>> jqGridResponseDto = new JQGridResponseDto<JerarquiaDto<ConfigEventos>>();
		jqGridResponseDto.addAdditionalParam(JQGridResponseDto.CHILDREN,
				this.configEventosDao.findAllChild(filterConfigEventos, jqGridRequestDto));
		return jqGridResponseDto;
	}

}
