package com.ejie.ab61.service.expedientes;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.dao.expedientes.FechasExpedienteDao;
import com.ejie.ab61.model.expedientes.FechasExpediente;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * FechasExpedienteServiceImpl  
 * 
 *  
 */

@Service(value = "fechasExpedienteService")
public class FechasExpedienteServiceImpl implements FechasExpedienteService {

	@Autowired
	private FechasExpedienteDao fechasExpedienteDao;

	/**
	 * Inserts a single row in the FechasExpediente table.
	 *
	 * @param fechasExpediente
	 *            FechasExpediente
	 * @return FechasExpediente
	 */
	@Transactional(rollbackFor = Throwable.class)
	public FechasExpediente add(FechasExpediente fechasExpediente) {
		return this.fechasExpedienteDao.add(fechasExpediente);
	}

	/**
	 * Updates a single row in the FechasExpediente table.
	 *
	 * @param fechasExpediente
	 *            FechasExpediente
	 * @return FechasExpediente
	 */
	@Transactional(rollbackFor = Throwable.class)
	public FechasExpediente update(FechasExpediente fechasExpediente) {
		return this.fechasExpedienteDao.update(fechasExpediente);
	}

	/**
	 * Finds a single row in the FechasExpediente table.
	 *
	 * @param fechasExpediente
	 *            FechasExpediente
	 * @return FechasExpediente
	 */
	public FechasExpediente find(FechasExpediente fechasExpediente) {
		return (FechasExpediente) this.fechasExpedienteDao.find(fechasExpediente);
	}

	/**
	 * Deletes a single row in the FechasExpediente table.
	 *
	 * @param fechasExpediente
	 *            FechasExpediente
	 */
	@Transactional(rollbackFor = Throwable.class)
	public void remove(FechasExpediente fechasExpediente) {
		this.fechasExpedienteDao.remove(fechasExpediente);
	}

	/**
	 * Finds a list of rows in the FechasExpediente table.
	 *
	 * @param fechasExpediente
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<FechasExpediente>
	 */
	public List<FechasExpediente> findAll(FechasExpediente fechasExpediente, JQGridRequestDto jqGridRequestDto) {
		return (List<FechasExpediente>) this.fechasExpedienteDao.findAll(fechasExpediente, jqGridRequestDto);
	}

	/**
	 * Finds rows in the FechasExpediente table using like.
	 *
	 * @param fechasExpediente
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<FechasExpediente>
	 */
	public List<FechasExpediente> findAllLike(FechasExpediente fechasExpediente, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		return (List<FechasExpediente>) this.fechasExpedienteDao.findAllLike(fechasExpediente, jqGridRequestDto,
				startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Removes rows from the FechasExpediente table.
	 *
	 * @param filterFechasExpediente
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	public void removeMultiple(FechasExpediente filterFechasExpediente, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		this.fechasExpedienteDao.removeMultiple(filterFechasExpediente, jqGridRequestDto, startsWith);
	}

	/**
	 * Filter method in the FechasExpediente table.
	 *
	 * @param filterFechasExpediente
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<FechasExpediente>
	 */
	public JQGridResponseDto<FechasExpediente> filter(FechasExpediente filterFechasExpediente,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		List<FechasExpediente> listaFechasExpediente = this.fechasExpedienteDao.findAllLike(filterFechasExpediente,
				jqGridRequestDto, false);

		for (FechasExpediente fechasExpediente : listaFechasExpediente) {
			fechasExpediente.setNumeroRelExpFech(this.fechasExpedienteDao.getNumeroRelExpFech(fechasExpediente));
		}

		Long recordNum = this.fechasExpedienteDao.findAllLikeCount(
				filterFechasExpediente != null ? filterFechasExpediente : new FechasExpediente(), false);
		if (jqGridRequestDto.getMultiselection().getSelectedIds() != null) {
			List<TableRowDto<FechasExpediente>> reorderSelection = this.fechasExpedienteDao
					.reorderSelection(filterFechasExpediente, jqGridRequestDto, startsWith);
			return new JQGridResponseDto<FechasExpediente>(jqGridRequestDto, recordNum, listaFechasExpediente,
					reorderSelection);
		}
		return new JQGridResponseDto<FechasExpediente>(jqGridRequestDto, recordNum, listaFechasExpediente);
	}

	/**
	 * Searches rows in the FechasExpediente table.
	 *
	 * @param filterFechasExpediente
	 *            FechasExpediente
	 * @param searchFechasExpediente
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<FechasExpediente>>
	 */
	public List<TableRowDto<FechasExpediente>> search(FechasExpediente filterFechasExpediente,
			FechasExpediente searchFechasExpediente, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		return this.fechasExpedienteDao.search(filterFechasExpediente, searchFechasExpediente, jqGridRequestDto,
				startsWith);
	}

	/**
	 * Reorder the selection made in FechasExpediente table.
	 *
	 * @param filterFechasExpediente
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return Object
	 */
	public Object reorderSelection(FechasExpediente filterFechasExpediente, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		return this.fechasExpedienteDao.reorderSelection(filterFechasExpediente, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Finder method in the hierarchical the FechasExpediente table.
	 *
	 * @param filterFechasExpediente
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<JerarquiaDto<FechasExpediente>>
	 */
	public JQGridResponseDto<JerarquiaDto<FechasExpediente>> jerarquia(FechasExpediente filterFechasExpediente,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		List<JerarquiaDto<FechasExpediente>> listaFechasExpediente = this.fechasExpedienteDao
				.findAllLikeJerarquia(filterFechasExpediente, jqGridRequestDto);
		Long recordNum = this.fechasExpedienteDao.findAllLikeCountJerarquia(filterFechasExpediente, jqGridRequestDto);
		return new JQGridResponseDto<JerarquiaDto<FechasExpediente>>(jqGridRequestDto, recordNum,
				listaFechasExpediente);
	}

	/**
	 * Finder method for siblings rows in the hierarchical the FechasExpediente
	 * table.
	 *
	 * @param filterFechasExpediente
	 *            FechasExpediente
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<FechasExpediente>>
	 */
	public JQGridResponseDto<JerarquiaDto<FechasExpediente>> jerarquiaChildren(FechasExpediente filterFechasExpediente,
			JQGridRequestDto jqGridRequestDto) {
		JQGridResponseDto<JerarquiaDto<FechasExpediente>> jqGridResponseDto = new JQGridResponseDto<JerarquiaDto<FechasExpediente>>();
		jqGridResponseDto.addAdditionalParam(JQGridResponseDto.CHILDREN,
				this.fechasExpedienteDao.findAllChild(filterFechasExpediente, jqGridRequestDto));
		return jqGridResponseDto;
	}

}
