package com.ejie.ab61.service.listados;

import java.util.List;

import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.model.listados.Listados;
import com.ejie.ab61.model.listados.RelLisPro;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;

/**
 * ListadosService  
 * 
 *  
 */

public interface ListadosService {

	/**
	 * Inserts a single row in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 * @return Listados
	 */
	Listados add(Listados listados);

	/**
	 * Updates a single row in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 * @return Listados
	 */
	Listados update(Listados listados);

	/**
	 * Finds a single row in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 * @return Listados
	 */
	Listados find(Listados listados);

	/**
	 * Deletes a single row in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 */
	void remove(Listados listados);

	/**
	 * Finds a list of rows in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Listados>
	 */
	List<Listados> findAll(Listados listados, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds rows in the Listados table using like.
	 *
	 * @param listados
	 *            Listados
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Listados>
	 */
	List<Listados> findAllLike(Listados listados, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Filter method in the Listados table.
	 *
	 * @param filterListados
	 *            Listados
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<Listados>
	 */
	JQGridResponseDto<Listados> filter(Listados filterListados, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Reorder the selection made in Listados table.
	 *
	 * @param filterListados
	 *            Listados
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return Object
	 */
	Object reorderSelection(Listados filterListados, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	Listados removeFicheros(Listados listados);

	Listados updateSinFichero(Listados listados);

	/**
	 * Filter method in the RelLisPro table.
	 *
	 * @param filterRelLisPro
	 *            RelLisPro
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return JQGridResponseDto<RelLisPro>
	 */
	JQGridResponseDto<RelLisPro> filterRelLisPro(RelLisPro filterRelLisPro, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds a single row in the RelLisPro table.
	 *
	 * @param idRelLisPro
	 *            Integer
	 * @return RelLisPro
	 */
	RelLisPro findRelLisPro(Integer idRelLisPro);

	/**
	 * Inserts a single row in the RelLisPro table.
	 *
	 * @param relLisPro
	 *            RelLisPro
	 * @return RelLisPro
	 */
	RelLisPro addRelLisPro(RelLisPro relLisPro);

	/**
	 * Updates a single row in the RelLisPro table.
	 *
	 * @param relLisPro
	 *            RelLisPro
	 * @return RelLisPro
	 */
	RelLisPro updateRelLisPro(RelLisPro relLisPro);

	/**
	 * Deletes a single row in the RelLisPro table.
	 *
	 * @param idRelLisPro
	 *            Integer
	 */
	void removeRelLisPro(Integer idRelLisPro);
}
