package com.ejie.ab61.service.listados;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.dao.busquedas.BusquedasDao;
import com.ejie.ab61.dao.listados.ListadosDao;
import com.ejie.ab61.dao.parametros.ParametrosDao;
import com.ejie.ab61.model.busquedas.Busquedas;
import com.ejie.ab61.model.listados.Listados;
import com.ejie.ab61.model.listados.RelLisPro;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * ListadosServiceImpl  
 * 
 *  
 */

@Service(value = "listadosService")
public class ListadosServiceImpl implements ListadosService {

	@Autowired
	private ListadosDao listadosDao;
	@Autowired
	private BusquedasDao busquedasDao;
	@Autowired
	private ParametrosDao parametrosDao;

	/**
	 * Inserts a single row in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 * @return Listados
	 */
	@Transactional(rollbackFor = Throwable.class)
	public Listados add(Listados listados) {
		return this.listadosDao.add(listados);
	}

	/**
	 * Updates a single row in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 * @return Listados
	 */
	@Transactional(rollbackFor = Throwable.class)
	public Listados update(Listados listados) {
		return this.listadosDao.update(listados);
	}

	@Override
	public Listados updateSinFichero(Listados listados) {
		return this.listadosDao.updateSinFichero(listados);
	}

	/**
	 * Finds a single row in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 * @return Listados
	 */
	public Listados find(Listados listados) {
		return (Listados) this.listadosDao.find(listados);
	}

	/**
	 * Deletes a single row in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 */
	@Transactional(rollbackFor = Throwable.class)
	public void remove(Listados listados) {
		// recuperar las busquedas asociadas
		Busquedas busquedas = new Busquedas();
		busquedas.setIdListado(listados.getIdListado());
		List<Busquedas> listaBusquedas = this.busquedasDao.findAllLike(busquedas, null, false);
		// eliminar los parametros de esas busquedas y las busquedas
		for (Busquedas busqueda : listaBusquedas) {
			this.parametrosDao.removeList(busqueda.getIdBusqueda());
			this.busquedasDao.remove(busqueda);
		}
		// eliminar el listado
		this.listadosDao.remove(listados);
	}

	/**
	 * Finds a list of rows in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Listados>
	 */
	public List<Listados> findAll(Listados listados, JQGridRequestDto jqGridRequestDto) {
		return (List<Listados>) this.listadosDao.findAll(listados, jqGridRequestDto);
	}

	/**
	 * Finds rows in the Listados table using like.
	 *
	 * @param listados
	 *            Listados
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Listados>
	 */
	public List<Listados> findAllLike(Listados listados, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		return (List<Listados>) this.listadosDao.findAllLike(listados, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Filter method in the Listados table.
	 *
	 * @param filterListados
	 *            Listados
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<Listados>
	 */
	public JQGridResponseDto<Listados> filter(Listados filterListados, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		List<Listados> listaListados = this.listadosDao.findAllLike(filterListados, jqGridRequestDto, false);
		Long recordNum = this.listadosDao.findAllLikeCount(filterListados != null ? filterListados : new Listados(),
				false);
		if (jqGridRequestDto.getMultiselection().getSelectedIds() != null) {
			List<TableRowDto<Listados>> reorderSelection = this.listadosDao.reorderSelection(filterListados,
					jqGridRequestDto, startsWith);
			return new JQGridResponseDto<Listados>(jqGridRequestDto, recordNum, listaListados, reorderSelection);
		}
		return new JQGridResponseDto<Listados>(jqGridRequestDto, recordNum, listaListados);
	}

	/**
	 * Reorder the selection made in Listados table.
	 *
	 * @param filterListados
	 *            Listados
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return Object
	 */
	public Object reorderSelection(Listados filterListados, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		return this.listadosDao.reorderSelection(filterListados, jqGridRequestDto, startsWith);
	}

	public Listados removeFicheros(Listados listados) {
		return this.listadosDao.borrarFichero(listados);
	}

	/**
	 * Filter method in the RelLisPro table.
	 *
	 * @param filterRelLisPro
	 *            RelLisPro
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return JQGridResponseDto<RelLisPro>
	 */
	public JQGridResponseDto<RelLisPro> filterRelLisPro(RelLisPro filterRelLisPro, JQGridRequestDto jqGridRequestDto) {
		List<RelLisPro> listaRelLisPro = this.listadosDao.findListadosRelLisPro(filterRelLisPro, jqGridRequestDto);
		Long recordNum = this.listadosDao.findListadosRelLisProCount(filterRelLisPro);

		return new JQGridResponseDto<RelLisPro>(jqGridRequestDto, recordNum, listaRelLisPro);
	}

	/**
	 * Finds a single row in the RelLisPro table.
	 *
	 * @param idRelLisPro
	 *            Integer
	 * @return RelLisPro
	 */
	public RelLisPro findRelLisPro(Integer idRelLisPro) {
		return this.listadosDao.findRelLisPro(idRelLisPro);
	}

	/**
	 * Inserts a single row in the RelLisPro table.
	 *
	 * @param relLisPro
	 *            RelLisPro
	 * @return RelLisPro
	 */
	@Transactional(rollbackFor = Throwable.class)
	public RelLisPro addRelLisPro(RelLisPro relLisPro) {
		return this.listadosDao.addRelLisPro(relLisPro);
	}

	/**
	 * Updates a single row in the RelLisPro table.
	 *
	 * @param relLisPro
	 *            RelLisPro
	 * @return RelLisPro
	 */
	@Transactional(rollbackFor = Throwable.class)
	public RelLisPro updateRelLisPro(RelLisPro relLisPro) {
		return this.listadosDao.updateRelLisPro(relLisPro);
	}

	/**
	 * Deletes a single row in the RelLisPro table.
	 *
	 * @param idRelLisPro
	 *            Integer
	 */
	@Transactional(rollbackFor = Throwable.class)
	public void removeRelLisPro(Integer idRelLisPro) {

		// eliminar el listado
		this.listadosDao.removeRelLisPro(idRelLisPro);
	}
}
