package com.ejie.ab61.service.plantillas;

import com.ejie.ab61.model.plantillas.CamposPlantilla;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;

/**
 * CamposPlantillaService  
 * 
 *  
 */

public interface CamposPlantillaService {

	/**
	 * Inserts a single row in the CamposPlantilla table.
	 *
	 * @param camposPlantilla
	 *            CamposPlantilla
	 * @return CamposPlantilla
	 */
	CamposPlantilla add(CamposPlantilla camposPlantilla);

	/**
	 * Updates a single row in the CamposPlantilla table.
	 *
	 * @param camposPlantilla
	 *            CamposPlantilla
	 * @return CamposPlantilla
	 */
	CamposPlantilla update(CamposPlantilla camposPlantilla);

	/**
	 * Finds a single row in the CamposPlantilla table.
	 *
	 * @param camposPlantilla
	 *            CamposPlantilla
	 * @return CamposPlantilla
	 */
	CamposPlantilla find(CamposPlantilla camposPlantilla);

	/**
	 * Deletes a single row in the CamposPlantilla table.
	 *
	 * @param camposPlantilla
	 *            CamposPlantilla
	 */
	void remove(CamposPlantilla camposPlantilla);

	/**
	 * Finds a list of rows in the CamposPlantilla table.
	 *
	 * @param camposPlantilla
	 *            CamposPlantilla
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<CamposPlantilla>
	 */
	// List<CamposPlantilla> findCampos();

	/**
	 * Finds a list of rows in the CamposPlantilla table.
	 *
	 * @param camposPlantilla
	 *            CamposPlantilla
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<CamposPlantilla>
	 */
	List<CamposPlantilla> findAll(CamposPlantilla camposPlantilla, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds rows in the CamposPlantilla table using like.
	 *
	 * @param camposPlantilla
	 *            CamposPlantilla
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<CamposPlantilla>
	 */
	List<CamposPlantilla> findAllLike(CamposPlantilla camposPlantilla, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Removes rows from the CamposPlantilla table.
	 *
	 * @param filterCamposPlantilla
	 *            CamposPlantilla
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	void removeMultiple(CamposPlantilla filterCamposPlantilla, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Filter method in the CamposPlantilla table.
	 *
	 * @param filterCamposPlantilla
	 *            CamposPlantilla
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<CamposPlantilla>
	 */
	JQGridResponseDto<CamposPlantilla> filter(CamposPlantilla filterCamposPlantilla, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith);

	/**
	 * Searches rows in the CamposPlantilla table.
	 *
	 * @param filterCamposPlantilla
	 *            CamposPlantilla
	 * @param searchCamposPlantilla
	 *            CamposPlantilla
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<CamposPlantilla>>
	 */
	List<TableRowDto<CamposPlantilla>> search(CamposPlantilla filterCamposPlantilla,
			CamposPlantilla searchCamposPlantilla, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Reorder the selection made in CamposPlantilla table.
	 *
	 * @param filterCamposPlantilla
	 *            CamposPlantilla
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return Object
	 */
	Object reorderSelection(CamposPlantilla filterCamposPlantilla, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Finder method in the hierarchical the CamposPlantilla table.
	 *
	 * @param filterCamposPlantilla
	 *            CamposPlantilla
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<JerarquiaDto<CamposPlantilla>>
	 */
	JQGridResponseDto<JerarquiaDto<CamposPlantilla>> jerarquia(CamposPlantilla filterCamposPlantilla,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Finder method for siblings rows in the hierarchical the CamposPlantilla
	 * table.
	 *
	 * @param filterCamposPlantilla
	 *            CamposPlantilla
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<CamposPlantilla>>
	 */
	JQGridResponseDto<JerarquiaDto<CamposPlantilla>> jerarquiaChildren(CamposPlantilla filterCamposPlantilla,
			JQGridRequestDto jqGridRequestDto);

	/**
	 * Filter method in the CamposPlantilla table.
	 *
	 * @param idDocumento
	 *            Integer
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return JQGridResponseDto<CamposPlantilla>
	 */
	public JQGridResponseDto<CamposPlantilla> obtenerRelacionCamposDocumento(Integer idDocumento,
			JQGridRequestDto jqGridRequestDto);
}
