package com.ejie.ab61.service.plantillas;

import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;

import com.ejie.ab61.model.plantillas.Documentos;
import com.ejie.ab61.model.usuario.Usuario;

/**
 * DocumentosService  
 * 
 *  
 */

public interface DocumentosService {

	/**
	 * Inserts a single row in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @return Documentos
	 */
	Documentos add(Documentos documentos);

	/**
	 * Updates a single row in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @return Documentos
	 */
	Documentos update(Documentos documentos);

	/**
	 * Finds a single row in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @return Documentos
	 */
	Documentos find(Documentos documentos);

	/**
	 * Deletes a single row in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 */
	void remove(Documentos documentos);

	/**
	 * Finds a list of rows in the Documentos table.
	 *
	 * @param documentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Documentos>
	 */
	List<Documentos> findAll(Documentos documentos, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds rows in the Documentos table using like.
	 *
	 * @param documentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Documentos>
	 */
	List<Documentos> findAllLike(Documentos documentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Removes rows from the Documentos table.
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	void removeMultiple(Documentos filterDocumentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Filter method in the Documentos table.
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<Documentos>
	 */
	JQGridResponseDto<Documentos> filter(Documentos filterDocumentos, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith);

	/**
	 * Searches rows in the Documentos table.
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param searchDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Documentos>>
	 */
	List<TableRowDto<Documentos>> search(Documentos filterDocumentos, Documentos searchDocumentos,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Reorder the selection made in Documentos table.
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return Object
	 */
	Object reorderSelection(Documentos filterDocumentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Finder method in the hierarchical the Documentos table.
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<JerarquiaDto<Documentos>>
	 */
	JQGridResponseDto<JerarquiaDto<Documentos>> jerarquia(Documentos filterDocumentos,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Finder method for siblings rows in the hierarchical the Documentos table.
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<Documentos>>
	 */
	JQGridResponseDto<JerarquiaDto<Documentos>> jerarquiaChildren(Documentos filterDocumentos,
			JQGridRequestDto jqGridRequestDto);

	Documentos removeFicheros(Documentos documento);

	void copiarFicheros(Documentos documento, Usuario usuario);

	Documentos updateSinFichero(Documentos documento);

	/**
	 * Filter method in the Documentos table.
	 *
	 * @param filterDocumentos
	 *            Documentos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<Documentos>
	 */
	JQGridResponseDto<Documentos> documentosPorDireccion(Documentos filterDocumentos, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith, Integer direccion);

	List<Documentos> findAllPorDireccion(Documentos documentos, JQGridRequestDto jqGridRequestDto, Integer direccion);

	List<Documentos> findAllLikeDocumentosPorDireccion(Documentos filterCamposPlantilla,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith, Integer direccion);

}
