package com.ejie.ab61.service.plantillas;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.dao.plantillas.TipoCampoDao;
import com.ejie.ab61.model.plantillas.TipoCampo;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * TipoCampoServiceImpl  
 * 
 *  
 */

@Service(value = "tipoCampoService")
public class TipoCampoServiceImpl implements TipoCampoService {

	@Autowired
	private TipoCampoDao tipoCampoDao;

	/**
	 * Inserts a single row in the TipoCampo table.
	 *
	 * @param tipoCampo
	 *            TipoCampo
	 * @return TipoCampo
	 */
	@Transactional(rollbackFor = Throwable.class)
	public TipoCampo add(TipoCampo tipoCampo) {
		return this.tipoCampoDao.add(tipoCampo);
	}

	/**
	 * Updates a single row in the TipoCampo table.
	 *
	 * @param tipoCampo
	 *            TipoCampo
	 * @return TipoCampo
	 */
	@Transactional(rollbackFor = Throwable.class)
	public TipoCampo update(TipoCampo tipoCampo) {
		return this.tipoCampoDao.update(tipoCampo);
	}

	/**
	 * Finds a single row in the TipoCampo table.
	 *
	 * @param tipoCampo
	 *            TipoCampo
	 * @return TipoCampo
	 */
	public TipoCampo find(TipoCampo tipoCampo) {
		return (TipoCampo) this.tipoCampoDao.find(tipoCampo);
	}

	/**
	 * Deletes a single row in the TipoCampo table.
	 *
	 * @param tipoCampo
	 *            TipoCampo
	 */
	@Transactional(rollbackFor = Throwable.class)
	public void remove(TipoCampo tipoCampo) {
		this.tipoCampoDao.remove(tipoCampo);
	}

	/**
	 * Finds a list of rows in the TipoCampo table.
	 *
	 * @param tipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TipoCampo>
	 */
	public List<TipoCampo> findAll(TipoCampo tipoCampo, JQGridRequestDto jqGridRequestDto) {
		return (List<TipoCampo>) this.tipoCampoDao.findAll(tipoCampo, jqGridRequestDto);
	}

	/**
	 * Finds rows in the TipoCampo table using like.
	 *
	 * @param tipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TipoCampo>
	 */
	public List<TipoCampo> findAllLike(TipoCampo tipoCampo, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		return (List<TipoCampo>) this.tipoCampoDao.findAllLike(tipoCampo, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Removes rows from the TipoCampo table.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	public void removeMultiple(TipoCampo filterTipoCampo, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		this.tipoCampoDao.removeMultiple(filterTipoCampo, jqGridRequestDto, startsWith);
	}

	/**
	 * Filter method in the TipoCampo table.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<TipoCampo>
	 */
	public JQGridResponseDto<TipoCampo> filter(TipoCampo filterTipoCampo, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		List<TipoCampo> listaTipoCampo = this.tipoCampoDao.findAllLike(filterTipoCampo, jqGridRequestDto, false);
		Long recordNum = this.tipoCampoDao.findAllLikeCount(filterTipoCampo != null ? filterTipoCampo : new TipoCampo(),
				false);
		if (jqGridRequestDto.getMultiselection().getSelectedIds() != null) {
			List<TableRowDto<TipoCampo>> reorderSelection = this.tipoCampoDao.reorderSelection(filterTipoCampo,
					jqGridRequestDto, startsWith);
			return new JQGridResponseDto<TipoCampo>(jqGridRequestDto, recordNum, listaTipoCampo, reorderSelection);
		}
		return new JQGridResponseDto<TipoCampo>(jqGridRequestDto, recordNum, listaTipoCampo);
	}

	/**
	 * Searches rows in the TipoCampo table.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param searchTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<TipoCampo>>
	 */
	public List<TableRowDto<TipoCampo>> search(TipoCampo filterTipoCampo, TipoCampo searchTipoCampo,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		return this.tipoCampoDao.search(filterTipoCampo, searchTipoCampo, jqGridRequestDto, startsWith);
	}

	/**
	 * Reorder the selection made in TipoCampo table.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return Object
	 */
	public Object reorderSelection(TipoCampo filterTipoCampo, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		return this.tipoCampoDao.reorderSelection(filterTipoCampo, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Finder method in the hierarchical the TipoCampo table.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<JerarquiaDto<TipoCampo>>
	 */
	public JQGridResponseDto<JerarquiaDto<TipoCampo>> jerarquia(TipoCampo filterTipoCampo,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		List<JerarquiaDto<TipoCampo>> listaTipoCampo = this.tipoCampoDao.findAllLikeJerarquia(filterTipoCampo,
				jqGridRequestDto);
		Long recordNum = this.tipoCampoDao.findAllLikeCountJerarquia(filterTipoCampo, jqGridRequestDto);
		return new JQGridResponseDto<JerarquiaDto<TipoCampo>>(jqGridRequestDto, recordNum, listaTipoCampo);
	}

	/**
	 * Finder method for siblings rows in the hierarchical the TipoCampo table.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<TipoCampo>>
	 */
	public JQGridResponseDto<JerarquiaDto<TipoCampo>> jerarquiaChildren(TipoCampo filterTipoCampo,
			JQGridRequestDto jqGridRequestDto) {
		JQGridResponseDto<JerarquiaDto<TipoCampo>> jqGridResponseDto = new JQGridResponseDto<JerarquiaDto<TipoCampo>>();
		jqGridResponseDto.addAdditionalParam(JQGridResponseDto.CHILDREN,
				this.tipoCampoDao.findAllChild(filterTipoCampo, jqGridRequestDto));
		return jqGridResponseDto;
	}

}
