package com.ejie.ab61.service.procedimientos;


import com.ejie.ab61.model.procedimientos.Areas;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;

/**
 * AreasService  
 *  
 */

public interface AreasService {

	/**
	 * Inserts a single row in the Areas table.
	 *
	 * @param areas Areas
	 * @return Areas
	 */
    Areas add(Areas areas);

	/**
	 * Updates a single row in the Areas table.
	 *
	 * @param areas Areas
	 * @return Areas
	 */
	Areas update(Areas areas);

	/**
	 * Finds a single row in the Areas table.
	 *
	 * @param areas Areas
	 * @return Areas
	 */
	Areas find(Areas areas);

	/**
	 * Deletes a single row in the Areas table.
	 *
	 * @param areas Areas
	 */
	void remove(Areas areas);

	/**
	 * Finds a list of rows in the Areas table.
	 *
	 * @param areas Areas
	 * @param jqGridRequestDto JQGridRequestDto
	 * @return List<Areas>
	 */
	List<Areas> findAll(Areas areas, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds rows in the Areas table using like.
	 *
	 * @param areas Areas
	 * @param jqGridRequestDto JQGridRequestDto
     * @param startsWith Boolean	 
	 * @return List<Areas>
	 */
	List<Areas> findAllLike(Areas areas, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Removes rows from the Areas table.
	 *
	 * @param filterAreas Areas
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 */	
	void removeMultiple(Areas filterAreas, JQGridRequestDto jqGridRequestDto,  Boolean startsWith);
   
    
	/**
	 * Filter method in the Areas table.
	 *
	 * @param filterAreas Areas
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return JQGridResponseDto<Areas>
	 */	
	JQGridResponseDto<Areas> filter(Areas filterAreas, JQGridRequestDto jqGridRequestDto,  Boolean startsWith);
    
    /**
	 * Searches rows in the Areas table.
	 *
	 * @param filterAreas Areas
	 * @param searchAreas Areas
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return List<TableRowDto<Areas>>
	 */	
    List<TableRowDto<Areas>> search(Areas filterAreas, Areas searchAreas, JQGridRequestDto jqGridRequestDto, Boolean startsWith);
    
    /**
	 * Reorder the selection made in Areas table.
	 *
	 * @param filterAreas Areas
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return Object
	 */	
    Object reorderSelection(Areas filterAreas, JQGridRequestDto jqGridRequestDto, Boolean startsWith);
    
    /*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */
	 
	/**
	 * Finder method in the hierarchical the Areas table.
	 *
	 * @param filterAreas Areas
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return JQGridResponseDto<JerarquiaDto<Areas>>
	 */	
	JQGridResponseDto<JerarquiaDto<Areas>> jerarquia(Areas filterAreas, JQGridRequestDto jqGridRequestDto, Boolean startsWith);
	
	/**
	 * Finder method for siblings rows in the hierarchical the Areas table.
	 *
	 * @param filterAreas Areas
	 * @param jqGridRequestDto JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<Areas>>
	 */	
	JQGridResponseDto<JerarquiaDto<Areas>> jerarquiaChildren(Areas filterAreas, JQGridRequestDto jqGridRequestDto);
    
}


