package com.ejie.ab61.service.procedimientos;
import com.ejie.ab61.dao.procedimientos.DepartamentosDao;
import com.ejie.ab61.model.procedimientos.Departamentos;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * DepartamentosServiceImpl  
 *  
 */

@Service(value = "departamentosService")
public class DepartamentosServiceImpl implements DepartamentosService {

	@Autowired
	private DepartamentosDao departamentosDao;

	/**
	 * Inserts a single row in the Departamentos table.
	 *
	 * @param departamentos Departamentos
	 * @return Departamentos
	 */
	@Transactional(rollbackFor = Throwable.class)
	public Departamentos add(Departamentos departamentos) {
		return this.departamentosDao.add(departamentos);
	}

	/**
	 * Updates a single row in the Departamentos table.
	 *
	 * @param departamentos Departamentos
	 * @return Departamentos
	 */
	@Transactional(rollbackFor = Throwable.class)
	public Departamentos update(Departamentos departamentos) {
		return this.departamentosDao.update(departamentos);
	 }

	/**
	 * Finds a single row in the Departamentos table.
	 *
	 * @param departamentos Departamentos
	 * @return Departamentos
	 */
	public Departamentos find(Departamentos departamentos) {
		return (Departamentos) this.departamentosDao.find(departamentos);
	}
	
	/**
	 * Deletes a single row in the Departamentos table.
	 *
	 * @param departamentos Departamentos
	 */
	@Transactional(rollbackFor = Throwable.class)
	public void remove(Departamentos departamentos) {
		this.departamentosDao.remove(departamentos);
	}

	/**
	 * Finds a list of rows in the Departamentos table.
	 *
	 * @param departamentos Departamentos
	 * @param jqGridRequestDto JQGridRequestDto
	 * @return List<Departamentos>
	 */
	public List<Departamentos> findAll(Departamentos departamentos, JQGridRequestDto jqGridRequestDto){
		return (List<Departamentos>) this.departamentosDao.findAll(departamentos, jqGridRequestDto);
	}
    
	/**
	 * Finds rows in the Departamentos table using like.
	 *
	 * @param departamentos Departamentos
	 * @param jqGridRequestDto JQGridRequestDto
     * @param startsWith Boolean	 
	 * @return List<Departamentos>
	 */
	public List<Departamentos> findAllLike(Departamentos departamentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		return (List<Departamentos>) this.departamentosDao.findAllLike(departamentos, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */
	 
	/**
	 * Removes rows from the Departamentos table.
	 *
	 * @param filterDepartamentos Departamentos
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 */	
	public void removeMultiple(Departamentos filterDepartamentos, JQGridRequestDto jqGridRequestDto,  Boolean startsWith){
		this.departamentosDao.removeMultiple(filterDepartamentos, jqGridRequestDto, startsWith);
	}
        
	/**
	 * Filter method in the Departamentos table.
	 *
	 * @param filterDepartamentos Departamentos
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return JQGridResponseDto<Departamentos>
	 */	
	public JQGridResponseDto< Departamentos> filter(Departamentos filterDepartamentos, JQGridRequestDto jqGridRequestDto,  Boolean startsWith){
		List<Departamentos> listaDepartamentos =  this.departamentosDao.findAllLike(filterDepartamentos, jqGridRequestDto, false);
		Long recordNum =  this.departamentosDao.findAllLikeCount(filterDepartamentos != null ? filterDepartamentos: new Departamentos (),false);
		if (jqGridRequestDto.getMultiselection().getSelectedIds()!=null){
			List< TableRowDto< Departamentos>> reorderSelection = this.departamentosDao.reorderSelection(filterDepartamentos, jqGridRequestDto, startsWith);
			return new JQGridResponseDto<Departamentos>(jqGridRequestDto, recordNum, listaDepartamentos, reorderSelection);
		}
		return new JQGridResponseDto<Departamentos>(jqGridRequestDto, recordNum, listaDepartamentos);   
	}
    
    /**
	 * Searches rows in the Departamentos table.
	 *
	 * @param filterDepartamentos Departamentos
	 * @param searchDepartamentos Departamentos
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return List<TableRowDto<Departamentos>>
	 */	
    public List< TableRowDto< Departamentos>> search(Departamentos filterDepartamentos, Departamentos searchDepartamentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		return this.departamentosDao.search(filterDepartamentos, searchDepartamentos, jqGridRequestDto, startsWith);
	}
    
    /**
	 * Reorder the selection made in Departamentos table.
	 *
	 * @param filterDepartamentos Departamentos
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return Object
	 */	
    public Object reorderSelection(Departamentos filterDepartamentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		return this.departamentosDao.reorderSelection(filterDepartamentos, jqGridRequestDto, startsWith);
	}
    
    /*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */
	 
	/**
	 * Finder method in the hierarchical the Departamentos table.
	 *
	 * @param filterDepartamentos Departamentos
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return JQGridResponseDto<JerarquiaDto<Departamentos>>
	 */	
	public JQGridResponseDto<JerarquiaDto<Departamentos>> jerarquia(Departamentos filterDepartamentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		List<JerarquiaDto<Departamentos>> listaDepartamentos =  this.departamentosDao.findAllLikeJerarquia(filterDepartamentos, jqGridRequestDto);
		Long recordNum = this.departamentosDao.findAllLikeCountJerarquia(filterDepartamentos, jqGridRequestDto);
		return new JQGridResponseDto<JerarquiaDto<Departamentos>>(jqGridRequestDto, recordNum, listaDepartamentos);
	}
	
	/**
	 * Finder method for siblings rows in the hierarchical the Departamentos table.
	 *
	 * @param filterDepartamentos Departamentos
	 * @param jqGridRequestDto JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<Departamentos>>
	 */	
	public JQGridResponseDto<JerarquiaDto<Departamentos>> jerarquiaChildren(Departamentos filterDepartamentos, JQGridRequestDto jqGridRequestDto){
		JQGridResponseDto<JerarquiaDto<Departamentos>> jqGridResponseDto = new JQGridResponseDto<JerarquiaDto<Departamentos>>();
		jqGridResponseDto.addAdditionalParam(JQGridResponseDto.CHILDREN, this.departamentosDao.findAllChild(filterDepartamentos, jqGridRequestDto));
		return jqGridResponseDto;
	}
	
	
}

