package com.ejie.ab61.service.procedimientos;
import com.ejie.ab61.dao.procedimientos.DireccionesDao;
import com.ejie.ab61.model.procedimientos.Direcciones;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * DireccionesServiceImpl  
 *  
 */

@Service(value = "direccionesService")
public class DireccionesServiceImpl implements DireccionesService {

	@Autowired
	private DireccionesDao direccionesDao;

	/**
	 * Inserts a single row in the Direcciones table.
	 *
	 * @param direcciones Direcciones
	 * @return Direcciones
	 */
	@Transactional(rollbackFor = Throwable.class)
	public Direcciones add(Direcciones direcciones) {
		return this.direccionesDao.add(direcciones);
	}

	/**
	 * Updates a single row in the Direcciones table.
	 *
	 * @param direcciones Direcciones
	 * @return Direcciones
	 */
	@Transactional(rollbackFor = Throwable.class)
	public Direcciones update(Direcciones direcciones) {
		return this.direccionesDao.update(direcciones);
	 }

	/**
	 * Finds a single row in the Direcciones table.
	 *
	 * @param direcciones Direcciones
	 * @return Direcciones
	 */
	public Direcciones find(Direcciones direcciones) {
		return (Direcciones) this.direccionesDao.find(direcciones);
	}
	
	/**
	 * Deletes a single row in the Direcciones table.
	 *
	 * @param direcciones Direcciones
	 */
	@Transactional(rollbackFor = Throwable.class)
	public void remove(Direcciones direcciones) {
		this.direccionesDao.remove(direcciones);
	}

	/**
	 * Finds a list of rows in the Direcciones table.
	 *
	 * @param direcciones Direcciones
	 * @param jqGridRequestDto JQGridRequestDto
	 * @return List<Direcciones>
	 */
	public List<Direcciones> findAll(Direcciones direcciones, JQGridRequestDto jqGridRequestDto){
		return (List<Direcciones>) this.direccionesDao.findAll(direcciones, jqGridRequestDto);
	}
    
	/**
	 * Finds rows in the Direcciones table using like.
	 *
	 * @param direcciones Direcciones
	 * @param jqGridRequestDto JQGridRequestDto
     * @param startsWith Boolean	 
	 * @return List<Direcciones>
	 */
	public List<Direcciones> findAllLike(Direcciones direcciones, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		return (List<Direcciones>) this.direccionesDao.findAllLike(direcciones, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */
	 
	/**
	 * Removes rows from the Direcciones table.
	 *
	 * @param filterDirecciones Direcciones
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 */	
	public void removeMultiple(Direcciones filterDirecciones, JQGridRequestDto jqGridRequestDto,  Boolean startsWith){
		this.direccionesDao.removeMultiple(filterDirecciones, jqGridRequestDto, startsWith);
	}
        
	/**
	 * Filter method in the Direcciones table.
	 *
	 * @param filterDirecciones Direcciones
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return JQGridResponseDto<Direcciones>
	 */	
	public JQGridResponseDto< Direcciones> filter(Direcciones filterDirecciones, JQGridRequestDto jqGridRequestDto,  Boolean startsWith){
		List<Direcciones> listaDirecciones =  this.direccionesDao.findAllLike(filterDirecciones, jqGridRequestDto, false);
		Long recordNum =  this.direccionesDao.findAllLikeCount(filterDirecciones != null ? filterDirecciones: new Direcciones (),false);
		if (jqGridRequestDto.getMultiselection().getSelectedIds()!=null){
			List< TableRowDto< Direcciones>> reorderSelection = this.direccionesDao.reorderSelection(filterDirecciones, jqGridRequestDto, startsWith);
			return new JQGridResponseDto<Direcciones>(jqGridRequestDto, recordNum, listaDirecciones, reorderSelection);
		}
		return new JQGridResponseDto<Direcciones>(jqGridRequestDto, recordNum, listaDirecciones);   
	}
    
    /**
	 * Searches rows in the Direcciones table.
	 *
	 * @param filterDirecciones Direcciones
	 * @param searchDirecciones Direcciones
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return List<TableRowDto<Direcciones>>
	 */	
    public List< TableRowDto< Direcciones>> search(Direcciones filterDirecciones, Direcciones searchDirecciones, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		return this.direccionesDao.search(filterDirecciones, searchDirecciones, jqGridRequestDto, startsWith);
	}
    
    /**
	 * Reorder the selection made in Direcciones table.
	 *
	 * @param filterDirecciones Direcciones
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return Object
	 */	
    public Object reorderSelection(Direcciones filterDirecciones, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		return this.direccionesDao.reorderSelection(filterDirecciones, jqGridRequestDto, startsWith);
	}
    
    /*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */
	 
	/**
	 * Finder method in the hierarchical the Direcciones table.
	 *
	 * @param filterDirecciones Direcciones
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return JQGridResponseDto<JerarquiaDto<Direcciones>>
	 */	
	public JQGridResponseDto<JerarquiaDto<Direcciones>> jerarquia(Direcciones filterDirecciones, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		List<JerarquiaDto<Direcciones>> listaDirecciones =  this.direccionesDao.findAllLikeJerarquia(filterDirecciones, jqGridRequestDto);
		Long recordNum = this.direccionesDao.findAllLikeCountJerarquia(filterDirecciones, jqGridRequestDto);
		return new JQGridResponseDto<JerarquiaDto<Direcciones>>(jqGridRequestDto, recordNum, listaDirecciones);
	}
	
	/**
	 * Finder method for siblings rows in the hierarchical the Direcciones table.
	 *
	 * @param filterDirecciones Direcciones
	 * @param jqGridRequestDto JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<Direcciones>>
	 */	
	public JQGridResponseDto<JerarquiaDto<Direcciones>> jerarquiaChildren(Direcciones filterDirecciones, JQGridRequestDto jqGridRequestDto){
		JQGridResponseDto<JerarquiaDto<Direcciones>> jqGridResponseDto = new JQGridResponseDto<JerarquiaDto<Direcciones>>();
		jqGridResponseDto.addAdditionalParam(JQGridResponseDto.CHILDREN, this.direccionesDao.findAllChild(filterDirecciones, jqGridRequestDto));
		return jqGridResponseDto;
	}
	
	
}

