package com.ejie.ab61.service.solicitudes;
import com.ejie.ab61.dao.solicitudes.MensajesSolicitudDao;
import com.ejie.ab61.model.solicitudes.MensajesSolicitud;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * MensajesSolicitudServiceImpl  
 *  
 */

@Service(value = "mensajesSolicitudService")
public class MensajesSolicitudServiceImpl implements MensajesSolicitudService {

	@Autowired
	private MensajesSolicitudDao mensajesSolicitudDao;

	/**
	 * Inserts a single row in the MensajesSolicitud table.
	 *
	 * @param mensajesSolicitud MensajesSolicitud
	 * @return MensajesSolicitud
	 */
	@Transactional(rollbackFor = Throwable.class)
	public MensajesSolicitud add(MensajesSolicitud mensajesSolicitud) {
		return this.mensajesSolicitudDao.add(mensajesSolicitud);
	}

	/**
	 * Updates a single row in the MensajesSolicitud table.
	 *
	 * @param mensajesSolicitud MensajesSolicitud
	 * @return MensajesSolicitud
	 */
	@Transactional(rollbackFor = Throwable.class)
	public MensajesSolicitud update(MensajesSolicitud mensajesSolicitud) {
		return this.mensajesSolicitudDao.update(mensajesSolicitud);
	 }

	/**
	 * Finds a single row in the MensajesSolicitud table.
	 *
	 * @param mensajesSolicitud MensajesSolicitud
	 * @return MensajesSolicitud
	 */
	public MensajesSolicitud find(MensajesSolicitud mensajesSolicitud) {
		return (MensajesSolicitud) this.mensajesSolicitudDao.find(mensajesSolicitud);
	}
	
	/**
	 * Deletes a single row in the MensajesSolicitud table.
	 *
	 * @param mensajesSolicitud MensajesSolicitud
	 */
	@Transactional(rollbackFor = Throwable.class)
	public void remove(MensajesSolicitud mensajesSolicitud) {
		this.mensajesSolicitudDao.remove(mensajesSolicitud);
	}

	/**
	 * Finds a list of rows in the MensajesSolicitud table.
	 *
	 * @param mensajesSolicitud MensajesSolicitud
	 * @param jqGridRequestDto JQGridRequestDto
	 * @return List<MensajesSolicitud>
	 */
	public List<MensajesSolicitud> findAll(MensajesSolicitud mensajesSolicitud, JQGridRequestDto jqGridRequestDto){
		return (List<MensajesSolicitud>) this.mensajesSolicitudDao.findAll(mensajesSolicitud, jqGridRequestDto);
	}
    
	/**
	 * Finds rows in the MensajesSolicitud table using like.
	 *
	 * @param mensajesSolicitud MensajesSolicitud
	 * @param jqGridRequestDto JQGridRequestDto
     * @param startsWith Boolean	 
	 * @return List<MensajesSolicitud>
	 */
	public List<MensajesSolicitud> findAllLike(MensajesSolicitud mensajesSolicitud, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		return (List<MensajesSolicitud>) this.mensajesSolicitudDao.findAllLike(mensajesSolicitud, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */
	 
	/**
	 * Removes rows from the MensajesSolicitud table.
	 *
	 * @param filterMensajesSolicitud MensajesSolicitud
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 */	
	public void removeMultiple(MensajesSolicitud filterMensajesSolicitud, JQGridRequestDto jqGridRequestDto,  Boolean startsWith){
		this.mensajesSolicitudDao.removeMultiple(filterMensajesSolicitud, jqGridRequestDto, startsWith);
	}
        
	/**
	 * Filter method in the MensajesSolicitud table.
	 *
	 * @param filterMensajesSolicitud MensajesSolicitud
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return JQGridResponseDto<MensajesSolicitud>
	 */	
	public JQGridResponseDto< MensajesSolicitud> filter(MensajesSolicitud filterMensajesSolicitud, JQGridRequestDto jqGridRequestDto,  Boolean startsWith){
		List<MensajesSolicitud> listaMensajesSolicitud =  this.mensajesSolicitudDao.findAllLike(filterMensajesSolicitud, jqGridRequestDto, false);
		Long recordNum =  this.mensajesSolicitudDao.findAllLikeCount(filterMensajesSolicitud != null ? filterMensajesSolicitud: new MensajesSolicitud (),false);
		if (jqGridRequestDto.getMultiselection().getSelectedIds()!=null){
			List< TableRowDto< MensajesSolicitud>> reorderSelection = this.mensajesSolicitudDao.reorderSelection(filterMensajesSolicitud, jqGridRequestDto, startsWith);
			return new JQGridResponseDto<MensajesSolicitud>(jqGridRequestDto, recordNum, listaMensajesSolicitud, reorderSelection);
		}
		return new JQGridResponseDto<MensajesSolicitud>(jqGridRequestDto, recordNum, listaMensajesSolicitud);   
	}
    
    /**
	 * Searches rows in the MensajesSolicitud table.
	 *
	 * @param filterMensajesSolicitud MensajesSolicitud
	 * @param searchMensajesSolicitud MensajesSolicitud
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return List<TableRowDto<MensajesSolicitud>>
	 */	
    public List< TableRowDto< MensajesSolicitud>> search(MensajesSolicitud filterMensajesSolicitud, MensajesSolicitud searchMensajesSolicitud, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		return this.mensajesSolicitudDao.search(filterMensajesSolicitud, searchMensajesSolicitud, jqGridRequestDto, startsWith);
	}
    
    /**
	 * Reorder the selection made in MensajesSolicitud table.
	 *
	 * @param filterMensajesSolicitud MensajesSolicitud
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return Object
	 */	
    public Object reorderSelection(MensajesSolicitud filterMensajesSolicitud, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		return this.mensajesSolicitudDao.reorderSelection(filterMensajesSolicitud, jqGridRequestDto, startsWith);
	}
    
    /*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */
	 
	/**
	 * Finder method in the hierarchical the MensajesSolicitud table.
	 *
	 * @param filterMensajesSolicitud MensajesSolicitud
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return JQGridResponseDto<JerarquiaDto<MensajesSolicitud>>
	 */	
	public JQGridResponseDto<JerarquiaDto<MensajesSolicitud>> jerarquia(MensajesSolicitud filterMensajesSolicitud, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		List<JerarquiaDto<MensajesSolicitud>> listaMensajesSolicitud =  this.mensajesSolicitudDao.findAllLikeJerarquia(filterMensajesSolicitud, jqGridRequestDto);
		Long recordNum = this.mensajesSolicitudDao.findAllLikeCountJerarquia(filterMensajesSolicitud, jqGridRequestDto);
		return new JQGridResponseDto<JerarquiaDto<MensajesSolicitud>>(jqGridRequestDto, recordNum, listaMensajesSolicitud);
	}
	
	/**
	 * Finder method for siblings rows in the hierarchical the MensajesSolicitud table.
	 *
	 * @param filterMensajesSolicitud MensajesSolicitud
	 * @param jqGridRequestDto JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<MensajesSolicitud>>
	 */	
	public JQGridResponseDto<JerarquiaDto<MensajesSolicitud>> jerarquiaChildren(MensajesSolicitud filterMensajesSolicitud, JQGridRequestDto jqGridRequestDto){
		JQGridResponseDto<JerarquiaDto<MensajesSolicitud>> jqGridResponseDto = new JQGridResponseDto<JerarquiaDto<MensajesSolicitud>>();
		jqGridResponseDto.addAdditionalParam(JQGridResponseDto.CHILDREN, this.mensajesSolicitudDao.findAllChild(filterMensajesSolicitud, jqGridRequestDto));
		return jqGridResponseDto;
	}
	
	
}

