package com.ejie.ab61.service.terceros;
import com.ejie.ab61.dao.terceros.TiposTerceroDao;
import com.ejie.ab61.model.terceros.TiposTercero;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * TiposTerceroServiceImpl  
 *  
 */

@Service(value = "tiposTerceroService")
public class TiposTerceroServiceImpl implements TiposTerceroService {

	@Autowired
	private TiposTerceroDao tiposTerceroDao;

	/**
	 * Inserts a single row in the TiposTercero table.
	 *
	 * @param tiposTercero TiposTercero
	 * @return TiposTercero
	 */
	@Transactional(rollbackFor = Throwable.class)
	public TiposTercero add(TiposTercero tiposTercero) {
		return this.tiposTerceroDao.add(tiposTercero);
	}

	/**
	 * Updates a single row in the TiposTercero table.
	 *
	 * @param tiposTercero TiposTercero
	 * @return TiposTercero
	 */
	@Transactional(rollbackFor = Throwable.class)
	public TiposTercero update(TiposTercero tiposTercero) {
		return this.tiposTerceroDao.update(tiposTercero);
	 }

	/**
	 * Finds a single row in the TiposTercero table.
	 *
	 * @param tiposTercero TiposTercero
	 * @return TiposTercero
	 */
	public TiposTercero find(TiposTercero tiposTercero) {
		return (TiposTercero) this.tiposTerceroDao.find(tiposTercero);
	}
	
	/**
	 * Deletes a single row in the TiposTercero table.
	 *
	 * @param tiposTercero TiposTercero
	 */
	@Transactional(rollbackFor = Throwable.class)
	public void remove(TiposTercero tiposTercero) {
		this.tiposTerceroDao.remove(tiposTercero);
	}

	/**
	 * Finds a list of rows in the TiposTercero table.
	 *
	 * @param tiposTercero TiposTercero
	 * @param jqGridRequestDto JQGridRequestDto
	 * @return List<TiposTercero>
	 */
	public List<TiposTercero> findAll(TiposTercero tiposTercero, JQGridRequestDto jqGridRequestDto){
		return (List<TiposTercero>) this.tiposTerceroDao.findAll(tiposTercero, jqGridRequestDto);
	}
    
	/**
	 * Finds rows in the TiposTercero table using like.
	 *
	 * @param tiposTercero TiposTercero
	 * @param jqGridRequestDto JQGridRequestDto
     * @param startsWith Boolean	 
	 * @return List<TiposTercero>
	 */
	public List<TiposTercero> findAllLike(TiposTercero tiposTercero, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		return (List<TiposTercero>) this.tiposTerceroDao.findAllLike(tiposTercero, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */
	 
	/**
	 * Removes rows from the TiposTercero table.
	 *
	 * @param filterTiposTercero TiposTercero
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 */	
	public void removeMultiple(TiposTercero filterTiposTercero, JQGridRequestDto jqGridRequestDto,  Boolean startsWith){
		this.tiposTerceroDao.removeMultiple(filterTiposTercero, jqGridRequestDto, startsWith);
	}
        
	/**
	 * Filter method in the TiposTercero table.
	 *
	 * @param filterTiposTercero TiposTercero
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return JQGridResponseDto<TiposTercero>
	 */	
	public JQGridResponseDto< TiposTercero> filter(TiposTercero filterTiposTercero, JQGridRequestDto jqGridRequestDto,  Boolean startsWith){
		List<TiposTercero> listaTiposTercero =  this.tiposTerceroDao.findAllLike(filterTiposTercero, jqGridRequestDto, false);
		Long recordNum =  this.tiposTerceroDao.findAllLikeCount(filterTiposTercero != null ? filterTiposTercero: new TiposTercero (),false);
		if (jqGridRequestDto.getMultiselection().getSelectedIds()!=null){
			List< TableRowDto< TiposTercero>> reorderSelection = this.tiposTerceroDao.reorderSelection(filterTiposTercero, jqGridRequestDto, startsWith);
			return new JQGridResponseDto<TiposTercero>(jqGridRequestDto, recordNum, listaTiposTercero, reorderSelection);
		}
		return new JQGridResponseDto<TiposTercero>(jqGridRequestDto, recordNum, listaTiposTercero);   
	}
    
    /**
	 * Searches rows in the TiposTercero table.
	 *
	 * @param filterTiposTercero TiposTercero
	 * @param searchTiposTercero TiposTercero
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return List<TableRowDto<TiposTercero>>
	 */	
    public List< TableRowDto< TiposTercero>> search(TiposTercero filterTiposTercero, TiposTercero searchTiposTercero, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		return this.tiposTerceroDao.search(filterTiposTercero, searchTiposTercero, jqGridRequestDto, startsWith);
	}
    
    /**
	 * Reorder the selection made in TiposTercero table.
	 *
	 * @param filterTiposTercero TiposTercero
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return Object
	 */	
    public Object reorderSelection(TiposTercero filterTiposTercero, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		return this.tiposTerceroDao.reorderSelection(filterTiposTercero, jqGridRequestDto, startsWith);
	}
    
    /*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */
	 
	/**
	 * Finder method in the hierarchical the TiposTercero table.
	 *
	 * @param filterTiposTercero TiposTercero
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return JQGridResponseDto<JerarquiaDto<TiposTercero>>
	 */	
	public JQGridResponseDto<JerarquiaDto<TiposTercero>> jerarquia(TiposTercero filterTiposTercero, JQGridRequestDto jqGridRequestDto, Boolean startsWith){
		List<JerarquiaDto<TiposTercero>> listaTiposTercero =  this.tiposTerceroDao.findAllLikeJerarquia(filterTiposTercero, jqGridRequestDto);
		Long recordNum = this.tiposTerceroDao.findAllLikeCountJerarquia(filterTiposTercero, jqGridRequestDto);
		return new JQGridResponseDto<JerarquiaDto<TiposTercero>>(jqGridRequestDto, recordNum, listaTiposTercero);
	}
	
	/**
	 * Finder method for siblings rows in the hierarchical the TiposTercero table.
	 *
	 * @param filterTiposTercero TiposTercero
	 * @param jqGridRequestDto JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<TiposTercero>>
	 */	
	public JQGridResponseDto<JerarquiaDto<TiposTercero>> jerarquiaChildren(TiposTercero filterTiposTercero, JQGridRequestDto jqGridRequestDto){
		JQGridResponseDto<JerarquiaDto<TiposTercero>> jqGridResponseDto = new JQGridResponseDto<JerarquiaDto<TiposTercero>>();
		jqGridResponseDto.addAdditionalParam(JQGridResponseDto.CHILDREN, this.tiposTerceroDao.findAllChild(filterTiposTercero, jqGridRequestDto));
		return jqGridResponseDto;
	}
	
	
}

