package com.ejie.ab61.control.busquedas;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.ejie.ab61.model.busquedas.Busquedas;
import com.ejie.ab61.model.usuario.Usuario;
import com.ejie.ab61.service.busquedas.BusquedasService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;

/**
 * ListadosController  
 * 
 *  
 */

@Controller
@SessionAttributes("usuario")
@RequestMapping(value = "/busquedas")

public class BusquedasController {

	private static final Logger logger = LoggerFactory.getLogger(BusquedasController.class);

	@Autowired
	private BusquedasService busquedasService;

	// /*
	// * OPERACIONES CRUD (Create, Read, Update, Delete)
	// *
	// */
	//
	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 *
	 * @param idListado
	 *            Integer
	 * @return Busquedas Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idListado}/{idBusqueda}", method = RequestMethod.GET)
	public @ResponseBody Busquedas get(@PathVariable Integer idListado, @PathVariable Integer idBusqueda) {
		Busquedas busquedas = new Busquedas();
		busquedas.setIdBusqueda(idBusqueda);
		busquedas = this.busquedasService.find(busquedas);
		BusquedasController.logger.info("[GET - findBy_PK] : Obtener Busquedas porPK");
		return busquedas;
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param busquedas
	 *            Listados Bean que contiene la informacion a modificar.
	 * @return Busquedas Bean resultante de la modificacion.
	 */
	@RequestMapping(value = "/{idListado}", method = RequestMethod.PUT)
	public @ResponseBody Busquedas edit(@PathVariable Integer idListado, @RequestBody Busquedas busquedas,
			@ModelAttribute(value = "usuario") Usuario usuario) {
		busquedas.setIdListado(idListado);
		busquedas.setUsuarioUltModificacion(usuario.getNombreUsuario());
		Busquedas busquedasAux = this.busquedasService.update(busquedas);
		BusquedasController.logger.info("[PUT] : Busquedas actualizado correctamente");
		return busquedasAux;
	}

	//
	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param busquedas
	 *            Busquedas Bean que contiene la informacion con la que se va a
	 *            crear el nuevo registro.
	 * @return Busquedas Bean resultante del proceso de creacion.
	 */
	@RequestMapping(value = "/{idListado}", method = RequestMethod.POST)
	public @ResponseBody Busquedas add(@PathVariable Integer idListado, @RequestBody Busquedas busquedas,
			@ModelAttribute(value = "usuario") Usuario usuario) {
		busquedas.setUsuarioRegistro(usuario.getNombreUsuario());
		Busquedas busquedasAux = this.busquedasService.add(busquedas);
		BusquedasController.logger.info("[POST] : Busquedas insertado correctamente");
		return busquedasAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idListado
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return Busquedas Bean eliminado.
	 */
	@RequestMapping(value = "/{idListado}/{idBusqueda}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody Busquedas remove(@PathVariable Integer idListado, @PathVariable Integer idBusqueda) {
		Busquedas busquedas = new Busquedas();
		busquedas.setIdBusqueda(idBusqueda);
		this.busquedasService.remove(busquedas);
		BusquedasController.logger.info("[DELETE] : Listados borrado correctamente");
		return busquedas;
	}

	//
	// /*
	// * METODOS COMPONENTE RUP_TABLE
	// *
	// */

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterBusquedas
	 *            Busquedas Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Listados> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/{idListado}/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Busquedas> filter(
			@RequestJsonBody(param = "filter") Busquedas filterBusquedas, @PathVariable Integer idListado,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		BusquedasController.logger.info("[POST - filter] : Obtener Busquedas");
		if (null == filterBusquedas.getIdListado()) {
			filterBusquedas.setIdListado(idListado);
		}
		return this.busquedasService.filter(filterBusquedas, jqGridRequestDto, false);
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterParametros
	 *            Busquedas Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<Busquedas> Lista de objetos correspondientes a la busqueda
	 *         realizada.
	 */
	@RequestMapping(value = "/{idListado}", method = RequestMethod.GET)
	public @ResponseBody List<Busquedas> getAll(Model model, @ModelAttribute Busquedas filterBusquedas,
			@PathVariable Integer idListado) {
		BusquedasController.logger.info("[GET - find_ALL] : Obtener busquedas por filtro");
		filterBusquedas.setIdListado(idListado);
		return busquedasService.findAllLike(filterBusquedas, null, null);
	}
}
