package com.ejie.ab61.control.expedientes;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.ejie.ab61.model.expedientes.Expedientes;
import com.ejie.ab61.service.expedientes.ExpedientesService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * ExpedientesController  
 * 
 *  
 */

@Controller
@SessionAttributes("usuario")
@RequestMapping(value = "/expedientes")

public class ExpedientesController {

	private static final Logger logger = LoggerFactory.getLogger(ExpedientesController.class);

	@Autowired
	private ExpedientesService expedientesService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idExpediente
	 *            Integer
	 * @return Expedientes Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idExpediente}", method = RequestMethod.GET)
	public @ResponseBody Expedientes get(@PathVariable Integer idExpediente) {
		Expedientes expedientes = new Expedientes();
		expedientes.setIdExpediente(idExpediente);
		expedientes = this.expedientesService.find(expedientes);
		ExpedientesController.logger.info("[GET - findBy_PK] : Obtener Expedientes por PK");
		return expedientes;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterExpedientes
	 *            Expedientes Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<Expedientes> Lista de objetos correspondientes a la busqueda
	 *         realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<Expedientes> getAll(@ModelAttribute Expedientes filterExpedientes) {
		ExpedientesController.logger.info("[GET - find_ALL] : Obtener Expedientes por filtro");
		return this.expedientesService.findAll(filterExpedientes, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param expedientes
	 *            Expedientes Bean que contiene la informacion a modificar.
	 * @return Expedientes Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody Expedientes edit(@RequestBody Expedientes expedientes) {
		Expedientes expedientesAux = this.expedientesService.update(expedientes);
		ExpedientesController.logger.info("[PUT] : Expedientes actualizado correctamente");
		return expedientesAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param expedientes
	 *            Expedientes Bean que contiene la informacion con la que se va
	 *            a crear el nuevo registro.
	 * @return Expedientes Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody Expedientes add(@RequestBody Expedientes expedientes) {
		Expedientes expedientesAux = this.expedientesService.add(expedientes);
		ExpedientesController.logger.info("[POST] : Expedientes insertado correctamente");
		return expedientesAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idExpediente
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return Expedientes Bean eliminado.
	 */
	@RequestMapping(value = "/{idExpediente}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody Expedientes remove(@PathVariable Integer idExpediente) {
		Expedientes expedientes = new Expedientes();
		expedientes.setIdExpediente(idExpediente);
		this.expedientesService.remove(expedientes);
		ExpedientesController.logger.info("[DELETE] : Expedientes borrado correctamente");
		return expedientes;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	// @RequestMapping(value = "/maint/{folderId}", method = RequestMethod.GET)
	// public RelExpTerEmp getFormEdit(Model model, @PathVariable String
	// folderId,
	// @ModelAttribute(value = "usuario") Usuario usuario) {
	// ExpedientesController.logger.info("[GET - View] : expedientes");
	//
	// Expedientes expediente = expedientesService.findByFolderId(folderId);
	//
	// TFolderList resultado = null;
	// try {
	// resultado = expedientesService.obtenerInfoPlatea(folderId);
	// } catch (Exception_Exception e) {
	// // TODO Auto-generated catch block
	// e.printStackTrace();
	// }
	//
	// RelExpTerEmp titular = relExpTerEmpService.obtenerTitular(expediente,
	// resultado, usuario);
	// RelExpTerEmp representante =
	// relExpTerEmpService.obtenerRepresentante(expediente, resultado, usuario);
	//
	// model.addAttribute("txtTitular", titular);
	// model.addAttribute("txtRepresentante", representante);
	//
	// return titular;
	// }

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterExpedientes
	 *            Expedientes Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Expedientes> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Expedientes> filter(
			@RequestJsonBody(param = "filter") Expedientes filterExpedientes,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ExpedientesController.logger.info("[POST - filter] : Obtener Expedientess");
		return this.expedientesService.filter(filterExpedientes, jqGridRequestDto, false);
	}

	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterExpedientes
	 *            Expedientes Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param searchExpedientes
	 *            Expedientes Bean que contiene los parametros de busqueda a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<Expedientes> Dto que contiene el resultado de la
	 *         busqueda realizada por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<Expedientes>> search(
			@RequestJsonBody(param = "filter") Expedientes filterExpedientes,
			@RequestJsonBody(param = "search") Expedientes searchExpedientes,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ExpedientesController.logger.info("[POST - search] : Buscar Expedientess");
		return this.expedientesService.search(filterExpedientes, searchExpedientes, jqGridRequestDto, false);
	}

	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterExpedientes
	 *            Expedientes Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String> Lista de los identificadores de los registros
	 *         eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(@RequestJsonBody(param = "filter") Expedientes filterExpedientes,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ExpedientesController.logger
				.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples Expedientess");
		this.expedientesService.removeMultiple(filterExpedientes, jqGridRequestDto, false);
		ExpedientesController.logger.info("All entities correctly deleted!");

		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterExpedientes
	 *            Expedientes Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Expedientes>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Expedientes>> jerarquia(
			@RequestJsonBody(param = "filter") Expedientes filterExpedientes,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ExpedientesController.logger.info("[POST - jerarquia] : Obtener Expedientess jerarquia");
		return this.expedientesService.jerarquia(filterExpedientes, jqGridRequestDto, false);
	}

	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion
	 * jerarquica.
	 * 
	 * @param filterExpedientes
	 *            Expedientes Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Expedientes>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Expedientes>> jerarquiaChildren(
			@RequestJsonBody(param = "filter") Expedientes filterExpedientes,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ExpedientesController.logger.info("[POST - jerarquia] : Obtener Expedientess jerarquia - Hijos");
		return this.expedientesService.jerarquiaChildren(filterExpedientes, jqGridRequestDto);
	}

}
