package com.ejie.ab61.control.expedientes;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.context.support.ReloadableResourceBundleMessageSource;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;

import com.ejie.ab61.model.expedientes.FechasExpediente;
import com.ejie.ab61.model.usuario.Usuario;
import com.ejie.ab61.service.expedientes.FechasExpedienteService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import com.ejie.x38.reports.ReportData;

/**
 * FechasExpedienteController  
 * 
 *  
 */

@Controller
@SessionAttributes("usuario")
@RequestMapping(value = "/fechasexpediente")

public class FechasExpedienteController {

	private static final Logger logger = LoggerFactory.getLogger(FechasExpedienteController.class);

	@Autowired
	private FechasExpedienteService fechasExpedienteService;

	@Resource
	private ReloadableResourceBundleMessageSource appMessageSource;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idFechaExpediente
	 *            Integer
	 * @return FechasExpediente Objeto correspondiente al identificador
	 *         indicado.
	 */
	@RequestMapping(value = "/{idFechaExpediente}", method = RequestMethod.GET)
	public @ResponseBody FechasExpediente get(@PathVariable Integer idFechaExpediente) {
		FechasExpediente fechasExpediente = new FechasExpediente();
		fechasExpediente.setIdFechaExpediente(idFechaExpediente);
		fechasExpediente = this.fechasExpedienteService.find(fechasExpediente);
		FechasExpedienteController.logger.info("[GET - findBy_PK] : Obtener FechasExpediente por PK");
		return fechasExpediente;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterFechasExpediente
	 *            FechasExpediente Objeto que contiene los parametros de
	 *            filtrado utilizados en la busqueda.
	 * @return List<FechasExpediente> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<FechasExpediente> getAll(@ModelAttribute FechasExpediente filterFechasExpediente) {
		FechasExpedienteController.logger.info("[GET - find_ALL] : Obtener FechasExpediente por filtro");
		return this.fechasExpedienteService.findAll(filterFechasExpediente, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param fechasExpediente
	 *            FechasExpediente Bean que contiene la informacion a modificar.
	 * @return FechasExpediente Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody FechasExpediente edit(@RequestBody FechasExpediente fechasExpediente, Usuario usuario) {

		Date fechaHoy = new Date();
		fechasExpediente.setFechaUltmodificacion(fechaHoy);
		fechasExpediente.setUsuarioUltmodificacion(usuario.getNombreUsuario());

		FechasExpediente fechasExpedienteAux = this.fechasExpedienteService.update(fechasExpediente);
		FechasExpedienteController.logger.info("[PUT] : FechasExpediente actualizado correctamente");
		return fechasExpedienteAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param fechasExpediente
	 *            FechasExpediente Bean que contiene la informacion con la que
	 *            se va a crear el nuevo registro.
	 * @return FechasExpediente Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody FechasExpediente add(@RequestBody FechasExpediente fechasExpediente, Usuario usuario) {

		Date fechaHoy = new Date();
		fechasExpediente.setFechaRegistro(fechaHoy);
		fechasExpediente.setUsuarioRegistro(usuario.getNombreUsuario());

		FechasExpediente fechasExpedienteAux = this.fechasExpedienteService.add(fechasExpediente);
		FechasExpedienteController.logger.info("[POST] : FechasExpediente insertado correctamente");
		return fechasExpedienteAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idFechaExpediente
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return FechasExpediente Bean eliminado.
	 */
	@RequestMapping(value = "/{idFechaExpediente}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody FechasExpediente remove(@PathVariable Integer idFechaExpediente) {
		FechasExpediente fechasExpediente = new FechasExpediente();
		fechasExpediente.setIdFechaExpediente(idFechaExpediente);
		this.fechasExpedienteService.remove(fechasExpediente);
		FechasExpedienteController.logger.info("[DELETE] : FechasExpediente borrado correctamente");
		return fechasExpediente;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		FechasExpedienteController.logger.info("[GET - View] : fechasexpediente");
		return "fechasexpediente";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterFechasExpediente
	 *            FechasExpediente Bean que contiene los parametros de filtrado
	 *            a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<FechasExpediente> Dto que contiene el resultado
	 *         del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<FechasExpediente> filter(
			@RequestJsonBody(param = "filter") FechasExpediente filterFechasExpediente,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		FechasExpedienteController.logger.info("[POST - filter] : Obtener FechasExpedientes");
		return this.fechasExpedienteService.filter(filterFechasExpediente, jqGridRequestDto, false);
	}

	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterFechasExpediente
	 *            FechasExpediente Bean que contiene los parametros de filtrado
	 *            a emplear.
	 * @param searchFechasExpediente
	 *            FechasExpediente Bean que contiene los parametros de busqueda
	 *            a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<FechasExpediente> Dto que contiene el resultado de la
	 *         busqueda realizada por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<FechasExpediente>> search(
			@RequestJsonBody(param = "filter") FechasExpediente filterFechasExpediente,
			@RequestJsonBody(param = "search") FechasExpediente searchFechasExpediente,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		FechasExpedienteController.logger.info("[POST - search] : Buscar FechasExpedientes");
		return this.fechasExpedienteService.search(filterFechasExpediente, searchFechasExpediente, jqGridRequestDto,
				false);
	}

	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterFechasExpediente
	 *            FechasExpediente Bean que contiene los parametros de filtrado
	 *            a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String> Lista de los identificadores de los registros
	 *         eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(
			@RequestJsonBody(param = "filter") FechasExpediente filterFechasExpediente,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		FechasExpedienteController.logger
				.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples FechasExpedientes");
		this.fechasExpedienteService.removeMultiple(filterFechasExpediente, jqGridRequestDto, false);
		FechasExpedienteController.logger.info("All entities correctly deleted!");

		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterFechasExpediente
	 *            FechasExpediente Bean que contiene los parametros de filtrado
	 *            a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<FechasExpediente>> Dto que
	 *         contiene el resultado del filtrado realizado por el componente
	 *         RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<FechasExpediente>> jerarquia(
			@RequestJsonBody(param = "filter") FechasExpediente filterFechasExpediente,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		FechasExpedienteController.logger.info("[POST - jerarquia] : Obtener FechasExpedientes jerarquia");
		return this.fechasExpedienteService.jerarquia(filterFechasExpediente, jqGridRequestDto, false);
	}

	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion
	 * jerarquica.
	 * 
	 * @param filterFechasExpediente
	 *            FechasExpediente Bean que contiene los parametros de filtrado
	 *            a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<FechasExpediente>> Dto que
	 *         contiene el resultado del filtrado realizado por el componente
	 *         RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<FechasExpediente>> jerarquiaChildren(
			@RequestJsonBody(param = "filter") FechasExpediente filterFechasExpediente,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		FechasExpedienteController.logger.info("[POST - jerarquia] : Obtener FechasExpedientes jerarquia - Hijos");
		return this.fechasExpedienteService.jerarquiaChildren(filterFechasExpediente, jqGridRequestDto);
	}

	@RequestMapping(value = { "/xlsReport", "/xlsxReport" }, method = RequestMethod.GET)
	public ModelAndView getExcelPOI(@ModelAttribute FechasExpediente filterFechasExpediente,
			@ModelAttribute JQGridRequestDto jqGridRequestDto, ModelMap modelMap,
			@RequestParam(value = "columns", required = false) String columns, HttpServletRequest request) {

		Locale locale = LocaleContextHolder.getLocale();

		// Acceso a BD para recuperar datos

		columns = "[['tipoEvento','" + appMessageSource.getMessage("fechasExpedienteExcel.tipoEvento", null, locale) + "']"
				+ ",['plateaType','" + appMessageSource.getMessage("fechasExpedienteExcel.plateaType", null, locale) + "']"
				+ ",['plateaSubtype','" + appMessageSource.getMessage("fechasExpedienteExcel.plateaSubtype", null, locale) + "']"
				+ ",['tipoDokusi','" + appMessageSource.getMessage("fechasExpedienteExcel.tipoDokusi", null, locale) + "']"
				+ ",['subtipoDokusi','" + appMessageSource.getMessage("fechasExpedienteExcel.subtipoDokusi", null, locale) + "']"
				+ ",['asuntoDocumental','" + appMessageSource.getMessage("fechasExpedienteExcel.asuntoDocumental", null, locale) + "']"
				+ ",['taskId','" + appMessageSource.getMessage("fechasExpedienteExcel.taskId", null, locale) + "']"
				+ ",['situacionId','" + appMessageSource.getMessage("fechasExpedienteExcel.situacionId", null, locale) + "']"
				+ ",['status','" + appMessageSource.getMessage("fechasExpedienteExcel.status", null, locale) + "']"
				+ ",['etiquetaFecha','" + appMessageSource.getMessage("fechasExpedienteExcel.etiquetaFecha", null, locale) + "']"
				+ ",['descripcionCastellano','" + appMessageSource.getMessage("fechasExpedienteExcel.descripcionCastellano", null, locale) + "']"
				+ ",['descripcionEuskera','" + appMessageSource.getMessage("fechasExpedienteExcel.descripcionEuskera", null, locale) + "']"
				+ ",['instanceId','" + appMessageSource.getMessage("fechasExpedienteExcel.instanceId", null, locale) + "']"
				+ "]";

		jqGridRequestDto.setPage(null);
		jqGridRequestDto.setRows(null);
		List<FechasExpediente> listProgramasAll = this.fechasExpedienteService.findAllLike(filterFechasExpediente,
				jqGridRequestDto, null);

		// Nombre fichero
		modelMap.put("fileName", appMessageSource.getMessage("fechasExpedienteExcel", null, locale));

		// Datos
		List<Object> reportData = new ArrayList<Object>();

		// Hoja 1
		ReportData<FechasExpediente> usuarioExcelDataAll = new ReportData<FechasExpediente>();

		// nombre hoja
		usuarioExcelDataAll.setSheetName(appMessageSource.getMessage("fechasExpedienteExcel", null, locale));

		// cabeceras hoja
		usuarioExcelDataAll.setHeaderNames(ReportData.parseColumns(columns));

		// datos hoja
		if (listProgramasAll.size() <= 0) {

			listProgramasAll.add(new FechasExpediente());

		}
		usuarioExcelDataAll.setModelData(listProgramasAll);
		reportData.add(usuarioExcelDataAll);

		modelMap.put("reportData", reportData);

		// Generación del XLS o XLSX
		if (request.getServletPath().indexOf("xlsReport") != -1) {
			return new ModelAndView("xlsReport", modelMap);
		} else {
			return new ModelAndView("xlsxReport", modelMap);
		}

	}

}
