package com.ejie.ab61.control.listados;

import java.io.IOException;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.ejie.ab61.model.listados.Listados;
import com.ejie.ab61.model.listados.RelLisPro;
import com.ejie.ab61.model.usuario.Usuario;
import com.ejie.ab61.service.listados.ListadosService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;

/**
 * ListadosController  
 * 
 *  
 */

@Controller
@SessionAttributes("usuario")
@RequestMapping(value = "/listados")

public class ListadosController {

	private static final Logger logger = LoggerFactory.getLogger(ListadosController.class);

	@Autowired
	private ListadosService listadosService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idListado
	 *            Integer
	 * @return Listados Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idListado}", method = RequestMethod.GET)
	public @ResponseBody Listados get(@PathVariable Integer idListado) {
		Listados listados = new Listados();
		listados.setIdListado(idListado);
		listados = this.listadosService.find(listados);
		ListadosController.logger.info("[GET - findBy_PK] : Obtener Listados por PK");
		return listados;
	}

	@RequestMapping(value = "/modificar/{idRelLisPro}/{idProcedimiento}", method = RequestMethod.GET)
	public String getFormEdit(Model model, @PathVariable Integer idRelLisPro, @PathVariable Integer idProcedimiento)
			throws IOException {
		ListadosController.logger.info("[GET - View] : busquedasParametros");
		RelLisPro relLisPro = this.listadosService.findRelLisPro(idRelLisPro);
		model.addAttribute("idRelLisPro", relLisPro.getIdRelLisPro());
		model.addAttribute("idProcedimiento", relLisPro.getProcedimientos().getIdProcedimiento());
		model.addAttribute("descripcionCastellano", relLisPro.getDescripcionCastellano());
		model.addAttribute("descripcionEuskera", relLisPro.getDescripcionEuskera());
		model.addAttribute("idListado", relLisPro.getListados().getIdListado());
		return "detalleListado";
	}

	@RequestMapping(value = "/anyadir/{idProcedimiento}", method = RequestMethod.GET)
	public String getFormAdd(Model model, @PathVariable Integer idProcedimiento) {
		ListadosController.logger.info("[GET - View] : busquedasParametros");
		model.addAttribute("idRelLisPro", 0);
		model.addAttribute("idListado", 0);
		model.addAttribute("idProcedimiento", idProcedimiento);
		return "detalleListado";
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param relLisPro
	 *            RelLisPro Bean que contiene la informacion a modificar.
	 * @return RelLisPro Bean resultante de la modificacion.
	 * @throws IOException
	 */
	@RequestMapping(value = "/modif", method = RequestMethod.POST)
	public @ResponseBody RelLisPro edit(Model model, @ModelAttribute RelLisPro relLisPro,
			@ModelAttribute(value = "usuario") Usuario usuario) throws IOException {
		RelLisPro RelLisProAux = new RelLisPro();

		if (0 == relLisPro.getIdRelLisPro()) {
			relLisPro.setUsuarioRegistro(usuario.getNombreUsuario());
			RelLisProAux = this.listadosService.addRelLisPro(relLisPro);
		} else {
			relLisPro.setUsuarioUltmodificacion(usuario.getNombreUsuario());
			RelLisProAux = this.listadosService.updateRelLisPro(relLisPro);
		}
		model.addAttribute("idRelLisPro", RelLisProAux.getIdRelLisPro());
		ListadosController.logger.info("[PUT] : Listados actualizado correctamente");
		return RelLisProAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param listados
	 *            Listados Bean que contiene la informacion con la que se va a
	 *            crear el nuevo registro.
	 * @return Listados Bean resultante del proceso de creacion.
	 */
	@RequestMapping(value = "/add", method = RequestMethod.POST)
	public @ResponseBody Listados add(@RequestBody Listados listados,
			@ModelAttribute(value = "usuario") Usuario usuario) {
		listados.setUsuarioRegistro(usuario.getNombreUsuario());
		Listados listadosAux = this.listadosService.add(listados);
		ListadosController.logger.info("[POST] : Listados insertado correctamente");
		return listadosAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idListado
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return RelLisPro Bean eliminado.
	 */
	@RequestMapping(value = "/{idRelLisPro}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody RelLisPro remove(@PathVariable Integer idRelLisPro) {
		RelLisPro relLisPro = new RelLisPro();
		relLisPro.setIdRelLisPro(idRelLisPro);

		this.listadosService.removeRelLisPro(idRelLisPro);
		ListadosController.logger.info("[DELETE] : Listados borrado correctamente");

		return relLisPro;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model, Usuario usuario) {
		ListadosController.logger.info("[GET - View] : listados");
		model.addAttribute("relLisPro", new RelLisPro());
		Authentication aut = SecurityContextHolder.getContext().getAuthentication();
		/** Dejamos el usuario logueado en session */
		String username = "";
		if (null != aut) {
			username = (String) aut.getPrincipal();
		}
		usuario.setNombreUsuario(username);
		model.addAttribute("usuario", usuario);
		return "listadosExpediente";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterRelLisPro
	 *            RelLisPro Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<RelLisPro> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<RelLisPro> filter(
			@RequestJsonBody(param = "filter") RelLisPro filterRelLisPro,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ListadosController.logger.info("[POST - filter] : Obtener Listadoss");
		return this.listadosService.filterRelLisPro(filterRelLisPro, jqGridRequestDto);
	}

	@RequestMapping(value = "/refrescarPantalla/{idProcedimiento}", method = RequestMethod.GET)
	public String refrescar(Model model, Usuario usuario, @PathVariable Integer idProcedimiento) {
		ListadosController.logger.info("[GET - View] : refrescarPantalla");
		model.addAttribute("relLisPro", new RelLisPro());
		Authentication aut = SecurityContextHolder.getContext().getAuthentication();
		/** Dejamos el usuario logueado en session */
		String username = "";
		if (null != aut) {
			username = (String) aut.getPrincipal();
		}
		usuario.setNombreUsuario(username);
		model.addAttribute("usuario", usuario);
		model.addAttribute("idProcedimiento", idProcedimiento);
		return "listadosExpediente";
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterListados
	 *            Listados Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<Listados> Lista de objetos correspondientes a la busqueda
	 *         realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<Listados> getAll(@ModelAttribute Listados filterListados) {
		ListadosController.logger.info("[GET - find_ALL] : Obtener Listados por filtro");
		return this.listadosService.findAll(filterListados, null);
	}
}
