package com.ejie.ab61.control.plantillas;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.context.support.ReloadableResourceBundleMessageSource;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;

import com.ejie.ab61.comun.Constantes;
import com.ejie.ab61.model.inicio.InitForm;
import com.ejie.ab61.model.plantillas.CamposPlantilla;
import com.ejie.ab61.model.plantillas.RelPlaCam;
import com.ejie.ab61.model.usuario.Usuario;
import com.ejie.ab61.service.plantillas.CamposPlantillaService;
import com.ejie.ab61.service.plantillas.RelPlaCamService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import com.ejie.x38.reports.ReportData;

/**
 * CamposPlantillaController  
 * 
 *  
 */

@Controller
@SessionAttributes("usuario")
@RequestMapping(value = "/camposplantilla")

public class CamposPlantillaController {

	private static final Logger logger = LoggerFactory.getLogger(CamposPlantillaController.class);

	@Autowired
	private CamposPlantillaService camposPlantillaService;

	@Autowired
	private RelPlaCamService relPlaCamService;

	@Resource
	private ReloadableResourceBundleMessageSource appMessageSource;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idCampo
	 *            Integer
	 * @return CamposPlantilla Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idCampo}", method = RequestMethod.GET)
	public @ResponseBody CamposPlantilla get(@PathVariable Integer idCampo) {
		CamposPlantilla camposPlantilla = new CamposPlantilla();
		camposPlantilla.setIdCampo(idCampo);
		camposPlantilla = this.camposPlantillaService.find(camposPlantilla);
		CamposPlantillaController.logger.info("[GET - findBy_PK] : Obtener CamposPlantilla por PK");
		return camposPlantilla;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterCamposPlantilla
	 *            CamposPlantilla Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<CamposPlantilla> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(value = "/verValorCombo", method = RequestMethod.GET)
	public @ResponseBody List<CamposPlantilla> init() {
		CamposPlantillaController.logger.info("[[GET - getValorCombo] : Obtener valores del combo ");
		// return this.camposPlantillaService.findCampos();
		return null;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterCamposPlantilla
	 *            CamposPlantilla Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<CamposPlantilla> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<CamposPlantilla> getAll(@ModelAttribute CamposPlantilla filterCamposPlantilla) {
		CamposPlantillaController.logger.info("[GET - find_ALL] : Obtener CamposPlantilla por filtro");
		return this.camposPlantillaService.findAll(filterCamposPlantilla, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param camposPlantilla
	 *            CamposPlantilla Bean que contiene la informacion a modificar.
	 * @return CamposPlantilla Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody CamposPlantilla edit(@RequestBody CamposPlantilla camposPlantilla,
			@ModelAttribute(value = "usuario") Usuario usuario) {

		camposPlantilla.setFechaUltmodificacion(new Date());
		camposPlantilla.setUsuarioUltmodificacion(usuario.getNombreUsuario());

		CamposPlantilla camposPlantillaAux = this.camposPlantillaService.update(camposPlantilla);
		CamposPlantillaController.logger.info("[PUT] : CamposPlantilla actualizado correctamente");
		return camposPlantillaAux;
	}

	@RequestMapping(value = "/getAllFromTipoCampo/{idTipoCampo}/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<CamposPlantilla> getAllFromTipoCampo(
			@RequestJsonBody(param = "filter") CamposPlantilla filterCamposPlantilla, @PathVariable Integer idTipoCampo,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {

		filterCamposPlantilla.setIdTipoCampo(idTipoCampo);

		CamposPlantillaController.logger.info("[POST - filter] : Obtener CamposPlantillas");
		return this.camposPlantillaService.filter(filterCamposPlantilla, jqGridRequestDto, false);
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param camposPlantilla
	 *            CamposPlantilla Bean que contiene la informacion con la que se
	 *            va a crear el nuevo registro.
	 * @return CamposPlantilla Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody CamposPlantilla add(@RequestBody CamposPlantilla camposPlantilla,
			@ModelAttribute(value = "usuario") Usuario usuario) {

		camposPlantilla.setFechaRegistro(new Date());
		camposPlantilla.setUsuarioRegistro(usuario.getNombreUsuario());

		CamposPlantilla camposPlantillaAux = this.camposPlantillaService.add(camposPlantilla);
		CamposPlantillaController.logger.info("[POST] : CamposPlantilla insertado correctamente");
		return camposPlantillaAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idCampo
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return CamposPlantilla Bean eliminado.
	 */
	@RequestMapping(value = "/{idCampo}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody CamposPlantilla remove(@PathVariable Integer idCampo) {
		CamposPlantilla camposPlantilla = new CamposPlantilla();
		camposPlantilla.setIdCampo(idCampo);

		RelPlaCam relPlaCam = new RelPlaCam();
		relPlaCam.setIdCampo(idCampo);

		// primero borramos relaciones y luego el objeto
		relPlaCamService.remove(relPlaCam);
		this.camposPlantillaService.remove(camposPlantilla);
		CamposPlantillaController.logger.info("[DELETE] : CamposPlantilla borrado correctamente");
		return camposPlantilla;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		CamposPlantillaController.logger.info("[GET - View] : camposplantilla");
		model.addAttribute("camposPlantilla", new CamposPlantilla());
		model.addAttribute("origen", Constantes.origen.GESTION_PLANTILLAS);
		return "camposplantilla";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterCamposPlantilla
	 *            CamposPlantilla Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<CamposPlantilla> Dto que contiene el resultado
	 *         del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<CamposPlantilla> filter(
			@RequestJsonBody(param = "filter") CamposPlantilla filterCamposPlantilla,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		CamposPlantillaController.logger.info("[POST - filter] : Obtener CamposPlantillas");
		return this.camposPlantillaService.filter(filterCamposPlantilla, jqGridRequestDto, false);
	}

	@RequestMapping(value = "/obtenerRelacionCamposDocumento/{idDocumento}/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<CamposPlantilla> filter(
			@RequestJsonBody(param = "filter") CamposPlantilla filterCamposPlantilla, @PathVariable Integer idDocumento,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {

		CamposPlantillaController.logger.info("[POST - filter] : Obtener obtenerRelacionCamposDocumento");
		return this.camposPlantillaService.obtenerRelacionCamposDocumento(idDocumento, jqGridRequestDto);
	}

	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterCamposPlantilla
	 *            CamposPlantilla Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param searchCamposPlantilla
	 *            CamposPlantilla Bean que contiene los parametros de busqueda a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<CamposPlantilla> Dto que contiene el resultado de la
	 *         busqueda realizada por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<CamposPlantilla>> search(
			@RequestJsonBody(param = "filter") CamposPlantilla filterCamposPlantilla,
			@RequestJsonBody(param = "search") CamposPlantilla searchCamposPlantilla,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		CamposPlantillaController.logger.info("[POST - search] : Buscar CamposPlantillas");
		return this.camposPlantillaService.search(filterCamposPlantilla, searchCamposPlantilla, jqGridRequestDto,
				false);
	}

	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterCamposPlantilla
	 *            CamposPlantilla Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String> Lista de los identificadores de los registros
	 *         eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(
			@RequestJsonBody(param = "filter") CamposPlantilla filterCamposPlantilla,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		CamposPlantillaController.logger
				.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples CamposPlantillas");
		this.camposPlantillaService.removeMultiple(filterCamposPlantilla, jqGridRequestDto, false);
		CamposPlantillaController.logger.info("All entities correctly deleted!");

		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterCamposPlantilla
	 *            CamposPlantilla Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<CamposPlantilla>> Dto que contiene
	 *         el resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<CamposPlantilla>> jerarquia(
			@RequestJsonBody(param = "filter") CamposPlantilla filterCamposPlantilla,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		CamposPlantillaController.logger.info("[POST - jerarquia] : Obtener CamposPlantillas jerarquia");
		return this.camposPlantillaService.jerarquia(filterCamposPlantilla, jqGridRequestDto, false);
	}

	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion
	 * jerarquica.
	 * 
	 * @param filterCamposPlantilla
	 *            CamposPlantilla Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<CamposPlantilla>> Dto que contiene
	 *         el resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<CamposPlantilla>> jerarquiaChildren(
			@RequestJsonBody(param = "filter") CamposPlantilla filterCamposPlantilla,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		CamposPlantillaController.logger.info("[POST - jerarquia] : Obtener CamposPlantillas jerarquia - Hijos");
		return this.camposPlantillaService.jerarquiaChildren(filterCamposPlantilla, jqGridRequestDto);
	}

	@RequestMapping(value = { "/xlsReport", "/xlsxReport" }, method = RequestMethod.POST)
	protected ModelAndView getExcelPOI(@ModelAttribute CamposPlantilla filterCamposPlantilla,
			@ModelAttribute JQGridRequestDto jqGridRequestDto, ModelMap modelMap,
			@ModelAttribute(value = "initForm") InitForm initForm,
			@RequestParam(value = "columns", required = false) String columns, HttpServletRequest request) {

		Locale locale = LocaleContextHolder.getLocale();
		if (locale.getLanguage().equals("es")) {
			columns = "[['tipoCampo.descripcionCastellano','" + appMessageSource.getMessage("Tipo campo", null, locale)
					+ "']" + ",['nombreCampo','" + appMessageSource.getMessage("Nombre campo", null, locale) + "']"
					+ ",['descripcion','" + appMessageSource.getMessage("Descripción", null, locale) + "']"
					+ ",['selectCampo','" + appMessageSource.getMessage("Select", null, locale) + "']]";
		} else {
			columns = "[['tipoCampo.descripcionEuskera','" + appMessageSource.getMessage("Eremu mota", null, locale)
					+ "']" + ",['nombreCampo','" + appMessageSource.getMessage("Eremu izena", null, locale) + "']"
					+ ",['descripcion','" + appMessageSource.getMessage("Deskribapena", null, locale) + "']"
					+ ",['selectCampo','" + appMessageSource.getMessage("Select", null, locale) + "']]";
		}
		// Acceso a BD para recuperar datos
		List<CamposPlantilla> listactSubPage = this.camposPlantillaService.findAllLike(filterCamposPlantilla, null,
				false);
		jqGridRequestDto.setPage(null);
		jqGridRequestDto.setRows(null);

		// Nombre fichero
		if (locale.getLanguage().equals("es")) {
			modelMap.put("fileName", appMessageSource.getMessage("Objetos", null, locale));
		} else {
			modelMap.put("fileName", appMessageSource.getMessage("Objektuak", null, locale));
		}

		// Datos
		List<Object> reportData = new ArrayList<Object>();
		// Hoja 1
		ReportData<CamposPlantilla> excelDataAll = new ReportData<CamposPlantilla>();
		// nombre hoja
		if (locale.getLanguage().equals("es")) {
			excelDataAll.setSheetName(appMessageSource.getMessage("Objetos", null, locale));
		} else {
			excelDataAll.setSheetName(appMessageSource.getMessage("Objektuak", null, locale));
		}
		// cabeceras hoja
		excelDataAll.setHeaderNames(ReportData.parseColumns(columns));
		// datos hoja
		excelDataAll.setModelData(listactSubPage);
		reportData.add(excelDataAll);
		modelMap.put("reportData", reportData);
		// GeneraciÃ³n del XLS o XLSX
		if (request.getServletPath().indexOf("xlsReport") != -1) {
			return new ModelAndView("xlsReport", modelMap);
		} else {
			return new ModelAndView("xlsxReport", modelMap);
		}
	}

}
