package com.ejie.ab61.control.plantillas;

import com.ejie.ab61.service.plantillas.DocumentosService;
import com.ejie.ab61.service.plantillas.PlantillasService;
import com.ejie.ab61.service.plantillas.RelPlaCamService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import com.ejie.x38.reports.ReportData;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Vector;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.FilenameUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.context.support.ReloadableResourceBundleMessageSource;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import com.ejie.ab61.comun.Constantes;
import com.ejie.ab61.comun.SeguridadAplicacion;
import com.ejie.ab61.comun.utilidades.Utils;
import com.ejie.ab61.model.inicio.InitForm;
import com.ejie.ab61.model.plantillas.CamposPlantilla;
import com.ejie.ab61.model.plantillas.Documentos;
import com.ejie.ab61.model.plantillas.RelPlaCam;
import com.ejie.ab61.model.usuario.Usuario;

/**
 * DocumentosController  
 * 
 *  
 */

@Controller
@SessionAttributes("usuario")
@RequestMapping(value = "/documentos")

public class DocumentosController {

	private static final Logger logger = LoggerFactory.getLogger(DocumentosController.class);

	@Autowired
	private DocumentosService documentosService;

	@Autowired
	private RelPlaCamService relPlaCamService;
	@Autowired
	private PlantillasService plantillasService;
	@Resource
	private ReloadableResourceBundleMessageSource appMessageSource;
	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idDocumento
	 *            Integer
	 * @return Documentos Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idDocumento}", method = RequestMethod.GET)
	public @ResponseBody Documentos get(@PathVariable Integer idDocumento) {
		Documentos documentos = new Documentos();
		documentos.setIdDocumento(idDocumento);
		documentos = this.documentosService.find(documentos);
		DocumentosController.logger.info("[GET - findBy_PK] : Obtener Documentos por PK");
		return documentos;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterDocumentos
	 *            Documentos Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<Documentos> Lista de objetos correspondientes a la busqueda
	 *         realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<Documentos> getAll(@ModelAttribute Documentos filterDocumentos) {
		DocumentosController.logger.info("[GET - find_ALL] : Obtener Documentos por filtro");
		return this.documentosService.findAll(filterDocumentos, null);
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterDocumentos
	 *            Documentos Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<Documentos> Lista de objetos correspondientes a la busqueda
	 *         realizada.
	 */
	@RequestMapping(value = "/getAllPorDireccion/{direccion}", method = RequestMethod.GET)
	public @ResponseBody List<Documentos> getAllPorDireccion(@ModelAttribute Documentos filterDocumentos,
			@PathVariable Integer direccion) {
		DocumentosController.logger.info("[GET - find_ALL] : Obtener Documentos por filtro");
		return this.documentosService.findAllPorDireccion(filterDocumentos, null, direccion);
	}

	@RequestMapping(value = "/modificar/{idDocumento}", method = RequestMethod.GET)
	public String getFormEdit(Model model, @PathVariable Integer idDocumento) throws IOException {
		DocumentosController.logger.info("[GET - View] : getFormEdit");
		Documentos documento = new Documentos();
		documento.setIdDocumento(idDocumento);
		documento = this.documentosService.find(documento);
		model.addAttribute("idDocumento", idDocumento);

		StringBuilder idsRelPlaCamSelecionados = new StringBuilder();

		RelPlaCam relPlaCam = new RelPlaCam();
		relPlaCam.setIdDocumento(idDocumento);

		List<RelPlaCam> relPlaCamList = this.relPlaCamService.findAll(relPlaCam, null);

		if (relPlaCamList != null) {
			for (int i = 0; i < relPlaCamList.size(); i++) {
				idsRelPlaCamSelecionados.append("#" + relPlaCamList.get(i).getIdCampo() + "#");
			}
		}

		documento.setListaIdCampo(idsRelPlaCamSelecionados.toString());

		model.addAttribute("documento", documento);
		model.addAttribute("camposPlantilla", new CamposPlantilla());
		return "documentodetalle";
	}

	@RequestMapping(value = "/anyadir", method = RequestMethod.GET)
	public String getFormAdd(Model model) throws IOException {
		DocumentosController.logger.info("[GET - View] : getFormAdd");
		model.addAttribute("idDocumento", 0);
		model.addAttribute("documento", new Documentos());
		return "documentodetalle";
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param documento
	 *            Documentos Bean que contiene la informacion a modificar.
	 * @return Documentos Bean resultante de la modificacion.
	 */
	@RequestMapping(value = "/modif", method = RequestMethod.POST)
	public @ResponseBody Documentos edit(Model model, @ModelAttribute Documentos documento,
			@RequestParam(value = "file", required = false) MultipartFile file,
			@ModelAttribute(value = "usuario") Usuario usuario) throws IOException {

		Documentos documentoAux = new Documentos();
		Date fechaReg = new Date();
		boolean updateSinFichero = false;

		if (file.getSize() > 0) {
			documento.setFicheroArray(file.getBytes());
			documento.setNombreArchivo(file.getOriginalFilename());
			documentoAux.setNombreArchivo(file.getOriginalFilename());
		} else {

			Documentos documentoSinFichero = this.documentosService.find(documento);

			if (documentoSinFichero.getNombreArchivo() != null
					&& documentoSinFichero.getNombreArchivo().equals(documento.getNombreArchivo())) {
				updateSinFichero = true;
			} else {
				documento.setFicheroArray(null);
				documento.setNombreArchivo(null);
				documentoAux.setNombreArchivo(null);
			}
		}

		if (null == documento.getIdDocumento() || 0 == documento.getIdDocumento()) {

			documento.setFechaRegistro(fechaReg);
			documento.setUsuarioRegistro(usuario.getNombreUsuario());
			documentoAux = this.documentosService.add(documento);

		} else {

			// Eliminamos todos los registros de ese id Documento de la tabla
			// AB6123T00
			RelPlaCam relPlaCam = new RelPlaCam();
			relPlaCam.setIdDocumento(documento.getIdDocumento());

			this.relPlaCamService.remove(relPlaCam);

			documento.setFechaUltmodificacion(new Date());
			documento.setUsuarioUltmodificacion(usuario.getNombreUsuario());

			if (updateSinFichero) {
				documentoAux = this.documentosService.updateSinFichero(documento);
			} else {
				documentoAux = this.documentosService.update(documento);
			}

		}

		if (documento.getListaIdCampo() != null && !"".equals(documento.getListaIdCampo())) {
			// Insertamos la relacion en la tabla AB6123T00
			String[] arrayIdsCampo = documento.getListaIdCampo().substring(1, documento.getListaIdCampo().length() - 1)
					.replace("##", "#").split("#");
			for (String i : arrayIdsCampo) {
				RelPlaCam relPlaCam = new RelPlaCam();
				relPlaCam.setIdCampo(Integer.valueOf(i));
				relPlaCam.setIdDocumento(documentoAux.getIdDocumento());
				relPlaCam.setUsuarioRegistro(usuario.getNombreUsuario());
				relPlaCam.setFechaRegistro(fechaReg);

				this.relPlaCamService.add(relPlaCam);
			}
		}

		DocumentosController.logger.info("[PUT] : Documento actualizado correctamente");
		return documentoAux;
	}

	@RequestMapping(value = "/borrarFichero/{idDocumento}", method = RequestMethod.POST)
	public @ResponseBody Documentos borrarFichero(@PathVariable Integer idDocumento,
			@ModelAttribute(value = "usuario") Usuario usuario) {
		Documentos documento = new Documentos(idDocumento);
		documento.setUsuarioUltmodificacion(usuario.getNombreUsuario());
		this.documentosService.copiarFicheros(documento, usuario);
		Documentos documentoRemove = this.documentosService.removeFicheros(documento);
		DocumentosController.logger.info("[POST] : Listados insertado correctamente");
		return documentoRemove;
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param documentos
	 *            Documentos Bean que contiene la informacion a modificar.
	 * @return Documentos Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody Documentos edit(@RequestBody Documentos documentos) {
		Documentos documentosAux = this.documentosService.update(documentos);
		DocumentosController.logger.info("[PUT] : Documentos actualizado correctamente");
		return documentosAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param documentos
	 *            Documentos Bean que contiene la informacion con la que se va a
	 *            crear el nuevo registro.
	 * @return Documentos Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody Documentos add(@RequestBody Documentos documentos) {
		Documentos documentosAux = this.documentosService.add(documentos);
		DocumentosController.logger.info("[POST] : Documentos insertado correctamente");
		return documentosAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idDocumento
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return Documentos Bean eliminado.
	 */
	@RequestMapping(value = "/{idDocumento}/{registrosHijo}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody Documentos remove(@PathVariable Integer idDocumento, @PathVariable Integer registrosHijo) {
		DocumentosController.logger.info("[DELETE] : Documentos  correctamente");
		// primero ponemos a null todos los id_documentos de la tabla pantillas
		if (registrosHijo > 0) {
			DocumentosController.logger.info("[upadte] : plantillas updateadas   correctamente");
			this.plantillasService.updateIdDocumento(idDocumento);
		}
		// Eliminamos todos los registros de ese id Documento de la tabla
		// AB6123T00
		RelPlaCam relPlaCam = new RelPlaCam();
		relPlaCam.setIdDocumento(idDocumento);

		this.relPlaCamService.remove(relPlaCam);

		Documentos documentos = new Documentos();
		documentos.setIdDocumento(idDocumento);
		this.documentosService.remove(documentos);
		DocumentosController.logger.info("[DELETE] : Documentos borrado correctamente");
		return documentos;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		DocumentosController.logger.info("[GET - View] : documentos");
		model.addAttribute("documento", new Documentos());

		Vector<String> rolesUsuario = SeguridadAplicacion
				.getRolesUsuario(SecurityContextHolder.getContext().getAuthentication());

		if (rolesUsuario.contains(SeguridadAplicacion.ROLE_AB61_IN_0008)) {

			model.addAttribute("roleAb61In0008", true);

			if (rolesUsuario.contains(SeguridadAplicacion.ROLE_TECNICO_TURISMO)) {
				model.addAttribute("roleTecnicoTurismo", true);
			} else {
				model.addAttribute("roleTecnicoTurismo", false);
			}

			if (rolesUsuario.contains(SeguridadAplicacion.ROLE_TECNICO_COMERCIO)) {
				model.addAttribute("roleTecnicoComercio", true);
			} else {
				model.addAttribute("roleTecnicoComercio", false);
			}

			if (rolesUsuario.contains(SeguridadAplicacion.ROLE_TECNICO_CONSUMO)) {
				model.addAttribute("roleTecnicoConsumo", true);
			} else {
				model.addAttribute("roleTecnicoConsumo", false);
			}

		} else {
			model.addAttribute("roleAb61In0008", false);
		}

		return "documentos";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterDocumentos
	 *            Documentos Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Documentos> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Documentos> filter(
			@RequestJsonBody(param = "filter") Documentos filterDocumentos,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DocumentosController.logger.info("[POST - filter] : Obtener Documentoss");
		return this.documentosService.filter(filterDocumentos, jqGridRequestDto, false);
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterDocumentos
	 *            Documentos Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Documentos> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/documentosPorDireccion/{direccion}/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Documentos> documentosPorDireccion(
			@RequestJsonBody(param = "filter") Documentos filterDocumentos,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto, @PathVariable Integer direccion) {
		DocumentosController.logger.info("[POST - filter] : Obtener Documentoss");
		return this.documentosService.documentosPorDireccion(filterDocumentos, jqGridRequestDto, false, direccion);
	}

	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterDocumentos
	 *            Documentos Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param searchDocumentos
	 *            Documentos Bean que contiene los parametros de busqueda a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<Documentos> Dto que contiene el resultado de la
	 *         busqueda realizada por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<Documentos>> search(
			@RequestJsonBody(param = "filter") Documentos filterDocumentos,
			@RequestJsonBody(param = "search") Documentos searchDocumentos,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DocumentosController.logger.info("[POST - search] : Buscar Documentoss");
		return this.documentosService.search(filterDocumentos, searchDocumentos, jqGridRequestDto, false);
	}

	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterDocumentos
	 *            Documentos Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String> Lista de los identificadores de los registros
	 *         eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(@RequestJsonBody(param = "filter") Documentos filterDocumentos,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DocumentosController.logger.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples Documentoss");
		this.documentosService.removeMultiple(filterDocumentos, jqGridRequestDto, false);
		DocumentosController.logger.info("All entities correctly deleted!");

		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterDocumentos
	 *            Documentos Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Documentos>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Documentos>> jerarquia(
			@RequestJsonBody(param = "filter") Documentos filterDocumentos,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DocumentosController.logger.info("[POST - jerarquia] : Obtener Documentoss jerarquia");
		return this.documentosService.jerarquia(filterDocumentos, jqGridRequestDto, false);
	}

	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion
	 * jerarquica.
	 * 
	 * @param filterDocumentos
	 *            Documentos Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Documentos>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Documentos>> jerarquiaChildren(
			@RequestJsonBody(param = "filter") Documentos filterDocumentos,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DocumentosController.logger.info("[POST - jerarquia] : Obtener Documentoss jerarquia - Hijos");
		return this.documentosService.jerarquiaChildren(filterDocumentos, jqGridRequestDto);
	}

	/**
	 * Method 'findBandejaFirmaDoc'.
	 * 
	 * @param tipoexpediente
	 *            String
	 * @param iddokusi
	 *            String
	 * @param response
	 *            HttpServletResponse
	 */
	@RequestMapping(value = "/descargaFichero/{idDocumento}", method = RequestMethod.GET)
	public @ResponseBody void findBandejaFirmaDoc(@PathVariable Integer idDocumento, HttpServletResponse response) {
		try {
			Documentos documentos = new Documentos();
			documentos.setIdDocumento(idDocumento);
			documentos = this.documentosService.find(documentos);

			byte[] buffer = documentos.getArchivo().getBytes(1, (int) documentos.getArchivo().length());

			String nombre = documentos.getNombreArchivo();
			response.setHeader("Content-Disposition", "attachment;filename=\"" + nombre + "\"");
			response.setContentType(Utils.obtenerContentType(FilenameUtils.getExtension(nombre)));
			response.getOutputStream().write(buffer);

			// Vaciamos el stream
			response.getOutputStream().flush();
		} catch (Exception e) {

		}
	}

	@RequestMapping(value = { "/xlsReport", "/xlsxReport" }, method = RequestMethod.POST)
	protected ModelAndView getExcelPOI(@ModelAttribute Documentos filterCamposPlantilla,
			@ModelAttribute JQGridRequestDto jqGridRequestDto, ModelMap modelMap,
			@ModelAttribute(value = "initForm") InitForm initForm,
			@RequestParam(value = "columns", required = false) String columns, HttpServletRequest request) {

		Vector<String> rolesUsuario = SeguridadAplicacion
				.getRolesUsuario(SecurityContextHolder.getContext().getAuthentication());

		Boolean roleAb61In0008 = null;
		Boolean roleTecnicoTurismo = null;
		Boolean roleTecnicoComercio = null;
		Boolean roleTecnicoConsumo = null;

		if (rolesUsuario.contains(SeguridadAplicacion.ROLE_AB61_IN_0008)) {
			roleAb61In0008 = true;
		}
		if (rolesUsuario.contains(SeguridadAplicacion.ROLE_TECNICO_TURISMO)) {
			roleTecnicoTurismo = true;
		} else {
			roleTecnicoTurismo = true;
		}

		if (rolesUsuario.contains(SeguridadAplicacion.ROLE_TECNICO_COMERCIO)) {
			roleTecnicoComercio = true;
		} else {
			roleTecnicoComercio = true;
		}

		if (rolesUsuario.contains(SeguridadAplicacion.ROLE_TECNICO_CONSUMO)) {
			roleTecnicoConsumo = true;
		} else {
			roleTecnicoConsumo = true;
		}

		Locale locale = LocaleContextHolder.getLocale();
		if (locale.getLanguage().equals("es")) {
			columns = "[['nombreDocumento','" + appMessageSource.getMessage("Nombre documento", null, locale) + "']"
					+ ",['nombreArchivo','" + appMessageSource.getMessage("Nombre Archivo", null, locale) + "']"
					+ ",['observaciones','" + appMessageSource.getMessage("Observaciones", null, locale) + "']]";
		} else {
			columns = "[['nombreDocumento','" + appMessageSource.getMessage("Dokumentu izena", null, locale) + "']"
					+ ",['nombreArchivo','" + appMessageSource.getMessage("Artxibo izena", null, locale) + "']"
					+ ",['observaciones','" + appMessageSource.getMessage("Oharrak", null, locale) + "']]";
		}
		// Acceso a BD para recuperar datos

		List<Documentos> listactSubPage = null;

		if (roleAb61In0008 == true && roleTecnicoTurismo == true) {
			listactSubPage = this.documentosService.findAllLikeDocumentosPorDireccion(filterCamposPlantilla, null, null,
					Constantes.TECNICO_TURISMO);
		} else if (roleAb61In0008 == true && roleTecnicoComercio == true) {
			listactSubPage = this.documentosService.findAllLikeDocumentosPorDireccion(filterCamposPlantilla, null, null,
					Constantes.TECNICO_COMERCIO);
		} else if (roleAb61In0008 == true && roleTecnicoConsumo == true) {
			listactSubPage = this.documentosService.findAllLikeDocumentosPorDireccion(filterCamposPlantilla, null, null,
					Constantes.TECNICO_CONSUMO);
		} else {
			listactSubPage = this.documentosService.findAllLike(filterCamposPlantilla, null, null);
		}

		jqGridRequestDto.setPage(null);
		jqGridRequestDto.setRows(null);

		// Nombre fichero
		if (locale.getLanguage().equals("es")) {
			modelMap.put("fileName", appMessageSource.getMessage("Documentos", null, locale));
		} else {
			modelMap.put("fileName", appMessageSource.getMessage("Dokumentuak", null, locale));
		}

		// Datos
		List<Object> reportData = new ArrayList<Object>();
		// Hoja 1
		ReportData<Documentos> excelDataAll = new ReportData<Documentos>();
		// nombre hoja
		if (locale.getLanguage().equals("es")) {
			excelDataAll.setSheetName(appMessageSource.getMessage("Documentos", null, locale));
		} else {
			excelDataAll.setSheetName(appMessageSource.getMessage("Dokumentuak", null, locale));
		}
		// cabeceras hoja
		excelDataAll.setHeaderNames(ReportData.parseColumns(columns));
		// datos hoja
		excelDataAll.setModelData(listactSubPage);
		reportData.add(excelDataAll);
		modelMap.put("reportData", reportData);
		// GeneraciÃ³n del XLS o XLSX
		if (request.getServletPath().indexOf("xlsReport") != -1) {
			return new ModelAndView("xlsReport", modelMap);
		} else {
			return new ModelAndView("xlsxReport", modelMap);
		}
	}
}
