package com.ejie.ab61.control.plantillas;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.ejie.ab61.model.plantillas.TipoCampo;
import com.ejie.ab61.service.plantillas.TipoCampoService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * TipoCampoController  
 * 
 *  
 */

@Controller
@RequestMapping(value = "/tipocampo")

public class TipoCampoController {

	private static final Logger logger = LoggerFactory.getLogger(TipoCampoController.class);

	@Autowired
	private TipoCampoService tipoCampoService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idTipoCampo
	 *            Integer
	 * @return TipoCampo Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idTipoCampo}", method = RequestMethod.GET)
	public @ResponseBody TipoCampo get(@PathVariable Integer idTipoCampo) {
		TipoCampo tipoCampo = new TipoCampo();
		tipoCampo.setIdTipoCampo(idTipoCampo);
		tipoCampo = this.tipoCampoService.find(tipoCampo);
		TipoCampoController.logger.info("[GET - findBy_PK] : Obtener TipoCampo por PK");
		return tipoCampo;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<TipoCampo> Lista de objetos correspondientes a la busqueda
	 *         realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<TipoCampo> getAll(@ModelAttribute TipoCampo filterTipoCampo) {
		TipoCampoController.logger.info("[GET - find_ALL] : Obtener TipoCampo por filtro");
		return this.tipoCampoService.findAll(filterTipoCampo, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param tipoCampo
	 *            TipoCampo Bean que contiene la informacion a modificar.
	 * @return TipoCampo Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody TipoCampo edit(@RequestBody TipoCampo tipoCampo) {
		TipoCampo tipoCampoAux = this.tipoCampoService.update(tipoCampo);
		TipoCampoController.logger.info("[PUT] : TipoCampo actualizado correctamente");
		return tipoCampoAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param tipoCampo
	 *            TipoCampo Bean que contiene la informacion con la que se va a
	 *            crear el nuevo registro.
	 * @return TipoCampo Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody TipoCampo add(@RequestBody TipoCampo tipoCampo) {
		TipoCampo tipoCampoAux = this.tipoCampoService.add(tipoCampo);
		TipoCampoController.logger.info("[POST] : TipoCampo insertado correctamente");
		return tipoCampoAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idTipoCampo
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return TipoCampo Bean eliminado.
	 */
	@RequestMapping(value = "/{idTipoCampo}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody TipoCampo remove(@PathVariable Integer idTipoCampo) {
		TipoCampo tipoCampo = new TipoCampo();
		tipoCampo.setIdTipoCampo(idTipoCampo);
		this.tipoCampoService.remove(tipoCampo);
		TipoCampoController.logger.info("[DELETE] : TipoCampo borrado correctamente");
		return tipoCampo;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		TipoCampoController.logger.info("[GET - View] : tipocampo");
		return "tipocampo";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterTipoCampo
	 *            TipoCampo Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<TipoCampo> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<TipoCampo> filter(
			@RequestJsonBody(param = "filter") TipoCampo filterTipoCampo,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TipoCampoController.logger.info("[POST - filter] : Obtener TipoCampos");
		return this.tipoCampoService.filter(filterTipoCampo, jqGridRequestDto, false);
	}

	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterTipoCampo
	 *            TipoCampo Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param searchTipoCampo
	 *            TipoCampo Bean que contiene los parametros de busqueda a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<TipoCampo> Dto que contiene el resultado de la
	 *         busqueda realizada por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<TipoCampo>> search(
			@RequestJsonBody(param = "filter") TipoCampo filterTipoCampo,
			@RequestJsonBody(param = "search") TipoCampo searchTipoCampo,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TipoCampoController.logger.info("[POST - search] : Buscar TipoCampos");
		return this.tipoCampoService.search(filterTipoCampo, searchTipoCampo, jqGridRequestDto, false);
	}

	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterTipoCampo
	 *            TipoCampo Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String> Lista de los identificadores de los registros
	 *         eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(@RequestJsonBody(param = "filter") TipoCampo filterTipoCampo,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TipoCampoController.logger.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples TipoCampos");
		this.tipoCampoService.removeMultiple(filterTipoCampo, jqGridRequestDto, false);
		TipoCampoController.logger.info("All entities correctly deleted!");

		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterTipoCampo
	 *            TipoCampo Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<TipoCampo>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<TipoCampo>> jerarquia(
			@RequestJsonBody(param = "filter") TipoCampo filterTipoCampo,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TipoCampoController.logger.info("[POST - jerarquia] : Obtener TipoCampos jerarquia");
		return this.tipoCampoService.jerarquia(filterTipoCampo, jqGridRequestDto, false);
	}

	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion
	 * jerarquica.
	 * 
	 * @param filterTipoCampo
	 *            TipoCampo Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<TipoCampo>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<TipoCampo>> jerarquiaChildren(
			@RequestJsonBody(param = "filter") TipoCampo filterTipoCampo,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TipoCampoController.logger.info("[POST - jerarquia] : Obtener TipoCampos jerarquia - Hijos");
		return this.tipoCampoService.jerarquiaChildren(filterTipoCampo, jqGridRequestDto);
	}

}
