package com.ejie.ab61.control.procedimientos;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.ejie.ab61.model.procedimientos.Areas;
import com.ejie.ab61.model.procedimientos.Direcciones;
import com.ejie.ab61.service.procedimientos.AreasService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * AreasController  
 * 
 *  
 */

@Controller
@RequestMapping(value = "/areas")

public class AreasController {

	private static final Logger logger = LoggerFactory.getLogger(AreasController.class);

	@Autowired
	private AreasService areasService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idArea
	 *            Integer
	 * @return Areas Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idArea}", method = RequestMethod.GET)
	public @ResponseBody Areas get(@PathVariable Integer idArea) {
		Areas areas = new Areas();
		areas.setIdArea(idArea);
		areas = this.areasService.find(areas);
		AreasController.logger.info("[GET - findBy_PK] : Obtener Areas por PK");
		return areas;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterAreas
	 *            Areas Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<Areas> Lista de objetos correspondientes a la busqueda
	 *         realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<Areas> getAll(@ModelAttribute Areas filterAreas,
			@RequestParam(value = "direccion.idDireccion", required = false) Integer idDireccion) {
		AreasController.logger.info("[GET - find_ALL] : Obtener Areas por filtro");

		if (idDireccion != null) {
			Direcciones direccion = new Direcciones();
			direccion.setIdDireccion(idDireccion);
			filterAreas.setDirecciones(direccion);
		}

		return this.areasService.findAll(filterAreas, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param areas
	 *            Areas Bean que contiene la informacion a modificar.
	 * @return Areas Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody Areas edit(@RequestBody Areas areas) {
		Areas areasAux = this.areasService.update(areas);
		AreasController.logger.info("[PUT] : Areas actualizado correctamente");
		return areasAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param areas
	 *            Areas Bean que contiene la informacion con la que se va a
	 *            crear el nuevo registro.
	 * @return Areas Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody Areas add(@RequestBody Areas areas) {
		Areas areasAux = this.areasService.add(areas);
		AreasController.logger.info("[POST] : Areas insertado correctamente");
		return areasAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idArea
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return Areas Bean eliminado.
	 */
	@RequestMapping(value = "/{idArea}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody Areas remove(@PathVariable Integer idArea) {
		Areas areas = new Areas();
		areas.setIdArea(idArea);
		this.areasService.remove(areas);
		AreasController.logger.info("[DELETE] : Areas borrado correctamente");
		return areas;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		AreasController.logger.info("[GET - View] : areas");
		return "areas";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterAreas
	 *            Areas Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Areas> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Areas> filter(@RequestJsonBody(param = "filter") Areas filterAreas,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		AreasController.logger.info("[POST - filter] : Obtener Areass");
		return this.areasService.filter(filterAreas, jqGridRequestDto, false);
	}

	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterAreas
	 *            Areas Bean que contiene los parametros de filtrado a emplear.
	 * @param searchAreas
	 *            Areas Bean que contiene los parametros de busqueda a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<Areas> Dto que contiene el resultado de la busqueda
	 *         realizada por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<Areas>> search(@RequestJsonBody(param = "filter") Areas filterAreas,
			@RequestJsonBody(param = "search") Areas searchAreas, @RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		AreasController.logger.info("[POST - search] : Buscar Areass");
		return this.areasService.search(filterAreas, searchAreas, jqGridRequestDto, false);
	}

	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterAreas
	 *            Areas Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String> Lista de los identificadores de los registros
	 *         eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(@RequestJsonBody(param = "filter") Areas filterAreas,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		AreasController.logger.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples Areass");
		this.areasService.removeMultiple(filterAreas, jqGridRequestDto, false);
		AreasController.logger.info("All entities correctly deleted!");

		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterAreas
	 *            Areas Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Areas>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Areas>> jerarquia(
			@RequestJsonBody(param = "filter") Areas filterAreas, @RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		AreasController.logger.info("[POST - jerarquia] : Obtener Areass jerarquia");
		return this.areasService.jerarquia(filterAreas, jqGridRequestDto, false);
	}

	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion
	 * jerarquica.
	 * 
	 * @param filterAreas
	 *            Areas Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Areas>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Areas>> jerarquiaChildren(
			@RequestJsonBody(param = "filter") Areas filterAreas, @RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		AreasController.logger.info("[POST - jerarquia] : Obtener Areass jerarquia - Hijos");
		return this.areasService.jerarquiaChildren(filterAreas, jqGridRequestDto);
	}

}
