package com.ejie.ab61.control.solicitudes;

import com.ejie.ab61.model.solicitudes.Solicitudes;
import com.ejie.ab61.service.solicitudes.SolicitudesService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

/**
 * SolicitudesController  
 * 
 *  
 */

@Controller
@RequestMapping(value = "/solicitudes")

public class SolicitudesController {

	private static final Logger logger = LoggerFactory.getLogger(SolicitudesController.class);

	@Autowired
	private SolicitudesService solicitudesService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idSolicitud
	 *            String
	 * @return Solicitudes Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idSolicitud}", method = RequestMethod.GET)
	public @ResponseBody Solicitudes get(@PathVariable int idSolicitud) {
		Solicitudes solicitudes = new Solicitudes();
		solicitudes.setIdSolicitud(idSolicitud);
		solicitudes = this.solicitudesService.find(solicitudes);
		SolicitudesController.logger.info("[GET - findBy_PK] : Obtener Solicitudes por PK");
		return solicitudes;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterSolicitudes
	 *            Solicitudes Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<Solicitudes> Lista de objetos correspondientes a la busqueda
	 *         realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<Solicitudes> getAll(@ModelAttribute Solicitudes filterSolicitudes) {
		SolicitudesController.logger.info("[GET - find_ALL] : Obtener Solicitudes por filtro");
		return this.solicitudesService.findAll(filterSolicitudes, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param solicitudes
	 *            Solicitudes Bean que contiene la informacion a modificar.
	 * @return Solicitudes Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody Solicitudes edit(@RequestBody Solicitudes solicitudes) {
		Solicitudes solicitudesAux = this.solicitudesService.update(solicitudes);
		SolicitudesController.logger.info("[PUT] : Solicitudes actualizado correctamente");
		return solicitudesAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param solicitudes
	 *            Solicitudes Bean que contiene la informacion con la que se va
	 *            a crear el nuevo registro.
	 * @return Solicitudes Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody Solicitudes add(@RequestBody Solicitudes solicitudes) {
		Solicitudes solicitudesAux = this.solicitudesService.add(solicitudes);
		SolicitudesController.logger.info("[POST] : Solicitudes insertado correctamente");
		return solicitudesAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idSolicitud
	 *            String Identificador del objeto que se desea eliminar.
	 * @return Solicitudes Bean eliminado.
	 */
	@RequestMapping(value = "/{idSolicitud}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody Solicitudes remove(@PathVariable int idSolicitud) {
		Solicitudes solicitudes = new Solicitudes();
		solicitudes.setIdSolicitud(idSolicitud);
		this.solicitudesService.remove(solicitudes);
		SolicitudesController.logger.info("[DELETE] : Solicitudes borrado correctamente");
		return solicitudes;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		SolicitudesController.logger.info("[GET - View] : solicitudes");
		return "solicitudes";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterSolicitudes
	 *            Solicitudes Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Solicitudes> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Solicitudes> filter(
			@RequestJsonBody(param = "filter") Solicitudes filterSolicitudes,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		SolicitudesController.logger.info("[POST - filter] : Obtener Solicitudess");
		return this.solicitudesService.filter(filterSolicitudes, jqGridRequestDto, false);
	}

	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterSolicitudes
	 *            Solicitudes Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param searchSolicitudes
	 *            Solicitudes Bean que contiene los parametros de busqueda a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<Solicitudes> Dto que contiene el resultado de la
	 *         busqueda realizada por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<Solicitudes>> search(
			@RequestJsonBody(param = "filter") Solicitudes filterSolicitudes,
			@RequestJsonBody(param = "search") Solicitudes searchSolicitudes,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		SolicitudesController.logger.info("[POST - search] : Buscar Solicitudess");
		return this.solicitudesService.search(filterSolicitudes, searchSolicitudes, jqGridRequestDto, false);
	}

	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterSolicitudes
	 *            Solicitudes Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String> Lista de los identificadores de los registros
	 *         eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(@RequestJsonBody(param = "filter") Solicitudes filterSolicitudes,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		SolicitudesController.logger
				.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples Solicitudess");
		this.solicitudesService.removeMultiple(filterSolicitudes, jqGridRequestDto, false);
		SolicitudesController.logger.info("All entities correctly deleted!");

		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterSolicitudes
	 *            Solicitudes Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Solicitudes>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Solicitudes>> jerarquia(
			@RequestJsonBody(param = "filter") Solicitudes filterSolicitudes,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		SolicitudesController.logger.info("[POST - jerarquia] : Obtener Solicitudess jerarquia");
		return this.solicitudesService.jerarquia(filterSolicitudes, jqGridRequestDto, false);
	}

	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion
	 * jerarquica.
	 * 
	 * @param filterSolicitudes
	 *            Solicitudes Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Solicitudes>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Solicitudes>> jerarquiaChildren(
			@RequestJsonBody(param = "filter") Solicitudes filterSolicitudes,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		SolicitudesController.logger.info("[POST - jerarquia] : Obtener Solicitudess jerarquia - Hijos");
		return this.solicitudesService.jerarquiaChildren(filterSolicitudes, jqGridRequestDto);
	}

}
