package com.ejie.ab61.control.terceros;

import java.util.Date;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.ejie.ab61.clientesWS.X43ExpedienteElectronico.TFolderList;

import com.ejie.ab61.clientesWS.X43Toolkit.Exception_Exception;
import com.ejie.ab61.comun.Constantes;
import com.ejie.ab61.model.terceros.DatosTerceros;
import com.ejie.ab61.model.terceros.Sexo;

import com.ejie.ab61.service.expedientes.ExpedientesService;
import com.ejie.ab61.service.terceros.DatosTercerosService;
import com.ejie.ab61.service.terceros.SexoService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * DatosTercerosController  
 * 
 *  
 */

@Controller
@RequestMapping(value = "/datosterceros")

public class DatosTercerosController {

	private static final Logger logger = LoggerFactory.getLogger(DatosTercerosController.class);

	@Autowired
	private DatosTercerosService datosTercerosService;
	@Autowired
	private SexoService sS;
	@Autowired
	private ExpedientesService expedienteService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Metodo que nos carga el formulario de un tercero * @param idTercero
	 * Integer
	 * 
	 * @return Terceros Objeto correspondiente al identificador indicado.
	 * @throws Exception_Exception
	 * 
	 */
	@RequestMapping(value = "/{idExpediente}/{idTercero}/{folderId}/{tipo}/{partipantId}")
	public String getByid(@PathVariable int idExpediente, @PathVariable int idTercero, @PathVariable String folderId,
			@PathVariable String tipo, Model model, @PathVariable String partipantId) throws Exception_Exception {
		long tipoPersona = 0L;
		String idioma = LocaleContextHolder.getLocale().getLanguage();
		DatosTerceros datosTercero = new DatosTerceros();
		if (partipantId.equals(Constantes.PARTICIPACIONID_NULA)) {
			if (tipo.equals(Constantes.PERSONA_CONTACTO)) {
				tipoPersona = Constantes.TiposTercero.PERSONA_CONTACTO;
			} else {
				tipoPersona = Constantes.TiposTercero.REPRESENTANTE_LEGAL;
			}
			datosTercero = this.datosTercerosService.findPersonacontactoResponsableLegal(idTercero, tipoPersona,
					idExpediente);
		} else {
			datosTercero = this.datosTercerosService.findTercero(idTercero, partipantId);
		}

		if (datosTercero != null) {

			TFolderList resultado = expedienteService.obtenerInfoPlatea(folderId);
			if (null != resultado) {
				datosTercero = expedienteService.obtenerInfoPlateaDetalle(datosTercero, resultado, tipo);
			}

		}

		if (datosTercero == null) {
			model.addAttribute("solicitante", new DatosTerceros());
		} else {
			model.addAttribute("solicitante", datosTercero);
		}

		String hayTipoSMS = "N";
		String hayTipoEMAIL = "N";

		if (null != datosTercero && datosTercero.getCanalesAvisos() != null
				&& datosTercero.getCanalesAvisos().size() > 0) {
			for (int i = 0; i < datosTercero.getCanalesAvisos().size(); i++) {
				if (datosTercero.getCanalesAvisos().get(i).getTiposCanalAviso() != null
						&& datosTercero.getCanalesAvisos().get(i).getTiposCanalAviso()
								.getIdTipoCanalAviso() == Constantes.TIPO_CANAL_AVISO_SMS) {
					hayTipoSMS = "S";
				}
				if (datosTercero.getCanalesAvisos().get(i).getTiposCanalAviso() != null
						&& datosTercero.getCanalesAvisos().get(i).getTiposCanalAviso()
								.getIdTipoCanalAviso() == Constantes.TIPO_CANAL_AVISO_EMAIL) {
					hayTipoEMAIL = "S";
				}
			}
		}

		model.addAttribute("hayTipoSMS", hayTipoSMS);
		model.addAttribute("hayTipoEMAIL", hayTipoEMAIL);

		model.addAttribute("lenguaje", idioma);

		return "datosSolicitantes";
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterDatosTerceros
	 *            DatosTerceros Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<DatosTerceros> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<DatosTerceros> getAll(@ModelAttribute DatosTerceros filterDatosTerceros) {
		DatosTercerosController.logger.info("[GET - find_ALL] : Obtener DatosTerceros por filtro");
		return this.datosTercerosService.findAll(filterDatosTerceros, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param datosTerceros
	 *            DatosTerceros Bean que contiene la informacion a modificar.
	 * @return DatosTerceros Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody DatosTerceros edit(@RequestBody DatosTerceros datosTerceros) {
		DatosTerceros datosTercerosAux = this.datosTercerosService.update(datosTerceros);
		DatosTercerosController.logger.info("[PUT] : DatosTerceros actualizado correctamente");
		return datosTercerosAux;
	}

	/**
	 * Metod para hacer un update
	 * 
	 * @param tercero
	 * @return Terceros Bean resultante del proceso de creacion
	 */
	@RequestMapping(value = "/save", method = RequestMethod.POST)
	public @ResponseBody() DatosTerceros savePersona(@RequestBody() DatosTerceros datosTercero) {
		logger.info(datosTercero.toString());
		Date fecha = new Date();
		datosTercero.setFechaUltmodificacion(fecha);
		this.datosTercerosService.update(datosTercero);
		return datosTercero;
	}

	@RequestMapping(value = "/{idTercero}/{tipo}/{idExpediente}/remove")
	public String removeByidRemove(@PathVariable int idTercero, @PathVariable String tipo,
			@PathVariable int idExpediente, Model model) throws Exception_Exception {

		long tipoPersona = 0L;
		String idioma = LocaleContextHolder.getLocale().getLanguage();
		DatosTerceros datosTercero = new DatosTerceros();
		if (tipo.equals(Constantes.PERSONA_CONTACTO)) {
			tipoPersona = Constantes.TiposTercero.PERSONA_CONTACTO;
		} else {
			tipoPersona = Constantes.TiposTercero.REPRESENTANTE_LEGAL;
		}
		datosTercero = this.datosTercerosService.findPersonacontactoResponsableLegal(idTercero, tipoPersona,
				idExpediente);
		List<Sexo> sexo = this.sS.findAll();
		if (datosTercero == null) {
			model.addAttribute("solicitante", new DatosTerceros());
		} else {
			model.addAttribute("solicitante", datosTercero);
		}
		model.addAttribute("sexoidiomas", sexo);
		model.addAttribute("lenguaje", idioma);

		String hayTipoSMS = "N";
		String hayTipoEMAIL = "N";
		model.addAttribute("hayTipoSMS", hayTipoSMS);
		model.addAttribute("hayTipoEMAIL", hayTipoEMAIL);
		model.addAttribute("eliminar", 1);
		model.addAttribute("idExpediente", idExpediente);
		return "datosSolicitantes";
	}

	/**
	 * Borrar tercero tipo personaContacto o representanteLegal
	 * 
	 * @param datosTercero
	 * @return DatosTercero
	 */
	@RequestMapping(value = "/delete/{idExpediente}", method = RequestMethod.POST)
	public @ResponseBody() DatosTerceros removePersona(@RequestBody() DatosTerceros datosTercero,
			@PathVariable int idExpediente) {
		logger.info("eliminamos el siguiente tercero  " + datosTercero.toString());

		datosTercerosService.removeIdTercero(datosTercero);
		return datosTercero;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		DatosTercerosController.logger.info("[GET - View] : datosterceros");
		return "datosterceros";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterDatosTerceros
	 *            DatosTerceros Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<DatosTerceros> Dto que contiene el resultado
	 *         del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<DatosTerceros> filter(
			@RequestJsonBody(param = "filter") DatosTerceros filterDatosTerceros,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DatosTercerosController.logger.info("[POST - filter] : Obtener DatosTerceross");
		return this.datosTercerosService.filter(filterDatosTerceros, jqGridRequestDto, false);
	}

	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterDatosTerceros
	 *            DatosTerceros Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param searchDatosTerceros
	 *            DatosTerceros Bean que contiene los parametros de busqueda a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<DatosTerceros> Dto que contiene el resultado de la
	 *         busqueda realizada por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<DatosTerceros>> search(
			@RequestJsonBody(param = "filter") DatosTerceros filterDatosTerceros,
			@RequestJsonBody(param = "search") DatosTerceros searchDatosTerceros,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DatosTercerosController.logger.info("[POST - search] : Buscar DatosTerceross");
		return this.datosTercerosService.search(filterDatosTerceros, searchDatosTerceros, jqGridRequestDto, false);
	}

	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterDatosTerceros
	 *            DatosTerceros Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String> Lista de los identificadores de los registros
	 *         eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(
			@RequestJsonBody(param = "filter") DatosTerceros filterDatosTerceros,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DatosTercerosController.logger
				.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples DatosTerceross");
		this.datosTercerosService.removeMultiple(filterDatosTerceros, jqGridRequestDto, false);
		DatosTercerosController.logger.info("All entities correctly deleted!");

		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterDatosTerceros
	 *            DatosTerceros Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<DatosTerceros>> Dto que contiene
	 *         el resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<DatosTerceros>> jerarquia(
			@RequestJsonBody(param = "filter") DatosTerceros filterDatosTerceros,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DatosTercerosController.logger.info("[POST - jerarquia] : Obtener DatosTerceross jerarquia");
		return this.datosTercerosService.jerarquia(filterDatosTerceros, jqGridRequestDto, false);
	}

	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion
	 * jerarquica.
	 * 
	 * @param filterDatosTerceros
	 *            DatosTerceros Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<DatosTerceros>> Dto que contiene
	 *         el resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<DatosTerceros>> jerarquiaChildren(
			@RequestJsonBody(param = "filter") DatosTerceros filterDatosTerceros,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DatosTercerosController.logger.info("[POST - jerarquia] : Obtener DatosTerceross jerarquia - Hijos");
		return this.datosTercerosService.jerarquiaChildren(filterDatosTerceros, jqGridRequestDto);
	}

}
